@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2020-2022 - pancake, thestr4ng3r */
+/* radare - LGPL - Copyright 2020-2023 - pancake, thestr4ng3r */
 
 #include "r2r.h"
 
@@ -14,7 +14,7 @@ static int execvp(const char *a, char **b) {return -1;}
 #define __SIG_IGN 0
 #endif
 
-#if __WINDOWS__
+#if R2__WINDOWS__
 struct r2r_subprocess_t {
 	HANDLE stdin_write;
 	HANDLE stdout_read;
@@ -212,7 +212,7 @@ R_API R2RSubprocess *r2r_subprocess_start(
 			NULL, NULL, TRUE, CREATE_UNICODE_ENVIRONMENT, env,
 			NULL, &start_info, &proc_info)) {
 		free (env);
-		eprintf ("CreateProcess failed: %#x\n", (int)GetLastError ());
+		R_LOG_ERROR ("CreateProcess failed: %#x", (int)GetLastError ());
 		goto error;
 	}
 	free (env);
@@ -303,7 +303,7 @@ R_API bool r2r_subprocess_wait(R2RSubprocess *proc, ut64 timeout_ms) {
 			proc_index = handles.len;
 			r_vector_push (&handles, &proc->proc);
 		}
-		
+
 		DWORD timeout = INFINITE;
 		if (timeout_us_abs != UT64_MAX) {
 			ut64 now = r_time_now_mono ();
@@ -415,7 +415,9 @@ static RThread *sigchld_thread;
 
 static void handle_sigchld(int sig) {
 	ut8 b = 1;
-	(void)write (sigchld_pipe[1], &b, 1);
+	if (write (sigchld_pipe[1], &b, 1) != 1) {
+		return;
+	}
 }
 
 static RThreadFunctionRet sigchld_th(RThread *th) {
@@ -484,7 +486,11 @@ R_API bool r2r_subprocess_init(void) {
 		return false;
 	}
 	sigchld_thread = r_th_new (sigchld_th, NULL, 0);
-	if (!sigchld_thread) {
+	if (!r_th_start (sigchld_thread)) {
+		if (sigchld_thread) {
+			r_th_free (sigchld_thread);
+			sigchld_thread = NULL;
+		}
 		close (sigchld_pipe [0]);
 		close (sigchld_pipe [1]);
 		r_th_lock_free (subprocs_mutex);
@@ -502,7 +508,9 @@ R_API bool r2r_subprocess_init(void) {
 R_API void r2r_subprocess_fini(void) {
 	r_sys_signal (SIGCHLD, SIG_IGN);
 	ut8 b = 0;
-	(void)write (sigchld_pipe[1], &b, 1);
+	if (write (sigchld_pipe[1], &b, 1) != 1) {
+		// nothing relevant here
+	}
 	close (sigchld_pipe [1]);
 	r_th_wait (sigchld_thread);
 	close (sigchld_pipe [0]);
@@ -743,7 +751,9 @@ R_API void r2r_subprocess_kill(R2RSubprocess *proc) {
 }
 
 R_API void r2r_subprocess_stdin_write(R2RSubprocess *proc, const ut8 *buf, size_t buf_size) {
-	(void)write (proc->stdin_fd, buf, buf_size);
+	if (write (proc->stdin_fd, buf, buf_size) != buf_size) {
+		// another ignored result
+	}
 	close (proc->stdin_fd);
 	proc->stdin_fd = -1;
 }
@@ -810,7 +820,7 @@ static R2RProcessOutput *subprocess_runner(const char *file, const char *args[],
 	return out;
 }
 
-#if __WINDOWS__
+#if R2__WINDOWS__
 static char *convert_win_cmds(const char *cmds) {
 	char *r = malloc (strlen (cmds) + 1);
 	if (!r) {
@@ -876,13 +886,22 @@ static char *convert_win_cmds(const char *cmds) {
 }
 #endif
 
-static R2RProcessOutput *run_r2_test(R2RRunConfig *config, ut64 timeout_ms, const char *cmds, RList *files, RList *extra_args, bool load_plugins, R2RCmdRunner runner, void *user) {
+static R2RProcessOutput *run_r2_test(R2RRunConfig *config, ut64 timeout_ms, int repeat, const char *cmds, RList *files, RList *extra_args, RList * extra_env, bool load_plugins, R2RCmdRunner runner, void *user) {
 	RPVector args;
+	RPVector envvars;
+	RPVector envvals;
+	r_pvector_init (&envvars, NULL);
+	r_pvector_init (&envvals, NULL);
 	r_pvector_init (&args, NULL);
+
 	r_pvector_push (&args, "-escr.utf8=0");
+	// r_pvector_push (&args, "-ebin.types=false");
 	r_pvector_push (&args, "-escr.color=0");
 	r_pvector_push (&args, "-escr.interactive=0");
-	r_pvector_push (&args, "-N");
+
+	if (!load_plugins) {
+		r_pvector_push (&args, "-NN");
+	}
 	RListIter *it;
 	void *extra_arg, *file_arg;
 	if (extra_args) {
@@ -891,7 +910,7 @@ static R2RProcessOutput *run_r2_test(R2RRunConfig *config, ut64 timeout_ms, cons
 		}
 	}
 	r_pvector_push (&args, "-Qc");
-#if __WINDOWS__
+#if R2__WINDOWS__
 	char *wcmds = convert_win_cmds (cmds);
 	r_pvector_push (&args, wcmds);
 #else
@@ -901,26 +920,49 @@ static R2RProcessOutput *run_r2_test(R2RRunConfig *config, ut64 timeout_ms, cons
 		r_pvector_push (&args, file_arg);
 	}
 
-	const char *envvars[] = {
-#if __WINDOWS__
-		"ANSICON",
+#if R2__WINDOWS__
+	r_pvector_push (&envvars, "ANSICON");
+	r_pvector_push (&envvals, "1");
 #endif
-		"R2_NOPLUGINS"
-	};
-	const char *envvals[] = {
-#if __WINDOWS__
-		"1",
-#endif
-		"1"
-	};
-#if __WINDOWS__
-	size_t env_size = load_plugins ? 1 : 2;
-#else
-	size_t env_size = load_plugins ? 0 : 1;
-#endif
-	R2RProcessOutput *out = runner (config->r2_cmd, args.v.a, r_pvector_len (&args), envvars, envvals, env_size, timeout_ms, user);
+	if (!load_plugins) {
+		r_pvector_push (&envvars, "R2_NOPLUGINS");
+		r_pvector_push (&envvals, "1");
+	}
+	
+	if (extra_env)
+	{
+		RListIter * eit;
+		char * kv;
+
+		r_list_foreach (extra_env, eit, kv) {
+			char * equal = strstr (kv, "=");
+			if (!equal) {
+				continue;
+			}
+			*equal = 0;
+			r_pvector_push (&envvars, kv);
+			r_pvector_push (&envvals, equal + 1);
+		}
+	}
+
+	size_t env_size = r_pvector_length (&envvars);
+
+	R2RProcessOutput *out;
+	if (repeat > 1) {
+		int rep = repeat;
+		while (rep-- > 0) {
+			out = runner (config->r2_cmd, args.v.a,
+				r_pvector_length (&args), envvars.v.a, envvals.v.a, env_size, timeout_ms, user);
+		}
+	} else {
+		out = runner (config->r2_cmd, args.v.a,
+			r_pvector_length (&args), envvars.v.a, envvals.v.a, env_size, timeout_ms, user);
+	}
+
 	r_pvector_clear (&args);
-#if __WINDOWS__
+	r_pvector_clear (&envvars);
+	r_pvector_clear (&envvals);
+#if R2__WINDOWS__
 	free (wcmds);
 #endif
 	return out;
@@ -931,6 +973,7 @@ R_API R2RProcessOutput *r2r_run_cmd_test(R2RRunConfig *config, R2RCmdTest *test,
 	RList *files = test->file.value? r_str_split_duplist (test->file.value, "\n", true): NULL;
 	RListIter *it;
 	RListIter *tmpit;
+	RList * extra_env = NULL;
 	char *token;
 	if (extra_args) {
 		r_list_foreach_safe (extra_args, it, tmpit, token) {
@@ -956,10 +999,16 @@ R_API R2RProcessOutput *r2r_run_cmd_test(R2RRunConfig *config, R2RCmdTest *test,
 		}
 		r_list_push (files, "-");
 	}
-	ut64 timeout_ms = test->timeout.set? test->timeout.value * 1000: config->timeout_ms;
-	R2RProcessOutput *out = run_r2_test (config, timeout_ms, test->cmds.value, files, extra_args, test->load_plugins, runner, user);
+	if (test->env.value) {
+		extra_env = r_str_split_duplist (test->env.value, ";", true);
+	}
+	int repeat = test->repeat.value;
+	const ut64 timeout_ms = test->timeout.set? test->timeout.value * 1000: config->timeout_ms;
+	R2RProcessOutput *out = run_r2_test (config, timeout_ms, repeat,
+			test->cmds.value, files, extra_args, extra_env, test->load_plugins, runner, user);
 	r_list_free (extra_args);
 	r_list_free (files);
+	r_list_free (extra_env);
 	return out;
 }
 
@@ -989,11 +1038,11 @@ R_API bool r2r_check_cmd_test(R2RProcessOutput *out, R2RCmdTest *test) {
 #define JQ_CMD "jq"
 
 R_API bool r2r_check_jq_available(void) {
-	const char *args[] = {"."};
+	const char *args[] = { "." };
 	const char *invalid_json = "this is not json lol";
 	R2RSubprocess *proc = r2r_subprocess_start (JQ_CMD, args, 1, NULL, NULL, 0);
 	if (!proc) {
-		eprintf ("Cnnot start subprocess\n");
+		R_LOG_ERROR ("Cannot start subprocess");
 		return false;
 	}
 	r2r_subprocess_stdin_write (proc, (const ut8 *)invalid_json, strlen (invalid_json));
@@ -1023,7 +1072,8 @@ R_API bool r2r_check_jq_available(void) {
 R_API R2RProcessOutput *r2r_run_json_test(R2RRunConfig *config, R2RJsonTest *test, R2RCmdRunner runner, void *user) {
 	RList *files = r_list_new ();
 	r_list_push (files, (void *)config->json_test_file);
-	R2RProcessOutput *ret = run_r2_test (config, config->timeout_ms, test->cmd, files, NULL, test->load_plugins, runner, user);
+	// TODO: config->timeout_ms is already inside config, no need to pass it twice! chk other calls
+	R2RProcessOutput *ret = run_r2_test (config, config->timeout_ms, 1, test->cmd, files, NULL, NULL, test->load_plugins, runner, user);
 	r_list_free (files);
 	return ret;
 }
@@ -1032,7 +1082,7 @@ R_API bool r2r_check_json_test(R2RProcessOutput *out, R2RJsonTest *test) {
 	if (!out || out->ret != 0 || !out->out || !out->err || out->timeout) {
 		return false;
 	}
-	const char *args[] = {"."};
+	const char *args[] = { "." };
 	R2RSubprocess *proc = r2r_subprocess_start (JQ_CMD, args, 1, NULL, NULL, 0);
 	r2r_subprocess_stdin_write (proc, (const ut8 *)out->out, strlen (out->out));
 	r2r_subprocess_wait (proc, UT64_MAX);
@@ -1082,7 +1132,7 @@ R_API R2RAsmTestOutput *r2r_run_asm_test(R2RRunConfig *config, R2RAsmTest *test)
 	r_strbuf_init (&cmd_buf);
 	if (test->mode & R2R_ASM_TEST_MODE_ASSEMBLE) {
 		r_pvector_push (&args, test->disasm);
-		R2RSubprocess *proc = r2r_subprocess_start (config->rasm2_cmd, args.v.a, r_pvector_len (&args), NULL, NULL, 0);
+		R2RSubprocess *proc = r2r_subprocess_start (config->rasm2_cmd, args.v.a, r_pvector_length (&args), NULL, NULL, 0);
 		if (!r2r_subprocess_wait (proc, config->timeout_ms)) {
 			r2r_subprocess_kill (proc);
 			out->as_timeout = true;
@@ -1120,7 +1170,7 @@ rip:
 		}
 		r_pvector_push (&args, "-d");
 		r_pvector_push (&args, hex);
-		R2RSubprocess *proc = r2r_subprocess_start (config->rasm2_cmd, args.v.a, r_pvector_len (&args), NULL, NULL, 0);
+		R2RSubprocess *proc = r2r_subprocess_start (config->rasm2_cmd, args.v.a, r_pvector_length (&args), NULL, NULL, 0);
 		if (!r2r_subprocess_wait (proc, config->timeout_ms)) {
 			r2r_subprocess_kill (proc);
 			out->disas_timeout = true;
@@ -1183,7 +1233,7 @@ R_API R2RProcessOutput *r2r_run_fuzz_test(R2RRunConfig *config, R2RFuzzTest *tes
 	const char *cmd = "aaa";
 	RList *files = r_list_new ();
 	r_list_push (files, test->file);
-	R2RProcessOutput *ret = run_r2_test (config, config->timeout_ms, cmd, files, NULL, false, runner, user);
+	R2RProcessOutput *ret = run_r2_test (config, config->timeout_ms, 1, cmd, files, NULL, NULL, false, runner, user);
 	r_list_free (files);
 	return ret;
 }
@@ -1234,6 +1284,46 @@ static bool check_cmd_asan_result(R2RProcessOutput *out) {
 }
 #endif
 
+static bool require_check(const char *require) {
+	if (R_STR_ISEMPTY (require)) {
+		return true;
+	}
+	bool res = true;
+	if (strstr (require, "gas")) {
+		char *as_bin = r_file_path ("as");
+		res &= (bool)as_bin;
+		free (as_bin);
+	}
+	if (strstr (require, "unix")) {
+#if R2__UNIX__
+		res &= true;
+#else
+		res = false;
+#endif
+	}
+	if (strstr (require, "windows")) {
+#if R2__WINDOWS__
+		res &= true;
+#else
+		res = false;
+#endif
+	}
+	if (strstr (require, "linux")) {
+#if __linux__
+		res &= true;
+#else
+		res = false;
+#endif
+	}
+	if (strstr (require, "x86")) {
+#if __i386__ || __x86_64__
+		res &= true;
+#else
+		res &= false;
+#endif
+	}
+	return res;
+}
 R_API R2RTestResultInfo *r2r_run_test(R2RRunConfig *config, R2RTest *test) {
 	R2RTestResultInfo *ret = R_NEW0 (R2RTestResultInfo);
 	if (!ret) {
@@ -1249,6 +1339,13 @@ R_API R2RTestResultInfo *r2r_run_test(R2RRunConfig *config, R2RTest *test) {
 			ret->run_failed = false;
 		} else {
 			R2RCmdTest *cmd_test = test->cmd_test;
+			const char *require = cmd_test->require.value;
+			if (!require_check (require)) {
+				R_LOG_WARN ("Skipping because of %s", require);
+				success = true;
+				ret->run_failed = false;
+				break;
+			}
 			R2RProcessOutput *out = r2r_run_cmd_test (config, cmd_test, subprocess_runner, NULL);
 			success = r2r_check_cmd_test (out, cmd_test);
 			ret->proc_out = out;
