@@ -1,5 +1,7 @@
 /* radare2 - LGPL - Copyright 2017-2022 - pancake, cgvwzq, Dennis Goodlett */
 
+#define R_LOG_ORIGIN "bin.wasm"
+
 #include <r_lib.h>
 #include <r_bin.h>
 #include "wasm.h"
@@ -82,7 +84,7 @@ static bool consume_str_r(RBuffer *b, ut64 bound, size_t len, char *out) {
 	return false;
 }
 
-static bool inline consume_str_new(RBuffer *b, ut64 bound, ut32 *len_out, char **str_out) {
+static inline bool consume_str_new(RBuffer *b, ut64 bound, ut32 *len_out, char **str_out) {
 	r_return_val_if_fail (str_out, false);
 	*str_out = NULL;
 	if (len_out) {
@@ -92,6 +94,10 @@ static bool inline consume_str_new(RBuffer *b, ut64 bound, ut32 *len_out, char *
 	ut32 len = 0;
 	// module_str
 	if (consume_u32_r (b, bound, &len)) {
+		if (len > 0xffff) {
+			// avoid large allocations can be caused by fuzzed bins
+			return false;
+		}
 		char *str = (char *)malloc (len + 1);
 		if (str && consume_str_r (b, bound, len, str)) {
 			if (len_out) {
@@ -184,7 +190,7 @@ static size_t consume_init_expr_r(RBuffer *b, ut64 bound, ut8 eoc, void *out) {
 static size_t consume_locals_r(RBuffer *b, ut64 bound, RBinWasmCodeEntry *out) {
 	r_return_val_if_fail (out, 0);
 	ut32 count = out->local_count;
-	if (count <= 0) {
+	if ((st32)count < 1 || count > ST16_MAX) {
 		return 0;
 	}
 	out->locals = R_NEWS0 (struct r_bin_wasm_local_entry_t, count);
@@ -257,7 +263,7 @@ static inline RBinWasmSection *section_first_with_id(RList *sections, ut8 id) {
 	return NULL;
 }
 
-const char *r_bin_wasm_valuetype_to_string (r_bin_wasm_value_type_t type) {
+const char *r_bin_wasm_valuetype_tostring (r_bin_wasm_value_type_t type) {
 	switch (type) {
 	case R_BIN_WASM_VALUETYPE_i32:
 		return "i32";
@@ -285,7 +291,7 @@ static inline bool strbuf_append_type_vec(RStrBuf *sb, RBinWasmTypeVec *vec) {
 		if (i > 0 && !r_strbuf_append (sb, ", ")) {
 			return false;
 		}
-		const char *s = r_bin_wasm_valuetype_to_string (vec->types[i]);
+		const char *s = r_bin_wasm_valuetype_tostring (vec->types[i]);
 		if (!s || !r_strbuf_append (sb, s)) {
 			return false;
 		}
@@ -302,14 +308,14 @@ static bool append_rets(RStrBuf *sb, RBinWasmTypeVec *rets) {
 	if (!rets->count) {
 		ret &= r_strbuf_append (sb, "nil");
 	} else if (rets->count == 1) {
-		ret &= r_strbuf_append (sb, r_bin_wasm_valuetype_to_string (rets->types[0]));
+		ret &= r_strbuf_append (sb, r_bin_wasm_valuetype_tostring (rets->types[0]));
 	} else {
 		ret &= strbuf_append_type_vec (sb, rets);
 	}
 	return ret;
 }
 
-static const char *r_bin_wasm_type_entry_to_string(RBinWasmTypeEntry *type) {
+static const char *r_bin_wasm_type_entry_tostring(RBinWasmTypeEntry *type) {
 	r_return_val_if_fail (type, NULL);
 	if (type->to_str) {
 		return type->to_str;
@@ -421,16 +427,24 @@ static inline RPVector *parse_vec(RBinWasmObj *bin, ut64 bound, ParseEntryFcn pa
 	if (!consume_u32_r (buf, bound, &count)) {
 		return NULL;
 	}
+	if (count > r_buf_size (buf)) {
+		count = r_buf_size (buf) - r_buf_tell (buf);
+	}
+	if ((st32)count < 1) {
+		return NULL;
+	}
 
 	RPVector *vec = r_pvector_new (free_entry);
 	if (vec) {
-		r_pvector_reserve (vec, count);
+		if (!r_pvector_reserve (vec, count)) {
+			return NULL;
+		}
 		ut32 i;
 		for (i = 0; i < count; i++) {
 			ut64 start = r_buf_tell (buf);
 			void *e = parse_entry (bin, bound, i);
 			if (!e || !r_pvector_push (vec, e)) {
-				eprintf ("[wasm] Failed to parse entry %u/%u of vec at 0x%" PFMT64x "\n", i, count, start);
+				R_LOG_ERROR ("Failed to parse entry %u/%u of vec at 0x%" PFMT64x, i, count, start);
 				free_entry (e);
 				break;
 			}
@@ -474,7 +488,7 @@ static RBinWasmTypeEntry *parse_type_entry(RBinWasmObj *bin, ut64 bound, ut32 in
 	if (!type->rets) {
 		goto beach;
 	}
-	r_bin_wasm_type_entry_to_string (type);
+	r_bin_wasm_type_entry_tostring (type);
 
 	return type;
 
@@ -602,7 +616,7 @@ static RBinWasmCodeEntry *parse_code_entry(RBinWasmObj *bin, ut64 bound, ut32 in
 	r_buf_read (b, &end, 1);
 	if (end != R_BIN_WASM_END_OF_CODE) {
 		ut32 where = r_buf_tell (b) - 1;
-		eprintf ("[wasm] Code entry at starting at 0x%x has ending byte 0x%x at 0x%x, should be 0x%x\n",
+		R_LOG_WARN ("Wasm code entry at starting at 0x%x has ending byte 0x%x at 0x%x, should be 0x%x",
 			(ut32)ptr->file_offset, end, where, R_BIN_WASM_END_OF_CODE);
 		goto beach;
 	}
@@ -699,7 +713,7 @@ static inline bool parse_custom_name_section(RBinWasmObj *bin, ut64 bound) {
 
 	ut64 new_bound = start + size - 1;
 	if (new_bound > bound) {
-		eprintf ("[wasm] custom name subection at 0x%" PFMT64x " extends beyond the custom section\n", start);
+		R_LOG_WARN ("custom name subection at 0x%" PFMT64x " extends beyond the custom section", start);
 		new_bound = bound;
 	}
 
@@ -730,7 +744,7 @@ static inline bool parse_custom_name_section(RBinWasmObj *bin, ut64 bound) {
 		}
 		break;
 	default:
-		eprintf ("[wasm] Unkown custom name subsection with id: %d\n", type);
+		R_LOG_WARN ("Unknown custom name subsection with id: %d", type);
 		break;
 	}
 
@@ -850,7 +864,7 @@ static inline bool r_bin_wasm_get_custom_name_entries(RBinWasmObj *bin, RBinWasm
 	if (!bin->names) {
 		bin->names = R_NEW0 (RBinWasmCustomNames);
 		if (!bin->names) {
-			return NULL;
+			return false;
 		}
 	}
 
@@ -860,7 +874,7 @@ static inline bool r_bin_wasm_get_custom_name_entries(RBinWasmObj *bin, RBinWasm
 		}
 	}
 
-	return bin->names;
+	return bin->names != NULL;
 }
 
 static bool parse_import_sec(RBinWasmObj *bin) {
@@ -894,10 +908,16 @@ static bool parse_import_sec(RBinWasmObj *bin) {
 	if (!consume_u32_r (buf, bound, &count)) {
 		return false;
 	}
+	if (count > 0xfffff) {
+		return false;
+	}
 
 	// over estimate size, shrink later
 	for (i = 0; i < R_ARRAY_SIZE (bin->g_imports_arr); i++) {
-		r_pvector_reserve (bin->g_imports_arr[i], count);
+		if (!r_pvector_reserve (bin->g_imports_arr[i], count)) {
+			R_LOG_ERROR ("Unable to allocate %d in import array", count);
+			return false;
+		}
 	}
 
 	for (i = 0; i < count; i++) {
@@ -906,7 +926,7 @@ static bool parse_import_sec(RBinWasmObj *bin) {
 		if (imp && imp->kind < R_ARRAY_SIZE (bin->g_imports_arr)) {
 			r_pvector_push (bin->g_imports_arr[imp->kind], imp);
 		} else {
-			eprintf ("[wasm] Failed to parse import entry %u/%u of vec at 0x%" PFMT64x "\n", i, count, start);
+			R_LOG_ERROR ("Failed to parse import entry %u/%u of vec at 0x%" PFMT64x, i, count, start);
 			free_import_entry (imp);
 			break;
 		}
@@ -915,7 +935,7 @@ static bool parse_import_sec(RBinWasmObj *bin) {
 	ut32 seen = 0;
 	for (i = 0; i < R_ARRAY_SIZE (bin->g_imports_arr); i++) {
 		r_pvector_shrink (bin->g_imports_arr[i]);
-		seen += r_pvector_len (bin->g_imports_arr[i]);
+		seen += r_pvector_length (bin->g_imports_arr[i]);
 	}
 	return seen == count? true: false;
 }
@@ -969,9 +989,9 @@ void wasm_obj_free(RBinWasmObj *bin) {
 }
 
 void r_bin_wasm_destroy(RBinFile *bf) {
-	if (bf && bf->o) {
-		wasm_obj_free (bf->o->bin_obj);
-		bf->o->bin_obj = NULL;
+	if (bf && bf->bo) {
+		wasm_obj_free (bf->bo->bin_obj);
+		bf->bo->bin_obj = NULL;
 	}
 }
 
@@ -1071,14 +1091,14 @@ RList *r_bin_wasm_get_sections(RBinWasmObj *bin) {
 			ptr->name_len = 4;
 			break;
 		default:
-			eprintf ("[wasm] error: unkown section id: %d\n", ptr->id);
+			R_LOG_ERROR ("unknown section id: %d", ptr->id);
 			r_buf_seek (b, ptr->size - 1, R_BUF_CUR);
 			continue;
 		}
 		if (ptr->offset + (ut64)ptr->size - 1 > bound) {
 			// TODO: Better error handling here
 			ut32 diff = ptr->size - (bound + 1 - ptr->offset);
-			eprintf ("[wasm] Artificially reducing size of section %s by 0x%x bytes so it fits in the file\n", ptr->name, diff);
+			R_LOG_INFO ("Artificially reducing size of section %s by 0x%x bytes so it fits in the file", ptr->name, diff);
 			ptr->size -= diff;
 		}
 		ptr->payload_data = r_buf_tell (b);
@@ -1096,7 +1116,7 @@ RList *r_bin_wasm_get_sections(RBinWasmObj *bin) {
 	bin->g_sections = ret;
 	return ret;
 beach:
-	eprintf ("[wasm] error: beach sections\n");
+	R_LOG_ERROR ("beach sections");
 	free (ptr);
 	return ret;
 }
@@ -1117,7 +1137,21 @@ ut32 r_bin_wasm_get_entrypoint(RBinWasmObj *bin) {
 	return 0;
 }
 
+static int _export_sorter(const void *_a, const void *_b) {
+	const RBinWasmExportEntry *a = _a;
+	const RBinWasmExportEntry *b = _b;
+	st64 diff = (st64)a->kind - b->kind;
+	if (!diff) {
+		diff = (st64)a->index - b->index;
+		if (!diff) { // index collision shouldn't happen
+			diff = (st64)a->sec_i - b->sec_i;
+		}
+	}
+	return diff > 0? 1: -1;
+}
+
 static RPVector *parse_sub_section_vec(RBinWasmObj *bin, RBinWasmSection *sec) {
+	RPVectorComparator sorter = NULL;
 	RPVector **cache = NULL;
 	RPVectorFree pfree = (RPVectorFree)free;
 	ParseEntryFcn parser;
@@ -1147,6 +1181,7 @@ static RPVector *parse_sub_section_vec(RBinWasmObj *bin, RBinWasmSection *sec) {
 		parser = (ParseEntryFcn)parse_export_entry;
 		pfree = (RPVectorFree)free_export_entry;
 		cache = &bin->g_exports;
+		sorter = (RPVectorComparator)_export_sorter;
 		break;
 	case R_BIN_WASM_SECTION_ELEMENT:
 		parser = (ParseEntryFcn)parse_element_entry;
@@ -1180,6 +1215,9 @@ static RPVector *parse_sub_section_vec(RBinWasmObj *bin, RBinWasmSection *sec) {
 	}
 
 	*cache = parse_vec (bin, bound, parser, pfree);
+	if (sorter) {
+		r_pvector_sort (*cache, sorter);
+	}
 	return *cache;
 }
 
