@@ -1,13 +1,10 @@
-#include <stdio.h>
-#include <stdlib.h>
+/* radare - LGPL - Copyright 2019-2023 - mrmacete, pancake, Siguza */
+
 #include <r_util.h>
+#include <r_util/r_xml.h>
 #include <r_list.h>
-
-#include "yxml.h"
 #include "r_cf_dict.h"
 
-#define XMLBUFSIZE 4096
-
 typedef enum {
 	R_CF_STATE_ROOT,
 	R_CF_STATE_IN_DICT,
@@ -15,10 +12,20 @@ typedef enum {
 	R_CF_STATE_IN_KEY,
 	R_CF_STATE_IN_SCALAR,
 	R_CF_STATE_IN_IGNORE,
+	R_CF_STATE_IN_ATTR_ID,
+	R_CF_STATE_IN_ATTR_IDREF,
 } RCFParsePhase;
 
+typedef enum {
+	R_CF_ID_STATE_NONE,
+	R_CF_ID_STATE_SET,
+	R_CF_ID_STATE_REF,
+} RCFIDState;
+
 typedef struct _RCFParseState {
 	RCFParsePhase phase;
+	RCFIDState idstate;
+	ut32 id;
 	char *key;
 	RCFValueType value_type;
 	RCFValueDict *dict;
@@ -31,11 +38,11 @@ static void r_cf_parse_state_free(RCFParseState *state);
 static RCFKeyValue *r_cf_key_value_new(char *key, RCFValue *value);
 static void r_cf_key_value_free(RCFKeyValue *key_value);
 
-static RCFValueDict *r_cf_value_dict_new();
+static RCFValueDict *r_cf_value_dict_new(void);
 static void r_cf_value_dict_add(RCFValueDict *dict, RCFKeyValue *key_value);
 static void r_cf_value_dict_print(RCFValueDict *dict);
 
-static RCFValueArray *r_cf_value_array_new();
+static RCFValueArray *r_cf_value_array_new(void);
 static void r_cf_value_array_free(RCFValueArray *array);
 static void r_cf_value_array_add(RCFValueArray *array, RCFValue *value);
 static void r_cf_value_array_print(RCFValueArray *dict);
@@ -52,7 +59,7 @@ static RCFValueData *r_cf_value_data_new(char *string);
 static void r_cf_value_data_free(RCFValueData *data);
 static void r_cf_value_data_print(RCFValueData *data);
 
-static RCFValueNULL *r_cf_value_null_new();
+static RCFValueNULL *r_cf_value_null_new(void);
 static void r_cf_value_null_free(RCFValueNULL *null);
 static void r_cf_value_null_print(RCFValueNULL *null);
 
@@ -60,26 +67,29 @@ static RCFValueBool *r_cf_value_bool_new(bool value);
 static void r_cf_value_bool_free(RCFValueBool *bool_value);
 static void r_cf_value_bool_print(RCFValueBool *bool_value);
 
+static RCFValue *r_cf_value_clone(RCFValue *value);
 static void r_cf_value_free(RCFValue *value);
 
 RCFValueDict *r_cf_value_dict_parse (RBuffer *file_buf, ut64 offset, ut64 size, int options) {
+	RList *idlist = NULL;
 	RCFValueDict *result = NULL;
-	yxml_t x;
-	int i, depth = 0;
+	int i;
 	char *content = NULL;
 
-	void *xml_buf = malloc (XMLBUFSIZE);
-	if (!xml_buf) {
-		return NULL;
-	}
-
-	yxml_init (&x, xml_buf, XMLBUFSIZE);
+	RXml *x = r_xml_new (4096);
 
 	RList *stack = r_list_newf ((RListFree)&r_cf_parse_state_free);
 	if (!stack) {
 		goto beach;
 	}
 
+	if (options & R_CF_OPTION_SUPPORT_IDREF) {
+		idlist = r_list_new ();
+		if (!idlist) {
+			goto beach;
+		}
+	}
+
 	r_list_push (stack, r_cf_parse_state_new (R_CF_STATE_ROOT));
 
 	for (i = 0; i < size; i++) {
@@ -89,49 +99,48 @@ RCFValueDict *r_cf_value_dict_parse (RBuffer *file_buf, ut64 offset, ut64 size,
 			break;
 		}
 
-		yxml_ret_t r = yxml_parse (&x, doc);
+		RXmlRet r = r_xml_parse (x, doc);
 		if (r < 0) {
-			eprintf ("Parsing error at :%" PRIu32 ":%" PRIu64 " byte offset %" PRIu64 "\n",
-				x.line, x.byte, x.total);
+			R_LOG_ERROR ("Parse at :%08" PFMT64d ":%" PFMT64d " byte offset %" PFMT64d, x->line, x->byte, x->total);
 			goto beach;
 		}
 
 		switch (r) {
-		case YXML_ELEMSTART: {
-			RCFParseState *state = (RCFParseState *)r_list_get_top (stack);
+		case R_XML_ELEMSTART: {
+			RCFParseState *state = (RCFParseState *)r_list_last (stack);
 			RCFParseState *next_state = NULL;
 
-			if (!strcmp (x.elem, "dict")) {
+			if (!strcmp (x->elem, "dict")) {
 				next_state = r_cf_parse_state_new (R_CF_STATE_IN_DICT);
 				if (!next_state) {
 					goto beach;
 				}
 				next_state->dict = r_cf_value_dict_new ();
-			} else if (!strcmp (x.elem, "array")) {
+			} else if (!strcmp (x->elem, "array")) {
 				next_state = r_cf_parse_state_new (R_CF_STATE_IN_ARRAY);
 				if (!next_state) {
 					goto beach;
 				}
 				next_state->array = r_cf_value_array_new ();
-			} else if (!strcmp (x.elem, "key") && state->phase == R_CF_STATE_IN_DICT) {
+			} else if (!strcmp (x->elem, "key") && state->phase == R_CF_STATE_IN_DICT) {
 				next_state = r_cf_parse_state_new (R_CF_STATE_IN_KEY);
 				if (!next_state) {
 					goto beach;
 				}
 				next_state->dict = state->dict;
-			} else if (!strcmp (x.elem, "string")) {
+			} else if (!strcmp (x->elem, "string")) {
 				next_state = r_cf_parse_state_new (R_CF_STATE_IN_SCALAR);
 				if (!next_state) {
 					goto beach;
 				}
 				next_state->value_type = R_CF_STRING;
-			} else if (!strcmp (x.elem, "integer")) {
+			} else if (!strcmp (x->elem, "integer")) {
 				next_state = r_cf_parse_state_new (R_CF_STATE_IN_SCALAR);
 				if (!next_state) {
 					goto beach;
 				}
 				next_state->value_type = R_CF_INTEGER;
-			} else if (!strcmp (x.elem, "data")) {
+			} else if (!strcmp (x->elem, "data")) {
 				if (options & R_CF_OPTION_SKIP_NSDATA) {
 					next_state = r_cf_parse_state_new (R_CF_STATE_IN_IGNORE);
 				} else {
@@ -141,13 +150,13 @@ RCFValueDict *r_cf_value_dict_parse (RBuffer *file_buf, ut64 offset, ut64 size,
 					}
 					next_state->value_type = R_CF_DATA;
 				}
-			} else if (!strcmp (x.elem, "true")) {
+			} else if (!strcmp (x->elem, "true")) {
 				next_state = r_cf_parse_state_new (R_CF_STATE_IN_SCALAR);
 				if (!next_state) {
 					goto beach;
 				}
 				next_state->value_type = R_CF_TRUE;
-			} else if (!strcmp (x.elem, "false")) {
+			} else if (!strcmp (x->elem, "false")) {
 				next_state = r_cf_parse_state_new (R_CF_STATE_IN_SCALAR);
 				if (!next_state) {
 					goto beach;
@@ -158,16 +167,15 @@ RCFValueDict *r_cf_value_dict_parse (RBuffer *file_buf, ut64 offset, ut64 size,
 			if (next_state) {
 				r_list_push (stack, next_state);
 			} else {
-				eprintf ("Missing next state for elem: %s phase: %d\n", x.elem, state->phase);
+				R_LOG_ERROR ("Missing next state for elem: %s phase: %d", x->elem, state->phase);
 				break;
 			}
-			depth++;
 
 			break;
 		}
-		case YXML_ELEMEND: {
+		case R_XML_ELEMEND: {
 			RCFParseState *state = (RCFParseState *)r_list_pop (stack);
-			RCFParseState *next_state = (RCFParseState *)r_list_get_top (stack);
+			RCFParseState *next_state = (RCFParseState *)r_list_last (stack);
 			if (!state || !next_state) {
 				goto beach;
 			}
@@ -178,14 +186,14 @@ RCFValueDict *r_cf_value_dict_parse (RBuffer *file_buf, ut64 offset, ut64 size,
 					r_cf_parse_state_free (state);
 					break;
 				} else {
-					eprintf ("Root element is not a dict\n");
+					R_LOG_ERROR ("Root element is not a dict");
 					goto beach;
 				}
 			}
 
 			if (next_state->phase == R_CF_STATE_IN_DICT && state->phase == R_CF_STATE_IN_KEY) {
 				if (!content) {
-					eprintf ("NULL key not supported");
+					R_LOG_ERROR ("NULL key not supported");
 					goto beach;
 				}
 				next_state->key = content;
@@ -194,42 +202,80 @@ RCFValueDict *r_cf_value_dict_parse (RBuffer *file_buf, ut64 offset, ut64 size,
 			if (state->phase != R_CF_STATE_IN_KEY) {
 				RCFValue *value = NULL;
 
-				switch (state->phase) {
-				case R_CF_STATE_IN_DICT:
-					value = (RCFValue *)state->dict;
-					break;
-				case R_CF_STATE_IN_ARRAY:
-					value = (RCFValue *)state->array;
-					break;
-				case R_CF_STATE_IN_SCALAR:
-					if (!content && state->value_type != R_CF_FALSE && state->value_type != R_CF_TRUE) {
-						value = (RCFValue *)r_cf_value_null_new ();
-					} else {
-						switch (state->value_type) {
-						case R_CF_STRING:
-							value = (RCFValue *)r_cf_value_string_new (content);
-							break;
-						case R_CF_INTEGER:
-							value = (RCFValue *)r_cf_value_integer_new (content);
-							R_FREE (content);
-							break;
-						case R_CF_DATA:
-							value = (RCFValue *)r_cf_value_data_new (content);
-							R_FREE (content);
-							break;
-						case R_CF_TRUE:
-							value = (RCFValue *)r_cf_value_bool_new (true);
-							break;
-						case R_CF_FALSE:
-							value = (RCFValue *)r_cf_value_bool_new (false);
-							break;
-						default:
-							break;
+				if (idlist && state->idstate == R_CF_ID_STATE_REF) {
+					value = r_list_get_n (idlist, (int)state->id);
+					if (!value) {
+						R_LOG_ERROR ("Missing value for IDREF %"PFMT64d, state->id);
+						goto beach;
+					}
+					if (state->phase == R_CF_STATE_IN_DICT) {
+						if (r_list_length (state->dict->pairs) != 0) {
+							R_LOG_ERROR ("Dict with IDREF already has elements");
+							goto beach;
+						}
+						r_cf_value_dict_free (state->dict);
+						state->dict = NULL;
+					} else if (state->phase == R_CF_STATE_IN_ARRAY) {
+						if (r_list_length (state->dict->pairs) != 0) {
+							R_LOG_ERROR ("Array with IDREF already has elements");
+							goto beach;
+						}
+						r_cf_value_array_free (state->array);
+						state->array = NULL;
+					} else if (state->phase == R_CF_STATE_IN_SCALAR && content) {
+						R_LOG_ERROR ("Element with IDREF already has content");
+						goto beach;
+					}
+					value = r_cf_value_clone (value);
+					if (!value) {
+						goto beach;
+					}
+				} else {
+					switch (state->phase) {
+					case R_CF_STATE_IN_DICT:
+						value = (RCFValue *)state->dict;
+						break;
+					case R_CF_STATE_IN_ARRAY:
+						value = (RCFValue *)state->array;
+						break;
+					case R_CF_STATE_IN_SCALAR:
+						if (!content && state->value_type != R_CF_FALSE && state->value_type != R_CF_TRUE) {
+							value = (RCFValue *)r_cf_value_null_new ();
+						} else {
+							switch (state->value_type) {
+							case R_CF_STRING:
+								value = (RCFValue *)r_cf_value_string_new (content);
+								break;
+							case R_CF_INTEGER:
+								value = (RCFValue *)r_cf_value_integer_new (content);
+								R_FREE (content);
+								break;
+							case R_CF_DATA:
+								value = (RCFValue *)r_cf_value_data_new (content);
+								R_FREE (content);
+								break;
+							case R_CF_TRUE:
+								value = (RCFValue *)r_cf_value_bool_new (true);
+								break;
+							case R_CF_FALSE:
+								value = (RCFValue *)r_cf_value_bool_new (false);
+								break;
+							default:
+								break;
+							}
+						}
+						break;
+					default:
+						break;
+					}
+
+					if (idlist && state->idstate == R_CF_ID_STATE_SET) {
+						if (value) {
+							r_list_insert (idlist, state->id, value);
+						} else {
+							R_LOG_WARN ("Missing value for ID %"PFMT64d, state->id);
 						}
 					}
-					break;
-				default:
-					break;
 				}
 
 				if (next_state->phase == R_CF_STATE_IN_DICT) {
@@ -237,33 +283,81 @@ RCFValueDict *r_cf_value_dict_parse (RBuffer *file_buf, ut64 offset, ut64 size,
 						RCFKeyValue *key_value = r_cf_key_value_new (next_state->key, value);
 						r_cf_value_dict_add (next_state->dict, key_value);
 					} else if (state->phase != R_CF_STATE_IN_IGNORE) {
-						eprintf ("Missing value for key %s\n", next_state->key);
+						R_LOG_ERROR ("Missing value for key %s", next_state->key);
+						r_cf_value_free ((RCFValue *)value);
 						goto beach;
 					}
 				} else if (next_state->phase == R_CF_STATE_IN_ARRAY) {
 					if (value) {
 						r_cf_value_array_add (next_state->array, value);
 					} else if (state->phase != R_CF_STATE_IN_IGNORE) {
-						eprintf ("Missing value for array\n");
+						R_LOG_ERROR ("Missing value for array");
+						r_cf_value_free ((RCFValue *)value);
 						goto beach;
 					}
 				}
 			}
-
-			depth--;
 			content = NULL;
 			r_cf_parse_state_free (state);
 			break;
 		}
-		case YXML_CONTENT: {
-			RCFParseState *state = (RCFParseState *)r_list_get_top (stack);
+		case R_XML_CONTENT: {
+			RCFParseState *state = (RCFParseState *)r_list_last (stack);
 			if (state->phase == R_CF_STATE_IN_IGNORE) {
 				break;
 			}
-			if (!content) {
-				content = r_str_new (x.data);
-			} else {
-				content = r_str_append (content, x.data);
+			content = r_str_append (content, x->data);
+			break;
+		}
+		case R_XML_ATTRSTART: {
+			if (idlist) {
+				RCFParseState *state = (RCFParseState *)r_list_last (stack);
+				if (state->phase != R_CF_STATE_IN_DICT && state->phase != R_CF_STATE_IN_ARRAY && state->phase != R_CF_STATE_IN_SCALAR) {
+					break;
+				}
+				RCFParsePhase next_phase;
+				if (!strcmp (x->attr, "ID")) {
+					next_phase = R_CF_STATE_IN_ATTR_ID;
+				} else if (!strcmp (x->attr, "IDREF")) {
+					next_phase = R_CF_STATE_IN_ATTR_IDREF;
+				} else {
+					break;
+				}
+				if (state->idstate != R_CF_ID_STATE_NONE) {
+					R_LOG_ERROR ("Cannot have ID and IDREF on the same element");
+					goto beach;
+				}
+				RCFParseState *next_state = r_cf_parse_state_new (next_phase);
+				if (!next_state) {
+					goto beach;
+				}
+				r_list_push (stack, next_state);
+			}
+			break;
+		}
+		case R_XML_ATTRVAL: {
+			if (idlist) {
+				RCFParseState *state = (RCFParseState *)r_list_last (stack);
+				if (state->phase != R_CF_STATE_IN_ATTR_ID && state->phase != R_CF_STATE_IN_ATTR_IDREF) {
+					break;
+				}
+				content = r_str_append (content, x->data);
+			}
+			break;
+		}
+		case R_XML_ATTREND: {
+			if (idlist) {
+				RCFParseState *state = (RCFParseState *)r_list_last (stack);
+				if (state->phase != R_CF_STATE_IN_ATTR_ID && state->phase != R_CF_STATE_IN_ATTR_IDREF) {
+					break;
+				}
+				r_list_pop (stack);
+				RCFParseState *next_state = (RCFParseState *)r_list_last (stack);
+				next_state->id = (ut32)r_num_get (NULL, content);
+				next_state->idstate = state->phase == R_CF_STATE_IN_ATTR_ID ? R_CF_ID_STATE_SET : R_CF_ID_STATE_REF;
+				R_FREE (content);
+				content = NULL;
+				r_cf_parse_state_free (state);
 			}
 			break;
 		}
@@ -276,16 +370,15 @@ RCFValueDict *r_cf_value_dict_parse (RBuffer *file_buf, ut64 offset, ut64 size,
 		}
 	}
 
-	yxml_ret_t r = yxml_eof (&x);
+	RXmlRet r = r_xml_eof (x);
 	if (r < 0) {
-		eprintf ("Invalid xml\n");
+		R_LOG_ERROR ("Invalid xml");
 	}
-
 beach:
-	R_FREE (xml_buf);
-	if (stack) {
-		r_list_free (stack);
-	}
+	r_xml_free (x);
+	r_list_free (stack);
+	r_list_free (idlist);
+	free (content);
 
 	return result;
 }
@@ -332,7 +425,7 @@ static void r_cf_key_value_free(RCFKeyValue *key_value) {
 	R_FREE (key_value);
 }
 
-static RCFValueDict *r_cf_value_dict_new() {
+static RCFValueDict *r_cf_value_dict_new(void) {
 	RCFValueDict *dict = R_NEW0 (RCFValueDict);
 	if (!dict) {
 		return NULL;
@@ -359,7 +452,6 @@ static void r_cf_value_dict_add(RCFValueDict *dict, RCFKeyValue *key_value) {
 	if (!dict || !dict->pairs) {
 		return;
 	}
-
 	r_list_push (dict->pairs, key_value);
 }
 
@@ -379,30 +471,23 @@ static void r_cf_value_dict_print(RCFValueDict *dict) {
 	printf ("}");
 }
 
-static RCFValueArray *r_cf_value_array_new() {
+static RCFValueArray *r_cf_value_array_new(void) {
 	RCFValueArray *array = R_NEW0 (RCFValueArray);
-	if (!array) {
-		return NULL;
+	if (array) {
+		array->type = R_CF_ARRAY;
+		array->values = r_list_newf ((RListFree)&r_cf_value_free);
 	}
-
-	array->type = R_CF_ARRAY;
-	array->values = r_list_newf ((RListFree)&r_cf_value_free);
-
 	return array;
 }
 
 static void r_cf_value_array_free(RCFValueArray *array) {
-	if (!array) {
-		return;
+	if (array) {
+		if (array->values) {
+			r_list_free (array->values);
+			array->values = NULL;
+		}
+		free (array);
 	}
-
-	if (array->values) {
-		r_list_free (array->values);
-		array->values = NULL;
-	}
-
-	array->type = R_CF_INVALID;
-	R_FREE (array);
 }
 
 static void r_cf_value_array_add(RCFValueArray *array, RCFValue *value) {
@@ -482,7 +567,7 @@ static void r_cf_value_integer_free(RCFValueInteger *integer) {
 }
 
 static void r_cf_value_integer_print(RCFValueInteger *integer) {
-	printf ("%llu", integer->value);
+	printf ("%"PFMT64u, integer->value);
 }
 
 static RCFValueData *r_cf_value_data_new(char *string) {
@@ -524,7 +609,7 @@ static void r_cf_value_data_print(RCFValueData *data) {
 	printf ("\"...\"");
 }
 
-static RCFValueNULL *r_cf_value_null_new() {
+static RCFValueNULL *r_cf_value_null_new(void) {
 	RCFValueNULL *null = R_NEW0 (RCFValueNULL);
 	if (!null) {
 		return NULL;
@@ -572,6 +657,110 @@ static void r_cf_value_bool_print(RCFValueBool *bool_value) {
 		printf ("false");
 	}
 }
+
+static RCFValue *r_cf_value_clone(RCFValue *value) {
+	if (!value) {
+		return NULL;
+	}
+
+	RCFValue *copy = NULL;
+
+	switch (value->type) {
+	case R_CF_DICT: {
+		RCFValueDict *dict = r_cf_value_dict_new ();
+		if (dict) {
+			copy = (RCFValue *)dict;
+			RListIter *iter;
+			RCFKeyValue *item;
+			r_list_foreach (((RCFValueDict *)value)->pairs, iter, item) {
+				char *key = r_str_new (item->key);
+				if (key) {
+					RCFValue *clone = r_cf_value_clone (item->value);
+					if (clone) {
+						RCFKeyValue *pair = r_cf_key_value_new (key, clone);
+						if (pair) {
+							r_cf_value_dict_add (dict, pair);
+						}
+						r_cf_value_free (clone);
+					}
+					R_FREE (key);
+				}
+				r_cf_value_dict_free (dict);
+				copy = NULL;
+				break;
+			}
+		}
+		break;
+	}
+	case R_CF_ARRAY: {
+		RCFValueArray *array = r_cf_value_array_new ();
+		if (array) {
+			copy = (RCFValue *)array;
+			RListIter *iter;
+			RCFValue *item;
+			r_list_foreach (((RCFValueArray *)value)->values, iter, item) {
+				RCFValue *clone = r_cf_value_clone (item);
+				if (clone) {
+					r_cf_value_array_add (array, clone);
+					continue;
+				}
+				r_cf_value_array_free (array);
+				copy = NULL;
+				break;
+			}
+		}
+		break;
+	}
+	case R_CF_STRING: {
+		RCFValueString *string = R_NEW0 (RCFValueString);
+		if (string) {
+			string->value = r_str_new (((RCFValueString *)value)->value);
+			if (string->value) {
+				copy = (RCFValue *)string;
+			} else {
+				R_FREE (string);
+			}
+		}
+		break;
+	}
+	case R_CF_INTEGER: {
+		RCFValueInteger *integer = R_NEW0 (RCFValueInteger);
+		if (integer) {
+			integer->value = ((RCFValueInteger *)value)->value;
+			copy = (RCFValue *)integer;
+		}
+		break;
+	}
+	case R_CF_DATA: {
+		RCFValueData *data = R_NEW0 (RCFValueData);
+		if (data) {
+			data->value = r_buf_new_with_buf(((RCFValueData *)value)->value);
+			if (data->value) {
+				copy = (RCFValue *)data;
+			} else {
+				R_FREE (data);
+			}
+		}
+		break;
+	}
+	case R_CF_NULL:
+		copy = (RCFValue *)(R_NEW0 (RCFValueNULL));
+		break;
+	case R_CF_TRUE:
+	case R_CF_FALSE:
+		copy = (RCFValue *)(R_NEW0 (RCFValueBool));
+		break;
+	default:
+		break;
+	}
+
+	if (copy) {
+		copy->type = value->type;
+	}
+
+	return copy;
+}
+
 static void r_cf_value_free(RCFValue *value) {
 	if (!value) {
 		return;
