@@ -1,8 +1,10 @@
-/* radare - LGPL - Copyright 2009-2022 - nibble, pancake, alvarofe */
+/* radare - LGPL - Copyright 2009-2024 - nibble, pancake, alvarofe */
 
-#include "bin_pe.inc"
+#include "bin_pe.inc.c"
 
-static bool check_buffer(RBinFile *bf, RBuffer *b) {
+extern struct r_bin_write_t r_bin_write_pe;
+
+static bool check(RBinFile *bf, RBuffer *b) {
 	ut64 length = r_buf_size (b);
 	if (length <= 0x3d) {
 		return false;
@@ -95,14 +97,14 @@ static RBuffer* create(RBin* bin, const ut8 *code, int codelen, const ut8 *data,
 
 	if (data && datalen > 0) {
 		//ut32 data_section = buf->length;
-		eprintf ("Warning: DATA section not support for PE yet\n");
+		R_LOG_WARN ("DATA section not support for PE yet");
 		B (data, datalen);
 	}
 	return buf;
 }
 
 static char *signature(RBinFile *bf, bool json) {
-	r_return_val_if_fail (bf && bf->o && bf->o->bin_obj, NULL);
+	r_return_val_if_fail (bf && bf->bo && bf->bo->bin_obj, NULL);
 	RBinPEObj *pe = PE_(get)(bf);
 	if (json) {
 		PJ *pj = r_pkcs7_cms_json (pe->cms);
@@ -111,7 +113,7 @@ static char *signature(RBinFile *bf, bool json) {
 		}
 		return strdup ("{}");
 	}
-	return r_pkcs7_cms_to_string (pe->cms);
+	return r_pkcs7_cms_tostring (pe->cms);
 }
 
 static RList *fields(RBinFile *bf) {
@@ -120,9 +122,8 @@ static RList *fields(RBinFile *bf) {
 		return NULL;
 	}
 
-	r_strf_buffer (16);
 	#define ROWL(nam,siz,val,fmt) \
-	r_list_append (ret, r_bin_field_new (addr, addr, siz, nam, r_strf ("0x%08x", val), fmt, false));
+	r_list_append (ret, r_bin_field_new (addr, addr, val, siz, nam, NULL, fmt, false));
 
 	RBinPEObj *pe = PE_(get)(bf);
 	ut64 addr = pe->rich_header_offset ? pe->rich_header_offset : 128;
@@ -130,7 +131,7 @@ static RList *fields(RBinFile *bf) {
 	RListIter *it;
 	Pe_image_rich_entry *rich;
 	r_list_foreach (pe->rich_entries, it, rich) {
-		r_list_append (ret, r_bin_field_new (addr, addr, 0, "RICH_ENTRY_NAME", rich->productName, "s", false));
+		r_list_append (ret, r_bin_field_new (addr, addr, 0, 0, "RICH_ENTRY_NAME", rich->productName, "s", false));
 		ROWL ("RICH_ENTRY_ID", 2, rich->productId, "x"); addr += 2;
 		ROWL ("RICH_ENTRY_VERSION", 2, rich->minVersion, "x"); addr += 2;
 		ROWL ("RICH_ENTRY_TIMES", 4, rich->timesUsed, "x"); addr += 4;
@@ -399,18 +400,49 @@ static void header(RBinFile *bf) {
 			cb_printf ("  Size : 0x%x\n", pe->nt_headers->optional_header.DataDirectory[i].Size);
 		}
 	}
+	if (pe->metadata_header) {
+		PE_(image_metadata_header) *mh = pe->metadata_header;
+		cb_printf ("Metadata Header:\n");
+		cb_printf ("  Signature: 0x%08"PFMT64x"\n", mh->Signature);
+		cb_printf ("  Version: %d.%d\n", mh->MajorVersion, mh->MinorVersion);
+		cb_printf ("  VersionString: %s\n", mh->VersionString);
+		cb_printf ("  Flags: 0x%x\n", mh->Flags);
+		cb_printf ("  Streams: %d\n", mh->NumberOfStreams);
+		if (pe->streams) {
+			for (i = 0; i < mh->NumberOfStreams; i++) {
+				PE_(image_metadata_stream) * stream = pe->streams[i];
+				cb_printf ("  Stream %d: %s\n", i, stream->Name);
+				cb_printf ("    offset: 0x%08"PFMT64x" size: 0x%x\n", (ut64)stream->Offset, stream->Size);
+			}
+		}
+	}
+	if (pe->clr_hdr) {
+		PE_(image_clr_header) *clr = pe->clr_hdr;
+		cb_printf ("Common Language Runtime Header (CLR):\n");
+		cb_printf ("  Header Size: %d\n", clr->HeaderSize);
+		cb_printf ("  CLR RuntimeVersion: %d.%d\n", clr->MajorRuntimeVersion, clr->MinorRuntimeVersion);
+		cb_printf ("  MetadataDirectory: 0x%08"PFMT64x" (%d)\n", (ut64) clr->MetaDataDirectoryAddress, clr->MetaDataDirectorySize);
+		cb_printf ("  Flags: 0x%x\n", clr->Flags);
+		cb_printf ("  EntryPointToken: 0x%x\n", clr->EntryPointToken);
+		cb_printf ("  ResourceDirectory: 0x%"PFMT64x" (%d)\n", (ut64)clr->ResourcesDirectoryAddress, clr->ResourcesDirectorySize);
+		cb_printf ("  StrongNameSignature: 0x%"PFMT64x" (%d)\n", (ut64)clr->StrongNameSignatureAddress, clr->StrongNameSignatureSize);
+		cb_printf ("  CodeManagerTable: 0x%"PFMT64x" (%d)\n", (ut64)clr->StrongNameSignatureAddress, clr->StrongNameSignatureSize);
+		cb_printf ("  VTableFixups: 0x%"PFMT64x" (%d)\n", (ut64)clr->VTableFixupsAddress, clr->VTableFixupsSize);
+		cb_printf ("  ExportAddressTableJumps: 0x%"PFMT64x" (%d)\n", (ut64)clr->ExportAddressTableJumpsAddress, clr->ExportAddressTableJumpsSize);
+		cb_printf ("  ManagedNativeHeader: 0x%"PFMT64x" (%d)\n", (ut64)clr->ManagedNativeHeaderAddress, clr->ManagedNativeHeaderSize);
+	}
 }
 
-extern struct r_bin_write_t r_bin_write_pe;
-
 RBinPlugin r_bin_plugin_pe = {
-	.name = "pe",
-	.desc = "PE bin plugin",
-	.license = "LGPL3",
+	.meta = {
+		.name = "pe",
+		.desc = "PE bin plugin",
+		.license = "LGPL3",
+	},
 	.get_sdb = &get_sdb,
-	.load_buffer = &load_buffer,
+	.load = &load,
 	.destroy = &destroy,
-	.check_buffer = &check_buffer,
+	.check = &check,
 	.baddr = &baddr,
 	.binsym = &binsym,
 	.entries = &entries,
