@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2008-2022 - pancake, Jody Frankowski */
+/* radare2 - LGPL - Copyright 2008-2024 - pancake, Jody Frankowski */
 
 #include <r_cons.h>
 #include <r_util.h>
@@ -8,25 +8,43 @@
 
 R_LIB_VERSION (r_cons);
 
-// Stub function that cb_main_output gets pointed to in util/log.c by r_cons_new
-// This allows Iaito to set per-task logging redirection
-static R_TH_LOCAL RThreadLock *lock = NULL;
+static R_TH_LOCAL int oldraw = -1;
 static R_TH_LOCAL RConsContext r_cons_context_default = {{{{0}}}};
 static R_TH_LOCAL RCons g_cons_instance = {0};
 static R_TH_LOCAL RCons *r_cons_instance = NULL;
-static R_TH_LOCAL RThreadLock r_cons_lock = R_THREAD_LOCK_INIT;
+static R_TH_LOCAL ut64 prev = 0LL; //r_time_now_mono ();
+static R_TH_LOCAL RStrBuf *echodata = NULL; // TODO: move into RConsInstance? maybe nope
 #define I (r_cons_instance)
-#define C (getctx())
+#define C (getctx ())
+
+static inline void cons_input_state_init(InputState *state) {
+	state->readbuffer = NULL;
+	state->readbuffer_length = 0;
+	state->bufactive = true;
+}
 
 static RConsContext *getctx(void) {
-	if (!r_cons_instance) {
+	if (R_UNLIKELY (!r_cons_instance)) {
 		r_cons_instance = &g_cons_instance;
 		r_cons_instance->context = &r_cons_context_default;
+		cons_input_state_init (&r_cons_instance->input_state);
 	}
 	return r_cons_instance->context;
 }
 
-//this structure goes into cons_stack when r_cons_push/pop
+R_API InputState *r_cons_input_state(void) {
+	if (R_UNLIKELY (!r_cons_instance)) {
+		r_cons_instance = &g_cons_instance;
+		r_cons_instance->context = &r_cons_context_default;
+		cons_input_state_init (&r_cons_instance->input_state);
+	}
+	return &r_cons_instance->input_state;
+}
+
+R_API bool r_cons_is_initialized(void) {
+	return r_cons_instance != NULL;
+}
+
 typedef struct {
 	char *buf;
 	int buf_len;
@@ -42,7 +60,7 @@ typedef struct {
 
 static void cons_grep_reset(RConsGrep *grep) {
 	if (grep) {
-		R_FREE (grep->str); // double-free
+		R_FREE (grep->str);
 		ZERO_FILL (*grep);
 		grep->line = -1;
 		grep->sort = -1;
@@ -58,11 +76,6 @@ static void break_stack_free(void *ptr) {
 static void cons_stack_free(void *ptr) {
 	RConsStack *s = (RConsStack *)ptr;
 	R_FREE (s->buf);
-/*
-	if (s->grep) {
-		R_FREE (s->grep->str); // FREE
-	}
-*/
 	cons_grep_reset (s->grep);
 	R_FREE (s->grep);
 	free (s);
@@ -116,11 +129,9 @@ static void cons_stack_load(RConsStack *data, bool free_current) {
 }
 
 static void cons_context_init(RConsContext *context, R_NULLABLE RConsContext *parent) {
+	context->marks = r_list_newf ((RListFree)r_cons_mark_free);
 	context->breaked = false;
-	context->cmd_depth = R_CONS_CMD_DEPTH + 1;
-	context->error = r_strbuf_new ("");
-	context->errmode = R_CONS_ERRMODE_ECHO;
-	context->buffer = NULL;
+	// context->cmd_depth = R_CONS_CMD_DEPTH + 1;
 	context->buffer_sz = 0;
 	context->lastEnabled = true;
 	context->buffer_len = 0;
@@ -141,13 +152,12 @@ static void cons_context_init(RConsContext *context, R_NULLABLE RConsContext *pa
 		context->color_mode = COLOR_MODE_DISABLED;
 		r_cons_pal_init (context);
 	}
-
 	cons_grep_reset (&context->grep);
 }
 
 static void cons_context_deinit(RConsContext *context) {
-	R_FREE (context->error);
 	r_stack_free (context->cons_stack);
+	r_list_free (context->marks);
 	context->cons_stack = NULL;
 	r_stack_free (context->break_stack);
 	context->break_stack = NULL;
@@ -159,7 +169,7 @@ static void __break_signal(int sig) {
 }
 
 static inline void __cons_write_ll(const char *buf, int len) {
-#if __WINDOWS__
+#if R2__WINDOWS__
 	if (I->vtmode) {
 		(void) write (I->fdout, buf, len);
 	} else {
@@ -259,7 +269,7 @@ R_API void r_cons_printat(const char *str, int x, char y) {
 	}
 }
 
-R_API void r_cons_strcat_justify(const char *str, int j, char c) {
+R_API void r_cons_print_justify(const char *str, int j, char c) {
 	int i, o, len;
 	for (o = i = len = 0; str[i]; i++, len++) {
 		if (str[i] == '\n') {
@@ -281,7 +291,48 @@ R_API void r_cons_strcat_justify(const char *str, int j, char c) {
 	}
 }
 
-R_API void r_cons_strcat_at(const char *_str, int x, char y, int w, int h) {
+
+#if 0
+// TODO: review and remove
+R_API void r_cons_print_at(char *s, int x, int y, int w, int h) {
+	if (w < 0) {
+		w = 0;
+	}
+	if (h < 0) {
+		h = 0;
+	}
+	while (s) {
+		int pos = 0;
+		int ochar = s[pos];
+		char *n = strchr (s, '\n');
+		if (n) {
+			*n = 0;
+			if (w && r_str_ansi_len (s) > w) {
+				const char *p = r_str_ansi_chrn (s, w);
+				if (p) {
+					pos = p - s;
+					ochar = s[pos];
+					s[pos] = 0;
+				}
+			}
+		}
+		r_cons_gotoxy (x, y);
+		r_cons_printf ("%s", s);
+		if (n) {
+			s[pos] = ochar;
+			*n = '\n';
+			s = n + 1;
+		} else {
+			break;
+		}
+		if (h && y > h) {
+			break;
+		}
+		y++;
+	}
+}
+#endif
+R_API void r_cons_print_at(const char *_str, int x, char y, int w, int h) {
 	int i, o, len;
 	int cols = 0;
 	int rows = 0;
@@ -295,7 +346,7 @@ R_API void r_cons_strcat_at(const char *_str, int x, char y, int w, int h) {
 		}
 	}
 	char *str = r_str_ansi_crop (_str, 0, 0, w + 1, h);
-	r_cons_strcat (R_CONS_CURSOR_SAVE);
+	r_cons_print (R_CONS_CURSOR_SAVE);
 	for (o = i = len = 0; str[i]; i++, len++) {
 		if (w < 0 || rows > w) {
 			break;
@@ -316,8 +367,8 @@ R_API void r_cons_strcat_at(const char *_str, int x, char y, int w, int h) {
 		r_cons_gotoxy (x, y + rows);
 		r_cons_write (str + o, len);
 	}
-	r_cons_strcat (Color_RESET);
-	r_cons_strcat (R_CONS_CURSOR_RESTORE);
+	r_cons_print (Color_RESET);
+	r_cons_print (R_CONS_CURSOR_RESTORE);
 	free (str);
 }
 
@@ -338,17 +389,17 @@ R_API void r_cons_break_clear(void) {
 }
 
 R_API void r_cons_context_break_push(RConsContext *context, RConsBreak cb, void *user, bool sig) {
+#if WANT_DEBUGSTUFF
 	if (!context || !context->break_stack) {
 		return;
 	}
-
-	//if we don't have any element in the stack start the signal
+	// if we don't have any element in the stack start the signal
 	RConsBreakStack *b = R_NEW0 (RConsBreakStack);
 	if (!b) {
 		return;
 	}
 	if (r_stack_is_empty (context->break_stack)) {
-#if __UNIX__
+#if R2__UNIX__
 		if (!C->unbreakable) {
 			if (sig && r_cons_context_is_main ()) {
 				r_sys_signal (SIGINT, __break_signal);
@@ -357,16 +408,18 @@ R_API void r_cons_context_break_push(RConsContext *context, RConsBreak cb, void
 #endif
 		context->breaked = false;
 	}
-	//save the actual state
+	// save the actual state
 	b->event_interrupt = context->event_interrupt;
 	b->event_interrupt_data = context->event_interrupt_data;
 	r_stack_push (context->break_stack, b);
-	//configure break
+	// configure break
 	context->event_interrupt = cb;
 	context->event_interrupt_data = user;
+#endif
 }
 
 R_API void r_cons_context_break_pop(RConsContext *context, bool sig) {
+#if WANT_DEBUGSTUFF
 	if (!context || !context->break_stack) {
 		return;
 	}
@@ -379,7 +432,7 @@ R_API void r_cons_context_break_pop(RConsContext *context, bool sig) {
 		break_stack_free (b);
 	} else {
 		//there is not more elements in the stack
-#if __UNIX__ && !__wasi__
+#if R2__UNIX__ && !__wasi__
 		if (sig && r_cons_context_is_main ()) {
 			if (!C->unbreakable) {
 				r_sys_signal (SIGINT, SIG_IGN);
@@ -389,30 +442,18 @@ R_API void r_cons_context_break_pop(RConsContext *context, bool sig) {
 		C->was_breaked = C->breaked;
 		context->breaked = false;
 	}
+#endif
 }
 
-#if RADARE2_5_7_X
-
-// ABI break
-R_API void r_cons_break_push(void) {
-	r_cons_context_break_push (C, NULL, NULL, true);
-}
-
-R_API void r_cons_break_popa(void) {
-	while (!r_stack_is_empty (C->break_stack)) {
-		r_cons_context_break_pop ();
-	}
-}
-
-#else
-
 R_API void r_cons_break_push(RConsBreak cb, void *user) {
+	if (r_stack_size (C->break_stack) > 0) {
+		r_cons_break_timeout (I->otimeout);
+	}
 	r_cons_context_break_push (C, cb, user, true);
 }
 
-#endif
-
 R_API void r_cons_break_pop(void) {
+	I->timeout = 0;
 	r_cons_context_break_pop (C, true);
 }
 
@@ -425,28 +466,37 @@ R_API bool r_cons_default_context_is_interactive(void) {
 }
 
 R_API bool r_cons_was_breaked(void) {
-	bool res = r_cons_is_breaked () || C->was_breaked;
+#if WANT_DEBUGSTUFF
+	const bool res = r_cons_is_breaked () || C->was_breaked;
 	C->breaked = false;
 	C->was_breaked = false;
 	return res;
+#else
+	return false;
+#endif
 }
 
 R_API bool r_cons_is_breaked(void) {
-	if (I->cb_break) {
+#if WANT_DEBUGSTUFF
+	if (R_UNLIKELY (I->cb_break)) {
 		I->cb_break (I->user);
 	}
 	if (R_UNLIKELY (I->timeout)) {
-		if (r_time_now_mono () > I->timeout) {
-			C->breaked = true;
-			C->was_breaked = true;
-			eprintf ("\nTimeout!\n");
-			I->timeout = 0;
+		if (r_stack_size (C->break_stack) > 0) {
+			if (r_time_now_mono () > I->timeout) {
+				C->breaked = true;
+				C->was_breaked = true;
+				r_cons_break_timeout (I->otimeout);
+			}
 		}
 	}
 	if (R_UNLIKELY (!C->was_breaked)) {
 		C->was_breaked = C->breaked;
 	}
-	return C && C->breaked;
+	return R_UNLIKELY (C && C->breaked);
+#else
+	return false;
+#endif
 }
 
 R_API void r_cons_line(int x, int y, int x2, int y2, int ch) {
@@ -462,14 +512,14 @@ R_API void r_cons_line(int x, int y, int x2, int y2, int ch) {
 
 R_API int r_cons_get_cur_line(void) {
 	int curline = 0;
-#if __WINDOWS__
+#if R2__WINDOWS__
 	CONSOLE_SCREEN_BUFFER_INFO info;
 	if (!GetConsoleScreenBufferInfo (GetStdHandle (STD_OUTPUT_HANDLE), &info)) {
 		return 0;
 	}
 	curline = info.dwCursorPosition.Y - info.srWindow.Top;
 #endif
-#if __UNIX__ && !__wasi__
+#if R2__UNIX__ && !__wasi__
 	char buf[8];
 	struct termios save,raw;
 	// flush the Arrow keys escape keys which was messing up the output
@@ -494,14 +544,23 @@ R_API int r_cons_get_cur_line(void) {
 }
 
 R_API void r_cons_break_timeout(int timeout) {
+	if (timeout > 0) {
+		I->timeout = r_time_now_mono () + (timeout * 1000);
+		I->otimeout = timeout;
+	} else {
+		I->otimeout = 0;
+		I->timeout = 0;
+	}
+#if 0
 	I->timeout = (timeout && !I->timeout)
 		? r_time_now_mono () + ((ut64) timeout << 20) : 0;
+#endif
 }
 
 R_API void r_cons_break_end(void) {
 	C->breaked = false;
 	I->timeout = 0;
-#if __UNIX__ && !__wasi__
+#if R2__UNIX__ && !__wasi__
 	if (!C->unbreakable) {
 		r_sys_signal (SIGINT, SIG_IGN);
 	}
@@ -517,6 +576,7 @@ R_API void r_cons_break_end(void) {
 }
 
 R_API void *r_cons_sleep_begin(void) {
+	R_CRITICAL_ENTER (I);
 	if (!r_cons_instance) {
 		r_cons_thready ();
 	}
@@ -533,9 +593,10 @@ R_API void r_cons_sleep_end(void *user) {
 	if (I->cb_sleep_end) {
 		I->cb_sleep_end (I->user, user);
 	}
+	R_CRITICAL_LEAVE (I);
 }
 
-#if __WINDOWS__
+#if R2__WINDOWS__
 static HANDLE h;
 static BOOL __w32_control(DWORD type) {
 	if (type == CTRL_C_EVENT) {
@@ -545,7 +606,7 @@ static BOOL __w32_control(DWORD type) {
 	}
 	return false;
 }
-#elif __UNIX__
+#elif R2__UNIX__ && !__wasi__
 volatile sig_atomic_t sigwinchFlag;
 static void resize(int sig) {
 	sigwinchFlag = 1;
@@ -584,7 +645,7 @@ R_API bool r_cons_enable_mouse(const bool enable) {
 	if ((I->mouse && enable) || (!I->mouse && !enable)) {
 		return I->mouse;
 	}
-#if __WINDOWS__
+#if R2__WINDOWS__
 	if (I->vtmode == 2) {
 #endif
 		const char *click = enable
@@ -600,7 +661,7 @@ R_API bool r_cons_enable_mouse(const bool enable) {
 		}
 		I->mouse = enable;
 		return enabled;
-#if __WINDOWS__
+#if R2__WINDOWS__
 	}
 	DWORD mode;
 	HANDLE h;
@@ -628,12 +689,12 @@ R_API RCons *r_cons_new(void) {
 	if (I->refcnt != 1) {
 		return I;
 	}
-	if (lock) {
-		r_th_lock_wait (lock);
+	if (I->lock) {
+		r_th_lock_wait (I->lock);
 	} else {
-		lock = r_th_lock_new (false);
+		I->lock = r_th_lock_new (false);
 	}
-	r_th_lock_enter (lock);
+	R_CRITICAL_ENTER (I);
 	I->use_utf8 = r_cons_is_utf8 ();
 	I->rgbstr = r_cons_rgb_str_off;
 	I->line = r_line_new ();
@@ -645,6 +706,7 @@ R_API RCons *r_cons_new(void) {
 	I->teefile = NULL;
 	I->fix_columns = 0;
 	I->fix_rows = 0;
+	RVecFdPairs_init (&I->fds);
 	I->mouse_event = 0;
 	I->force_rows = 0;
 	I->force_columns = 0;
@@ -659,11 +721,12 @@ R_API RCons *r_cons_new(void) {
 
 	r_cons_context_reset ();
 	cons_context_init (C, NULL);
+	cons_input_state_init (&I->input_state);
 
 	r_cons_get_size (&I->pagesize);
 	I->num = NULL;
 	I->null = 0;
-#if __WINDOWS__
+#if R2__WINDOWS__
 	I->old_cp = GetConsoleOutputCP ();
 	I->vtmode = r_cons_is_vtcompat ();
 #else
@@ -671,7 +734,7 @@ R_API RCons *r_cons_new(void) {
 #endif
 #if EMSCRIPTEN || __wasi__
 	/* do nothing here :? */
-#elif __UNIX__
+#elif R2__UNIX__
 	tcgetattr (0, &I->term_buf);
 	memcpy (&I->term_raw, &I->term_buf, sizeof (I->term_raw));
 	I->term_raw.c_iflag &= ~(BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
@@ -680,12 +743,12 @@ R_API RCons *r_cons_new(void) {
 	I->term_raw.c_cflag |= CS8;
 	I->term_raw.c_cc[VMIN] = 1; // Solaris stuff hehe
 	r_sys_signal (SIGWINCH, resize);
-#elif __WINDOWS__
+#elif R2__WINDOWS__
 	h = GetStdHandle (STD_INPUT_HANDLE);
 	GetConsoleMode (h, &I->term_buf);
 	I->term_raw = 0;
 	if (!SetConsoleCtrlHandler ((PHANDLER_ROUTINE)__w32_control, TRUE)) {
-		R_LOG_ERROR ("r_cons: Cannot set control console handler");
+		R_LOG_ERROR ("Cannot set control console handler");
 	}
 #endif
 	I->pager = NULL; /* no pager by default */
@@ -693,15 +756,13 @@ R_API RCons *r_cons_new(void) {
 	I->show_vals = false;
 	r_cons_reset ();
 	r_cons_rgb_init ();
-
 	r_print_set_is_interrupted_cb (r_cons_is_breaked);
-	r_th_lock_leave (lock);
-
+	R_CRITICAL_LEAVE (I);
 	return I;
 }
 
 R_API RCons *r_cons_free(void) {
-#if __WINDOWS__
+#if R2__WINDOWS__
 	r_cons_enable_mouse (false);
 	if (I->old_cp) {
 		(void)SetConsoleOutputCP (I->old_cp);
@@ -723,22 +784,21 @@ R_API RCons *r_cons_free(void) {
 	R_FREE (C->lastOutput);
 	C->lastLength = 0;
 	R_FREE (I->pager);
+	RVecFdPairs_fini (&I->fds);
 	return NULL;
 }
 
 #define MOAR (4096 * 8)
 static bool palloc(int moar) {
-	void *temp;
 	if (moar <= 0) {
 		return false;
 	}
 	if (!C->buffer) {
-		int new_sz;
 		if ((INT_MAX - MOAR) < moar) {
 			return false;
 		}
-		new_sz = moar + MOAR;
-		temp = calloc (1, new_sz);
+		size_t new_sz = moar + MOAR;
+		void *temp = calloc (1, new_sz);
 		if (temp) {
 			C->buffer_sz = new_sz;
 			C->buffer = temp;
@@ -767,7 +827,7 @@ R_API int r_cons_eof(void) {
 }
 
 R_API void r_cons_gotoxy(int x, int y) {
-#if __WINDOWS__
+#if R2__WINDOWS__
 	r_cons_w32_gotoxy (1, x, y);
 #else
 	r_cons_printf ("\x1b[%d;%dH", y, x);
@@ -775,21 +835,20 @@ R_API void r_cons_gotoxy(int x, int y) {
 }
 
 R_API void r_cons_print_clear(void) {
-	r_cons_strcat ("\x1b[0;0H\x1b[0m");
+	r_cons_print ("\x1b[0;0H\x1b[0m");
 }
 
 R_API void r_cons_fill_line(void) {
-	char *p, white[1024];
+	char white[1024];
 	int cols = I->columns - 1;
 	if (cols < 1) {
 		return;
 	}
-	p = (cols >= sizeof (white))
-		?  malloc (cols + 1): white;
+	char *p = (cols >= sizeof (white))?  malloc (cols + 1): white;
 	if (p) {
 		memset (p, ' ', cols);
 		p[cols] = 0;
-		r_cons_strcat (p);
+		r_cons_print (p);
 		if (white != p) {
 			free (p);
 		}
@@ -797,7 +856,7 @@ R_API void r_cons_fill_line(void) {
 }
 
 R_API void r_cons_clear_line(int std_err) {
-#if __WINDOWS__
+#if R2__WINDOWS__
 	if (I->vtmode) {
 		fprintf (std_err? stderr: stdout,"%s", R_CONS_CLEAR_LINE);
 	} else {
@@ -824,15 +883,15 @@ R_API void r_cons_clear00(void) {
 }
 
 R_API void r_cons_reset_colors(void) {
-	r_cons_strcat (Color_RESET_BG Color_RESET);
+	r_cons_print (Color_RESET_BG Color_RESET);
 }
 
 R_API void r_cons_clear(void) {
 	I->lines = 0;
-#if __WINDOWS__
+#if R2__WINDOWS__
 	r_cons_w32_clear ();
 #else
-	r_cons_strcat (Color_RESET R_CONS_CLEAR_SCREEN);
+	r_cons_print (Color_RESET R_CONS_CLEAR_SCREEN);
 #endif
 }
 
@@ -876,8 +935,9 @@ R_API void r_cons_filter(void) {
 		}
 		free (input);
 	}
-	if (C->was_html) {
-		C->is_html = true;
+	if (C->tmp_html) {
+		C->is_html = C->was_html;
+		C->tmp_html = false;
 		C->was_html = false;
 	}
 }
@@ -973,7 +1033,6 @@ static bool lastMatters(void) {
 }
 
 R_API void r_cons_echo(const char *msg) {
-	static R_TH_LOCAL RStrBuf *echodata = NULL; // TODO: move into RConsInstance? maybe nope
 	if (msg) {
 		if (echodata) {
 			r_strbuf_append (echodata, msg);
@@ -984,7 +1043,7 @@ R_API void r_cons_echo(const char *msg) {
 	} else {
 		if (echodata) {
 			char *data = r_strbuf_drain (echodata);
-			r_cons_strcat (data);
+			r_cons_print (data);
 			r_cons_newline ();
 			echodata = NULL;
 			free (data);
@@ -992,21 +1051,13 @@ R_API void r_cons_echo(const char *msg) {
 	}
 }
 
-R_API void r_cons_eflush(void) {
-	char *s = r_cons_errstr ();
-	if (s) {
-		eprintf ("%s", s);
-		free (s);
-	}
-}
-
 // TODO: must be called twice to remove all unnecessary reset codes. maybe adding the last two words would be faster
 // TODO remove all the strdup
 // TODO remove the slow memmove
 static void optimize(void) {
 	char *buf = C->buffer;
 	int len = C->buffer_len;
-	int i, codes = 0;
+	int i;
 	int escape_n = 0;
 	char escape[32];
 	bool onescape = false;
@@ -1029,7 +1080,7 @@ static void optimize(void) {
 			//	eprintf ("ERN (%d) %s%c", pos, escape, 10);
 				onescape = false;
 			} else {
-				if (escape_n + 1 >= sizeof(escape)) {
+				if (escape_n + 1 >= sizeof (escape)) {
 					escape_n = 0;
 					onescape = false;
 				}
@@ -1039,7 +1090,6 @@ static void optimize(void) {
 			onescape = true;
 			escape[escape_n++] = buf[i];
 			escape[escape_n] = 0;
-			codes++;
 		}
 	}
 	// eprintf ("FROM %d TO %d (%d)%c", C->buffer_len, len, codes, 10);
@@ -1047,15 +1097,13 @@ static void optimize(void) {
 	free (oldstr);
 }
 
-#if R2_580
 R_API char *r_cons_drain(void) {
-	const char *buf = r_cons_get_buffer();
-	size_t buf_size = r_cons_get_buffer_size();
+	const char *buf = r_cons_get_buffer ();
+	size_t buf_size = r_cons_get_buffer_len ();
 	char *s = r_str_ndup (buf, buf_size);
 	r_cons_reset ();
 	return s;
 }
-#endif
 
 R_API void r_cons_flush(void) {
 	if (!r_cons_instance) {
@@ -1068,13 +1116,14 @@ R_API void r_cons_flush(void) {
 	if (C->noflush) {
 		return;
 	}
-	if (C->errmode == R_CONS_ERRMODE_FLUSH) {
-		r_cons_eflush ();
-	}
 	if (I->null) {
 		r_cons_reset ();
 		return;
 	}
+	if (!r_list_empty (C->marks)) {
+		r_list_free (C->marks);
+		C->marks = r_list_newf ((RListFree)r_cons_mark_free);
+	}
 	if (lastMatters () && !C->lastMode) {
 		// snapshot of the output
 		if (C->buffer_len > C->lastLength) {
@@ -1094,6 +1143,10 @@ R_API void r_cons_flush(void) {
 		}
 	}
 	r_cons_filter ();
+	if (C->buffer_len < 1) {
+		r_cons_reset ();
+		return;
+	}
 	if (r_cons_is_interactive () && I->fdout == 1) {
 		/* Use a pager if the output doesn't fit on the terminal window. */
 		if (C->pageable && C->buffer && I->pager && *I->pager && C->buffer_len > 0 && r_str_char_count (C->buffer, '\n') >= I->rows) {
@@ -1134,7 +1187,7 @@ R_API void r_cons_flush(void) {
 			r_cons_set_raw (true);
 		}
 	}
-	if (tee && *tee) {
+	if (R_STR_ISNOTEMPTY (tee)) {
 		FILE *d = r_sandbox_fopen (tee, "a+");
 		if (d) {
 			if (C->buffer_len != fwrite (C->buffer, 1, C->buffer_len, d)) {
@@ -1147,7 +1200,6 @@ R_API void r_cons_flush(void) {
 	}
 	r_cons_highlight (I->highlight);
 
-	// is_html must be a filter, not a write endpoint
 	if (r_cons_is_interactive () && !r_sandbox_enable (false)) {
 		if (I->linesleep > 0 && I->linesleep < 1000) {
 			int i = 0;
@@ -1180,6 +1232,11 @@ R_API void r_cons_flush(void) {
 		eprintf ("\n");
 		I->newline = false;
 	}
+	if (C->tmp_html) {
+		C->is_html = C->was_html;
+		C->tmp_html = false;
+		C->was_html = false;
+	}
 }
 
 R_API void r_cons_visual_flush(void) {
@@ -1189,7 +1246,7 @@ R_API void r_cons_visual_flush(void) {
 	r_cons_highlight (I->highlight);
 	if (!I->null) {
 /* TODO: this ifdef must go in the function body */
-#if __WINDOWS__
+#if R2__WINDOWS__
 		if (I->vtmode) {
 			r_cons_visual_write (C->buffer);
 		} else {
@@ -1207,7 +1264,6 @@ R_API void r_cons_visual_flush(void) {
 
 R_API void r_cons_print_fps(int col) {
 	int fps = 0, w = r_cons_get_size (NULL);
-	static R_TH_LOCAL ut64 prev = 0LL; //r_time_now_mono ();
 	fps = 0;
 	if (prev) {
 		ut64 now = r_time_now_mono ();
@@ -1224,7 +1280,7 @@ R_API void r_cons_print_fps(int col) {
 	if (col < 1) {
 		col = 12;
 	}
-#ifdef __WINDOWS__
+#ifdef R2__WINDOWS__
 	if (I->vtmode) {
 		eprintf ("\x1b[0;%dH[%d FPS] \n", w - col, fps);
 	} else {
@@ -1348,7 +1404,7 @@ club:
 			C->buffer_len += written;
 		}
 	} else {
-		r_cons_strcat (format);
+		r_cons_print (format);
 	}
 	va_end (ap2);
 	va_end (ap3);
@@ -1356,7 +1412,7 @@ club:
 
 R_API int r_cons_printf(const char *format, ...) {
 	va_list ap;
-	if (!format || !*format) {
+	if (R_STR_ISEMPTY (format)) {
 		return -1;
 	}
 	va_start (ap, format);
@@ -1366,56 +1422,6 @@ R_API int r_cons_printf(const char *format, ...) {
 	return 0;
 }
 
-R_API void r_cons_errmode(int mode) {
-	C->errmode = mode;
-}
-
-R_API void r_cons_errmodes(const char *mode) {
-	int m = -1;
-	if (!strcmp (mode, "echo")) {
-		m = R_CONS_ERRMODE_ECHO;
-	} else if (!strcmp (mode, "null")) {
-		m = R_CONS_ERRMODE_NULL;
-	} else if (!strcmp (mode, "buffer")) {
-		m = R_CONS_ERRMODE_BUFFER;
-	} else if (!strcmp (mode, "quiet")) {
-		m = R_CONS_ERRMODE_QUIET;
-	} else if (!strcmp (mode, "flush")) {
-		m = R_CONS_ERRMODE_FLUSH;
-	}
-	C->errmode = m;
-}
-
-R_API char *r_cons_errstr(void) {
-	char *s = r_strbuf_drain (C->error);
-	C->error = NULL;
-	return s;
-}
-
-// XXX overriden by RLOG apis imho
-R_API int r_cons_eprintf(const char *format, ...) {
-	va_list ap;
-	r_return_val_if_fail (!R_STR_ISEMPTY (format), -1);
-	va_start (ap, format);
-	switch (C->errmode) {
-	case R_CONS_ERRMODE_NULL:
-		break;
-	case R_CONS_ERRMODE_ECHO:
-		vfprintf (stderr, format, ap);
-		break;
-	case R_CONS_ERRMODE_QUIET:
-	case R_CONS_ERRMODE_BUFFER:
-	case R_CONS_ERRMODE_FLUSH:
-		if (!C->error) {
-			C->error = r_strbuf_new ("");
-		}
-		r_strbuf_vappendf (C->error, format, ap);
-		break;
-	}
-	va_end (ap);
-	return C->error? r_strbuf_length (C->error): 0;
-}
-
 R_API int r_cons_get_column(void) {
 	char *line = strrchr (C->buffer, '\n');
 	if (!line) {
@@ -1428,7 +1434,7 @@ R_API int r_cons_get_column(void) {
 /* final entrypoint for adding stuff in the buffer screen */
 R_API int r_cons_write(const char *str, int len) {
 	r_return_val_if_fail (str && len >= 0, -1);
-	if (len == 0) {
+	if (len < 1) {
 		return 0;
 	}
 	if (I->echo) {
@@ -1439,13 +1445,13 @@ R_API int r_cons_write(const char *str, int len) {
 		}
 	}
 	if (str && len > 0 && !I->null) {
-		r_th_lock_enter (&r_cons_lock);
+		R_CRITICAL_ENTER (I);
 		if (palloc (len + 1)) {
 			memcpy (C->buffer + C->buffer_len, str, len);
 			C->buffer_len += len;
 			C->buffer[C->buffer_len] = 0;
 		}
-		r_th_lock_leave (&r_cons_lock);
+		R_CRITICAL_LEAVE (I);
 	}
 	if (C->flush) {
 		r_cons_flush ();
@@ -1468,12 +1474,12 @@ R_API void r_cons_memset(char ch, int len) {
 	}
 }
 
-R_API void r_cons_strcat(const char *str) {
-	int len;
-	if (!str || I->null) {
+R_API void r_cons_print(const char *str) {
+	r_return_if_fail (str);
+	if (!I || I->null) {
 		return;
 	}
-	len = strlen (str);
+	size_t len = strlen (str);
 	if (len > 0) {
 		r_cons_write (str, len);
 	}
@@ -1481,7 +1487,7 @@ R_API void r_cons_strcat(const char *str) {
 
 R_API void r_cons_newline(void) {
 	if (!I->null) {
-		r_cons_strcat ("\n");
+		r_cons_print ("\n");
 	}
 #if 0
 This place is wrong to manage the color reset, can interfire with r2pipe output sending resetchars
@@ -1489,18 +1495,24 @@ and break json output appending extra chars.
 this code now is managed into output.c:118 at function r_cons_w32_print
 now the console color is reset with each \n (same stuff do it here but in correct place ... i think)
 
-#if __WINDOWS__
+#if R2__WINDOWS__
 	r_cons_reset_colors();
 #else
-	r_cons_strcat (Color_RESET_ALL"\n");
+	r_cons_print (Color_RESET_ALL"\n");
 #endif
-	if (I->is_html) r_cons_strcat ("<br />\n");
+	if (I->is_html) r_cons_print ("<br />\n");
 #endif
 }
 
 /* return the aproximated x,y of cursor before flushing */
 // XXX this function is a huge bottleneck
 R_API int r_cons_get_cursor(int *rows) {
+	// This implementation is very slow
+	if (rows) {
+		*rows = 0;
+	}
+	return 0;
+#if 0
 	RConsContext *c = C;
 	int i, col = 0;
 	int row = 0;
@@ -1517,7 +1529,7 @@ R_API int r_cons_get_cursor(int *rows) {
 					i += 18;
 				}
 			} else if (ch2 == '[') {
-				for (++i; str[i] && str[i] != 'J' && str[i] != 'm' && str[i] != 'H'; i++) {
+				for (i++; str[i] && str[i] != 'J' && str[i] != 'm' && str[i] != 'H'; i++) {
 					;
 				}
 			}
@@ -1532,10 +1544,11 @@ R_API int r_cons_get_cursor(int *rows) {
 		*rows = row;
 	}
 	return col;
+#endif
 }
 
 R_API bool r_cons_is_windows(void) {
-#if __WINDOWS__
+#if R2__WINDOWS__
 	return true;
 #else
 	char *e = r_sys_getenv ("WSL_INTEROP");
@@ -1548,10 +1561,9 @@ R_API bool r_cons_is_windows(void) {
 R_API bool r_cons_is_tty(void) {
 #if EMSCRIPTEN || __wasi__
 	return false;
-#elif __UNIX__
+#elif R2__UNIX__
 	struct winsize win = {0};
-	const char *tty;
-	struct stat sb;
+	struct stat sb = {0};
 
 	if (!isatty (1)) {
 		return false;
@@ -1562,7 +1574,7 @@ R_API bool r_cons_is_tty(void) {
 	if (!win.ws_col || !win.ws_row) {
 		return false;
 	}
-	tty = ttyname (1);
+	const char *tty = ttyname (1);
 	if (!tty) {
 		return false;
 	}
@@ -1570,13 +1582,20 @@ R_API bool r_cons_is_tty(void) {
 		return false;
 	}
 	return true;
+#elif R2__WINDOWS__
+	HANDLE hOut = GetStdHandle (STD_OUTPUT_HANDLE);
+	if (GetFileType (hOut) == FILE_TYPE_CHAR) {
+		DWORD unused;
+		return GetConsoleMode (hOut, &unused);
+	}
+	return false;
 #else
 	/* non-UNIX do not have ttys */
 	return false;
 #endif
 }
 
-#if __WINDOWS__
+#if R2__WINDOWS__
 static int __xterm_get_cur_pos(int *xpos) {
 	int ypos = 0;
 	const char *get_pos = R_CONS_GET_CURSOR_POSITION;
@@ -1654,7 +1673,7 @@ static bool __xterm_get_size(void) {
 
 // XXX: if this function returns <0 in rows or cols expect MAYHEM
 R_API int r_cons_get_size(int *rows) {
-#if __WINDOWS__
+#if R2__WINDOWS__
 	CONSOLE_SCREEN_BUFFER_INFO csbi;
 	bool ret = GetConsoleScreenBufferInfo (GetStdHandle (STD_OUTPUT_HANDLE), &csbi);
 	if (ret) {
@@ -1673,7 +1692,7 @@ R_API int r_cons_get_size(int *rows) {
 #elif EMSCRIPTEN || __wasi__
 	I->columns = 80;
 	I->rows = 23;
-#elif __UNIX__
+#elif R2__UNIX__
 	struct winsize win = {0};
 	if (isatty (0) && !ioctl (0, TIOCGWINSZ, &win)) {
 		if ((!win.ws_col) || (!win.ws_row)) {
@@ -1739,7 +1758,7 @@ R_API int r_cons_get_size(int *rows) {
 	return R_MAX (0, I->columns);
 }
 
-#if __WINDOWS__
+#if R2__WINDOWS__
 R_API int r_cons_is_vtcompat(void) {
 	DWORD major;
 	DWORD minor;
@@ -1763,11 +1782,11 @@ R_API int r_cons_is_vtcompat(void) {
 	char *term = r_sys_getenv ("TERM");
 	if (term) {
 		if (strstr (term, "xterm")) {
-			I->term_xterm = 1;
+			I->term_xterm = true;
 			free (term);
 			return 2;
 		}
-		I->term_xterm = 0;
+		I->term_xterm = false;
 		free (term);
 	}
 	char *ansicon = r_sys_getenv ("ANSICON");
@@ -1778,9 +1797,10 @@ R_API int r_cons_is_vtcompat(void) {
 	bool win_support = 0;
 	RSysInfo *info = r_sys_info ();
 	if (info && info->version) {
-		char *dot = strtok (info->version, ".");
+		char *save_ptr = NULL;
+		char *dot = r_str_tok_r (info->version, ".", &save_ptr);
 		major = atoi (dot);
-		dot = strtok (NULL, ".");
+		dot = r_str_tok_r (NULL, ".", &save_ptr);
 		minor = atoi (dot);
 		if (info->release) {
 			release = atoi (info->release);
@@ -1797,13 +1817,13 @@ R_API int r_cons_is_vtcompat(void) {
 #endif
 
 R_API void r_cons_show_cursor(int cursor) {
-#if __WINDOWS__
+#if R2__WINDOWS__
 	if (I->vtmode) {
 #endif
 		if (write (1, cursor ? "\x1b[?25h" : "\x1b[?25l", 6) != 6) {
 			C->breaked = true;
 		}
-#if __WINDOWS__
+#if R2__WINDOWS__
 	} else {
 		static R_TH_LOCAL HANDLE hStdout = NULL;
 		static R_TH_LOCAL DWORD size = -1;
@@ -1829,13 +1849,12 @@ R_API void r_cons_show_cursor(int cursor) {
  *
  * For optimization reasons, there's no initialization flag, so you need to
  * ensure that the make the first call to r_cons_set_raw() with '1' and
- * the next calls ^=1, so: 1, 0, 1, 0, 1, ...
+ * the next calls ^= 1, so: 1, 0, 1, 0, 1, ...
  *
  * If you doesn't use this order you'll probably loss your terminal properties.
  *
  */
 R_API void r_cons_set_raw(bool is_raw) {
-	static R_TH_LOCAL int oldraw = -1;
 	if (oldraw != -1) {
 		if (is_raw == oldraw) {
 			return;
@@ -1843,7 +1862,7 @@ R_API void r_cons_set_raw(bool is_raw) {
 	}
 #if EMSCRIPTEN || __wasi__
 	/* do nothing here */
-#elif __UNIX__
+#elif R2__UNIX__
 	// enforce echo off
 	if (is_raw) {
 		I->term_raw.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
@@ -1851,7 +1870,14 @@ R_API void r_cons_set_raw(bool is_raw) {
 	} else {
 		tcsetattr (0, TCSANOW, &I->term_buf);
 	}
-#elif __WINDOWS__
+#elif R2__WINDOWS__
+	if (I->term_xterm) {
+		char *stty = r_file_path ("stty");
+		if (!stty || *stty == 's') {
+			I->term_xterm = false;
+		}
+		free (stty);
+	}
 	if (is_raw) {
 		if (I->term_xterm) {
 			r_sandbox_system ("stty raw -echo", 1);
@@ -1874,7 +1900,7 @@ R_API void r_cons_set_raw(bool is_raw) {
 
 R_API void r_cons_set_utf8(bool b) {
 	I->use_utf8 = b;
-#if __WINDOWS__
+#if R2__WINDOWS__
 	if (b) {
 		if (IsValidCodePage (CP_UTF8)) {
 			if (!SetConsoleOutputCP (CP_UTF8)) {
@@ -1901,17 +1927,17 @@ R_API void r_cons_set_utf8(bool b) {
 }
 
 R_API void r_cons_invert(int set, int color) {
-	r_cons_strcat (R_CONS_INVERT (set, color));
+	r_cons_print (R_CONS_INVERT (set, color));
 }
 
-/*
-  Enable/Disable scrolling in terminal:
-    FMI: cd libr/cons/t ; make ti ; ./ti
-  smcup: disable terminal scrolling (fullscreen mode)
-  rmcup: enable terminal scrolling (normal mode)
-*/
+#if 0
+Enable/Disable scrolling in terminal:
+FMI: cd libr/cons/t ; make ti ; ./ti
+smcup: disable terminal scrolling (fullscreen mode)
+rmcup: enable terminal scrolling (normal mode)
+#endif
 R_API bool r_cons_set_cup(bool enable) {
-#if __UNIX__
+#if R2__UNIX__
 	const char *code = enable
 		? "\x1b[?1049h" "\x1b" "7\x1b[?47h"
 		: "\x1b[?1049l" "\x1b[?47l" "\x1b" "8";
@@ -1920,7 +1946,7 @@ R_API bool r_cons_set_cup(bool enable) {
 		return false;
 	}
 	fflush (stdout);
-#elif __WINDOWS__
+#elif R2__WINDOWS__
 	if (I->vtmode) {
 		if (enable) {
 			const char *code = enable // xterm + xterm-color
@@ -1946,7 +1972,7 @@ R_API void r_cons_column(int c) {
 	b[C->buffer_len] = 0;
 	r_cons_reset ();
 	// align current buffer N chars right
-	r_cons_strcat_justify (b, c, 0);
+	r_cons_print_justify (b, c, 0);
 	r_cons_gotoxy (0, 0);
 	free (b);
 }
@@ -1964,7 +1990,7 @@ R_API void r_cons_set_last_interactive(void) {
 }
 
 R_API void r_cons_set_title(const char *str) {
-#if __WINDOWS__
+#if R2__WINDOWS__
 #  if defined(_UNICODE)
 	wchar_t* wstr = r_utf8_to_utf16_l (str, strlen (str));
 	if (wstr) {
@@ -2136,7 +2162,7 @@ R_API bool r_cons_drop(int n) {
 	return true;
 }
 
-R_API void r_cons_chop(void) {
+R_API void r_cons_trim(void) {
 	RConsContext *c = C;
 	while (c->buffer_len > 0) {
 		char ch = c->buffer[c->buffer_len - 1];
@@ -2196,11 +2222,71 @@ R_API void r_cons_clear_buffer(void) {
 }
 
 R_API void r_cons_thready(void) {
-	r_th_lock_enter (&r_cons_lock);
-	if (!r_cons_instance) {
-		r_cons_new ();
+	if (r_cons_instance) {
+		R_CRITICAL_ENTER (I);
 	}
 	C->unbreakable = true;
 	r_sys_signable (false); // disable signal handling
-	r_th_lock_leave (&r_cons_lock);
+	if (!r_cons_instance) {
+		r_cons_new ();
+	}
+	if (r_cons_instance) {
+		R_CRITICAL_LEAVE (I);
+	}
+}
+
+#if WITH_STATIC_THEMES
+#include "d_themes.inc.c"
+
+R_API const RConsTheme* r_cons_themes(void) {
+	return (const RConsTheme *)d_themes;
+}
+
+#else
+R_API const RConsTheme* r_cons_themes(void) {
+	return NULL;
+}
+#endif
+
+R_API void r_cons_mark_free(RConsMark *m) {
+	if (m) {
+		free (m->name);
+		free (m);
+	}
+}
+
+R_API void r_cons_mark(ut64 addr, const char *name) {
+	RConsMark *mark = R_NEW0 (RConsMark);
+	if (mark) {
+		RConsContext *ctx = getctx ();
+		mark->addr = addr;
+		int row = 0, col = r_cons_get_cursor (&row);
+		mark->name = strdup (name); // TODO. use a const pool
+		mark->pos = ctx->buffer_len;
+		mark->col = col;
+		mark->row = row;
+		r_list_append (ctx->marks, mark);
+	}
+};
+
+// must be called before
+R_API void r_cons_mark_flush(void) {
+	r_list_free (C->marks);
+}
+
+R_API RConsMark *r_cons_mark_at(ut64 addr, const char *name) {
+	RListIter *iter;
+	RConsMark *mark;
+	r_list_foreach (C->marks, iter, mark) {
+		if (R_STR_ISNOTEMPTY (name)) {
+			if (strcmp (mark->name, name)) {
+				continue;
+			}
+			return mark;
+		}
+		if (addr != UT64_MAX && mark->addr == addr) {
+			return mark;
+		}
+	}
+	return NULL;
 }
