@@ -1,8 +1,11 @@
-/* radare - LGPL - Copyright 2009-2022 - pancake, nibble */
+/* radare - LGPL - Copyright 2009-2024 - pancake, nibble */
 
 #include <r_cons.h>
 #include <r_util/r_print.h>
-#include <sdb.h>
+#include <r_util/r_json.h>
+#include <sdb/sdb.h>
+
+// R2R db/cmd/cons_grep
 
 #define I(x) r_cons_singleton ()->x
 
@@ -18,18 +21,19 @@ static char *strchr_ns(char *s, const char ch) {
 	return p;
 }
 
-static const char *help_detail_tilde[] = {
+static RCoreHelpMessage help_detail_tilde = {
 	"Usage: [command]~[modifier][word,word][endmodifier][[column]][:line]\n"
 	"modifier:", "", "",
 	" &",        "", "all words must match to grep the line",
 	" $[n]",     "", "sort numerically / alphabetically the Nth column",
 	" $",        "", "sort in alphabetic order",
+	" $$",       "", "sort + uniq",
 	" $!",       "", "inverse alphabetical sort",
 	" $!!",      "", "reverse the lines (like the `tac` tool)",
 	" ,",        "", "token to define another keyword",
 	" +",        "", "case insensitive grep (grep -i)",
+	" *",        "", "zoom level",
 	" ^",        "", "words must be placed at the beginning of line",
-	" <",        "", "perform zoom operation on the buffer",
 	" !",        "", "negate grep",
 	" ?",        "", "count number of matching lines",
 	" ?.",       "", "count number chars",
@@ -40,12 +44,15 @@ static const char *help_detail_tilde[] = {
 	" ...",      "", "internal 'hud' (like V_)",
 	" ....",     "", "internal 'hud' in one line",
 	" :)",       "", "parse C-like output from decompiler",
+	" <50",      "", "perform zoom to the given text width on the buffer",
+	" <>",       "", "xml indentation",
 	" {:",       "", "human friendly indentation (yes, it's a smiley)",
 	" {:..",     "", "less the output of {:",
 	" {:...",    "", "hud the output of {:",
 	" {}",       "", "json indentation",
 	" {}..",     "", "less json indentation",
 	" {}...",    "", "hud json indentation",
+	" {=}",       "", "gron-like output (key=value)",
 	" {path}",   "", "json path grep",
 	"endmodifier:", "", "",
 	" $",        "", "words must be placed at the end of line",
@@ -70,9 +77,9 @@ R_API void r_cons_grep_help(void) {
 
 #define R_CONS_GREP_BUFSIZE 4096
 
-static void parse_grep_expression(const char *str) {
-	static R_TH_LOCAL char buf[R_CONS_GREP_BUFSIZE];
-	static R_TH_LOCAL char *ptrs[R_CONS_GREP_COUNT];
+R_API void r_cons_grep_expression(const char *str) {
+	char buf[R_CONS_GREP_BUFSIZE];
+	char *ptrs[R_CONS_GREP_COUNT];
 	int wlen, len, is_range, num_is_parsed, fail = false;
 	char *ptr, *optr, *ptr2, *ptr3, *end_ptr = NULL, last;
 	ut64 range_begin, range_end;
@@ -106,6 +113,7 @@ static void parse_grep_expression(const char *str) {
 	ptrs_length = 1;
 	ptrs[0] = ptr;
 
+	// TODO: replace with r_str_split_by ("~");
 	while ((ptrs[ptrs_length] = (strchr (ptr, '~')))) {
 		*(ptrs[ptrs_length]) = '\0';
 		ptrs[ptrs_length]++;
@@ -117,8 +125,7 @@ static void parse_grep_expression(const char *str) {
 		}
 	}
 
-	free (grep->str);
-	grep->str = NULL;
+	R_FREE (grep->str);
 	bool first = true;
 	ctx->sorted_column = 0;
 	size_t i;
@@ -157,6 +164,9 @@ static void parse_grep_expression(const char *str) {
 					} else if (r_str_startswith (ptr, "{:..")) {
 						grep->less = 1;
 					}
+				} else if (ptr[1] == '=' && ptr[2] == '}') {
+					grep->gron = true;
+					ptr += 2;
 				} else if (ptr[1] == '}') {
 					// standard json indentation
 					grep->json = 1;
@@ -190,6 +200,8 @@ static void parse_grep_expression(const char *str) {
 					}
 					grep->sort_invert = true;
 					ptr++;
+				} else if (*ptr == '$') {
+					grep->sort_uniq = true;
 				} else {
 					grep->sort_invert = false;
 				}
@@ -197,7 +209,8 @@ static void parse_grep_expression(const char *str) {
 					ptr++;
 				}
 				if (*ptr == ':') {
-					grep->sort_row = atoi (++ptr);
+					ptr++;
+					grep->sort_row = atoi (ptr);
 					ptr++;
 				}
 				break;
@@ -206,8 +219,12 @@ static void parse_grep_expression(const char *str) {
 				grep->amp = 1;
 				break;
 			case '<':
-				grep->zoom = atoi (++ptr);
-				//grep->zoomy = atoi (arg);
+				ptr++;
+				if (*ptr == '>') {
+					grep->xml = true;
+				} else {
+					grep->zoom = atoi (ptr);
+				}
 				break;
 			case '+':
 				if (first) {
@@ -383,16 +400,15 @@ static void parse_grep_expression(const char *str) {
 // Finds and returns next intgrep expression,
 // unescapes escaped twiddles
 static char *find_next_intgrep(char *cmd, const char *quotes) {
-	char *p;
 	do {
-		p = (char *)r_str_firstbut (cmd, '~', quotes);
+		char *p = (char *)r_str_firstbut (cmd, '~', quotes);
 		if (!p) {
 			break;
 		}
 		if (p == cmd || *(p - 1) != '\\') {
 			return (char *)p;
 		}
-		//twiddle unescape
+		// twiddle unescape
 		r_str_cpy (p - 1, p);
 		cmd = p + 1;
 	} while (*cmd);
@@ -404,17 +420,16 @@ static char *find_next_intgrep(char *cmd, const char *quotes) {
  * with reshaped grep expression.
 */
 static char *preprocess_filter_expr(char *cmd, const char *quotes) {
-	char *p1, *p2, *ns = NULL;
+	char *p2, *ns = NULL;
 	const char *strsep = "&";
-	int len;
 	int i;
 
-	p1 = find_next_intgrep (cmd, quotes);
+	char *p1 = find_next_intgrep (cmd, quotes);
 	if (!p1) {
 		return NULL;
 	}
 
-	len = strlen (p1);
+	int len = strlen (p1);
 	if (len > 4 && r_str_endswith (p1, "~?") && p1[len - 3] != '\\') {
 		p1[len - 2] = '\0';
 		ns = r_str_append (ns, "?");
@@ -436,9 +451,7 @@ static char *preprocess_filter_expr(char *cmd, const char *quotes) {
 		ns = r_str_append (ns, "~");
 	}
 
-	ns = r_str_append (ns, p1 + 1);
-
-	return ns;
+	return r_str_append (ns, p1 + 1);
 }
 
 R_API void r_cons_grep_parsecmd(char *cmd, const char *quotestr) {
@@ -446,14 +459,13 @@ R_API void r_cons_grep_parsecmd(char *cmd, const char *quotestr) {
 	char *ptr = preprocess_filter_expr (cmd, quotestr);
 	if (ptr) {
 		r_str_trim (cmd);
-		parse_grep_expression (ptr);
+		r_cons_grep_expression (ptr);
 		free (ptr);
 	}
 }
 
 R_API char *r_cons_grep_strip(char *cmd, const char *quotestr) {
 	char *ptr = NULL;
-
 	if (cmd) {
 		ptr = preprocess_filter_expr (cmd, quotestr);
 		r_str_trim (cmd);
@@ -461,13 +473,6 @@ R_API char *r_cons_grep_strip(char *cmd, const char *quotestr) {
 	return ptr;
 }
 
-R_API void r_cons_grep_process(char *grep) {
-	if (grep) {
-		parse_grep_expression (grep);
-		free (grep);
-	}
-}
-
 static int cmp(const void *a, const void *b) {
 	char *da = NULL;
 	char *db = NULL;
@@ -494,23 +499,81 @@ static int cmp(const void *a, const void *b) {
 		return ret;
 	}
 	if (da && db) {
-		int ret = strcmp (ca, cb);
-		free (da);
-		free (db);
-		return ret;
+		a = ca;
+		b = cb;
 	}
+	int res = strcmp (a, b);
 	free (da);
 	free (db);
-	return strcmp (a, b);
+	return res;
+}
+
+static bool gron(RStrBuf *sb, RJson *node, const char *root) {
+	r_return_val_if_fail (sb && node && root, false);
+	switch (node->type) {
+	case R_JSON_ARRAY:
+		{
+			RJson *cn = node->children.first;
+			int n = 0;
+			r_strbuf_appendf (sb, "%s = [];\n", root);
+			while (cn) {
+				char *newroot = r_str_newf ("%s[%d]", root, n);
+				gron (sb, cn, newroot);
+				free (newroot);
+				cn = cn->next;
+				n++;
+			}
+		}
+		break;
+	case R_JSON_OBJECT:
+		{
+			RJson *cn = node->children.first;
+			r_strbuf_appendf (sb, "%s = {};\n", root);
+			while (cn) {
+				char *newroot = r_str_newf ("%s.%s", root, cn->key);
+				gron (sb, cn, newroot);
+				cn = cn->next;
+			}
+		}
+		break;
+	case R_JSON_STRING:
+		{
+			size_t l = strlen (node->str_value);
+			char *estr = r_str_encoded_json (node->str_value, l, PJ_ENCODING_STR_DEFAULT);
+			r_strbuf_appendf (sb, "%s = \"%s\";\n", root, estr);
+			free (estr);
+		}
+		break;
+	case R_JSON_BOOLEAN:
+		r_strbuf_appendf (sb, "%s = %s;\n", root, r_str_bool (node->num.u_value));
+		break;
+	case R_JSON_INTEGER:
+		r_strbuf_appendf (sb, "%s = %"PFMT64d";\n", root, node->num.u_value);
+		break;
+	case R_JSON_NULL:
+		r_strbuf_appendf (sb, "%s = null;\n", root);
+		break;
+	case R_JSON_DOUBLE:
+		r_strbuf_appendf (sb, "%s = %lf;\n", root, node->num.dbl_value);
+		break;
+	default:
+		R_LOG_WARN ("unknown json type %s", r_json_type (node));
+		break;
+	}
+	return true;
+}
+
+static inline ut64 cmpstrings(const void *a) {
+	return r_str_hash64 (a);
 }
 
 R_API void r_cons_grepbuf(void) {
 	RCons *cons = r_cons_singleton ();
 	const char *buf = cons->context->buffer;
-	const int len = cons->context->buffer_len;
+	size_t len = cons->context->buffer_len;
 	RConsGrep *grep = &cons->context->grep;
 	const char *in = buf;
-	int ret, total_lines = 0, l = 0, tl = 0;
+	int ret, l = 0, tl = 0;
 	bool show = false;
 	if (cons->context->filter) {
 		cons->context->buffer_len = 0;
@@ -541,10 +604,10 @@ R_API void r_cons_grepbuf(void) {
 		return;
 	}
 
-	if ((!len || !buf || !*buf) && (grep->json || grep->less)) {
-		grep->json = 0;
+	if ((!len || R_STR_ISEMPTY (buf)) && (grep->json || grep->less)) {
+		grep->json = false;
+		grep->hud = false;
 		grep->less = 0;
-		grep->hud = 0;
 		return;
 	}
 	if (grep->ascart) {
@@ -573,6 +636,33 @@ R_API void r_cons_grepbuf(void) {
 		free (sin);
 		return;
 	}
+	if (grep->gron) {
+		char *a = strdup (cons->context->buffer);
+		RJson *node = r_json_parse (a);
+		RStrBuf *sb = r_strbuf_new ("");
+		gron (sb, node, "json");
+		char *s = r_strbuf_drain (sb);
+		R_FREE (cons->context->buffer);
+		cons->context->buffer_len = 0;
+		cons->context->buffer_sz = 0;
+		r_cons_print (s);
+		buf = cons->context->buffer;
+		len = cons->context->buffer_len;
+		r_json_free (node);
+		free (a);
+		free (s);
+		goto continuation;
+	}
+	if (grep->xml) {
+		// parse and indent xml
+		char *x = r_str_ndup (buf, len);
+		char *xi = r_xml_indent (x);
+		free (cons->context->buffer);
+		in = buf = cons->context->buffer = xi;
+		len = cons->context->buffer_len = strlen (xi);
+		free (x);
+		return;
+	}
 	if (grep->json) {
 		if (grep->json_path) {
 			char *u = sdb_json_get_str (cons->context->buffer, grep->json_path);
@@ -606,16 +696,44 @@ R_API void r_cons_grepbuf(void) {
 			cons->context->buffer = out;
 			cons->context->buffer_len = strlen (out);
 			cons->context->buffer_sz = cons->context->buffer_len + 1;
-			grep->json = 0;
+			grep->json = false;
+			in = buf = out;
+			len = cons->context->buffer_len;
+			cons->context->grep_color = true;
+			// R2R db/cmd/cmd_iz
+			R_FREE (grep->str);
+			if (grep->nstrings > 0) {
+				cons->context->grep_color = false;
+				// shift them all!!
+				int i;
+				for (i = 0; i < grep->nstrings; i++) {
+					r_str_cpy (grep->strings[i], grep->strings[i + 1]);
+				}
+				grep->nstrings--;
+			}
+#if 1
 			if (grep->hud) {
 				grep->hud = false;
 				r_cons_hud_string (cons->context->buffer);
-			} else if (grep->less) {
+				return;
+			}
+			if (grep->less) {
 				grep->less = 0;
 				r_cons_less_str (cons->context->buffer, NULL);
+				return;
 			}
+			if (grep->nstrings < 1) {
+				return;
+			}
+#endif
 		}
-		return;
+		if (grep->nstrings < 1) {
+			return;
+		}
+		buf = cons->context->buffer;
+		len = cons->context->buffer_len;
+		grep->range_line = 1;
+		goto continuation;
 		// cons->lines = ?? return 3;
 	}
 	if (grep->less) {
@@ -636,42 +754,46 @@ R_API void r_cons_grepbuf(void) {
 		} else {
 			r_cons_less_str (buf, NULL);
 			cons->context->buffer_len = 0;
-			if (cons->context->buffer) {
-				cons->context->buffer[0] = 0;
-			}
+			cons->context->buffer_sz = 0;
 			R_FREE (cons->context->buffer);
 		}
 		return;
 	}
-	RStrBuf *ob = r_strbuf_new ("");
+	RStrBuf *ob = NULL;
+continuation:
+	ob = r_strbuf_new ("");
 	// if we modify cons->lines we should update I.context->buffer too
 	cons->lines = 0;
 	// used to count lines and change negative grep.line values
-	while ((int) (size_t) (in - buf) < len) {
-		char *p = strchr (in, '\n');
-		if (!p) {
-			break;
+	if ((!grep->range_line && grep->line < 0) || grep->range_line) {
+		int total_lines = 0;
+		while ((int) (size_t) (in - buf) < len) {
+			char *p = strchr (in, '\n');
+			if (!p) {
+				break;
+			}
+			l = p - in;
+			if (l > 0) {
+				in += l + 1;
+			} else {
+				in++;
+			}
+			total_lines++;
 		}
-		l = p - in;
-		if (l > 0) {
-			in += l + 1;
-		} else {
-			in++;
+		if (!grep->range_line && grep->line < 0) {
+			grep->line = total_lines + grep->line;
 		}
-		total_lines++;
-	}
-	if (!grep->range_line && grep->line < 0) {
-		grep->line = total_lines + grep->line;
-	}
-	if (grep->range_line == 1) {
-		if (grep->f_line < 0) {
-			grep->f_line = total_lines + grep->f_line;
-		}
-		if (grep->l_line <= 0) {
-			grep->l_line = total_lines + grep->l_line;
+		if (grep->range_line == 1) {
+			if (grep->f_line < 0) {
+				grep->f_line = total_lines + grep->f_line;
+			}
+			if (grep->l_line <= 0) {
+				grep->l_line = total_lines + grep->l_line;
+			}
 		}
 	}
 	bool is_range_line_grep_only = grep->range_line != 2 && grep->str && *grep->str == '\0';
+
 	in = buf;
 	while ((int) (size_t) (in - buf) < len) {
 		char *p = strchr (in, '\n');
@@ -705,6 +827,9 @@ R_API void r_cons_grepbuf(void) {
 					show = true;
 				}
 			}
+			if (grep->counter) {
+				show = false;
+			}
 			if ((!ret && is_range_line_grep_only) || ret > 0) {
 				if (show) {
 					char *str = r_str_ndup (tline, ret);
@@ -725,8 +850,8 @@ R_API void r_cons_grepbuf(void) {
 					if (str) {
 						r_strbuf_append (ob, str);
 						r_strbuf_append (ob, "\n");
+						free (str);
 					}
-					free (str);
 				}
 				if (!grep->range_line) {
 					show = false;
@@ -744,12 +869,11 @@ R_API void r_cons_grepbuf(void) {
 		}
 	}
 
-	cons->context->buffer_len = r_strbuf_length (ob);
+	const int ob_len = r_strbuf_length (ob);
+	cons->context->buffer_len = ob_len;
 	if (grep->counter) {
 		int cnt = grep->charCounter? strlen (cons->context->buffer): cons->lines;
-		if (cons->context->buffer) {
-			free (cons->context->buffer);
-		}
+		free (cons->context->buffer);
 		cons->context->buffer = r_str_newf ("%d\n", cnt);
 		cons->context->buffer_len = strlen (cons->context->buffer);
 		cons->context->buffer_sz = cons->context->buffer_len+1;
@@ -757,8 +881,7 @@ R_API void r_cons_grepbuf(void) {
 		r_strbuf_free (ob);
 		return;
 	}
-	
-	const int ob_len = r_strbuf_length (ob);
+
 	if (ob_len >= cons->context->buffer_sz) {
 		cons->context->buffer_sz = ob_len + 1;
 		cons->context->buffer = r_strbuf_drain (ob);
@@ -767,11 +890,10 @@ R_API void r_cons_grepbuf(void) {
 		cons->context->buffer[ob_len] = 0;
 		r_strbuf_free (ob);
 	}
-	cons->context->buffer_len = ob_len;
 
 	if (grep->sort != -1 || grep->sort_invert) {
 #define INSERT_LINES(list)\
-		do {\
+		if (list) {\
 			r_list_foreach (list, iter, str) {\
 				int slen = strlen (str);\
 				memcpy (ptr, str, slen);\
@@ -779,8 +901,7 @@ R_API void r_cons_grepbuf(void) {
 				ptr += slen + 1;\
 				nl++;\
 			}\
-		}\
-		while (false)
+		}
 
 		RListIter *iter;
 		int nl = 0;
@@ -795,8 +916,16 @@ R_API void r_cons_grepbuf(void) {
 		if (grep->sort_invert) {
 			r_list_reverse (ctx->sorted_lines);
 		}
+		if (grep->sort_uniq) {
+			r_list_uniq_inplace (ctx->sorted_lines, cmpstrings);
+			r_list_free (ctx->unsorted_lines);
+			ctx->unsorted_lines = NULL;
+			nl = 0;
+		}
+		cons->context->buffer_len = 0;
 		INSERT_LINES (ctx->unsorted_lines);
 		INSERT_LINES (ctx->sorted_lines);
+		cons->context->buffer_len = (ptr - cons->context->buffer);
 		cons->lines = nl;
 		r_list_free (ctx->sorted_lines);
 		ctx->sorted_lines = NULL;
@@ -811,6 +940,7 @@ R_API int r_cons_grep_line(char *buf, int len) {
 	RConsGrep *grep = &cons->context->grep;
 	const char *delims = " |,;=\t";
 	char *tok = NULL;
+	char *save_ptr = NULL;
 	bool hit = true;
 	int outlen = 0;
 	bool use_tok = false;
@@ -878,7 +1008,7 @@ R_API int r_cons_grep_line(char *buf, int len) {
 		}
 		if (use_tok && grep->tokens_used) {
 			for (i = 0; i < R_CONS_GREP_TOKENS; i++) {
-				tok = strtok (i? NULL: in, delims);
+				tok = r_str_tok_r (i? NULL: in, delims, &save_ptr);
 				if (tok) {
 					if (grep->tokens[i]) {
 						int toklen = strlen (tok);
@@ -948,6 +1078,6 @@ R_API int r_cons_grep_line(char *buf, int len) {
 
 R_API void r_cons_grep(const char *grep) {
 	r_return_if_fail (grep);
-	parse_grep_expression (grep);
+	r_cons_grep_expression (grep);
 	r_cons_grepbuf ();
 }
