@@ -1,19 +1,21 @@
-/* radare - LGPL - Copyright 2010-2022 - pancake, rhl */
-
-#include <r_core.h>
-#include <rvc.h>
-#define USE_R2 1
-#include <spp/spp.h>
+/* radare - LGPL - Copyright 2010-2023 - pancake, rhl */
 
 #define PROJECT_EXPERIMENTAL 0
 
+#include <r_core.h>
+#include <rvc.h>
+// required to make spp use RStrBuf instead of SStrBuf
+#define USE_R2 1
+#include <spp/spp.h>
+
 // project apis to be used from cmd_project.c
+// TODO: Use .zrp as in zipped radare project
 
 static bool is_valid_project_name(const char *name) {
 	if (r_str_len_utf8 (name) >= 64) {
 		return false;
 	}
-	const char *extention = r_str_endswith (name, ".zip")? r_str_last (name, ".zip"): NULL;
+	const char * const extention = r_str_endswith (name, ".zip")? r_str_last (name, ".zip"): NULL;
 	for (; *name && name != extention; name++) {
 		if (IS_DIGIT (*name) || IS_LOWER (*name) || *name == '_') {
 			continue;
@@ -100,13 +102,12 @@ R_API void r_core_project_cat(RCore *core, const char *name) {
 R_API int r_core_project_list(RCore *core, int mode) {
 	PJ *pj = NULL;
 	RListIter *iter;
-	RList *list;
 
 	char *foo, *path = r_file_abspath (r_config_get (core->config, "dir.projects"));
 	if (!path) {
 		return 0;
 	}
-	list = r_sys_dir (path);
+	RList *list = r_sys_dir (path);
 	switch (mode) {
 	case 'j':
 		pj = pj_new ();
@@ -168,7 +169,7 @@ R_API int r_core_project_delete(RCore *core, const char *prjfile) {
 }
 
 static bool load_project_rop(RCore *core, const char *prjfile) {
-	r_return_val_if_fail (core && !R_STR_ISEMPTY (prjfile), false);
+	r_return_val_if_fail (core && R_STR_ISNOTEMPTY (prjfile), false);
 	char *path, *db = NULL, *path_ns;
 	bool found = 0;
 	SdbListIter *it;
@@ -272,12 +273,13 @@ R_API void r_core_project_execute_cmds(RCore *core, const char *prjfile) {
 	spp_eval (data, &out);
 	free (data);
 	data = strdup (r_strbuf_get (out.cout));
-	char *bol = strtok (data, "\n");
+	char *save_ptr = NULL;
+	char *bol = r_str_tok_r (data, "\n", &save_ptr);
 	while (bol) {
 		if (bol[0] == ':') {
 			r_core_cmd0 (core, bol + 1);
 		}
-		bol = strtok (NULL, "\n");
+		bol = r_str_tok_r (NULL, "\n", &save_ptr);
 	}
 	free (data);
 }
@@ -286,14 +288,15 @@ typedef struct {
 	RCore *core;
 	char *prj_name;
 	char *rc_path;
-} projectState;
+} ProjectState;
 
 static bool r_core_project_load(RCore *core, const char *prj_name, const char *rcpath) {
+	r_return_val_if_fail (core, false);
 	if (R_STR_ISEMPTY (prj_name)) {
 		prj_name = r_core_project_name (core, rcpath);
 	}
 	if (r_project_is_loaded (core->prj)) {
-		eprintf ("o--;e prj.name=\n");
+		R_LOG_INFO ("o--;e prj.name=");
 	//	return false;
 	}
 	if (!r_project_open (core->prj, prj_name, rcpath)) {
@@ -320,11 +323,7 @@ static bool r_core_project_load(RCore *core, const char *prj_name, const char *r
 	char *prj_path = r_file_dirname(rcpath);
 	if (prj_path) {
 		//check if the project uses git
-		Rvc *vc = r_vc_git_open (prj_path);
-		if (!vc) {
-			// if the project does not use git, try rvc
-			vc = r_vc_open (prj_path);
-		}
+		Rvc *vc = rvc_open (prj_path, RVC_TYPE_GIT);
 		core->prj->rvc = vc;
 		free (prj_path);
 	} else {
@@ -339,7 +338,7 @@ static bool r_core_project_load(RCore *core, const char *prj_name, const char *r
 }
 
 static RThreadFunctionRet project_load_background(RThread *th) {
-	projectState *ps = th->user;
+	ProjectState *ps = th->user;
 	r_core_project_load (ps->core, ps->prj_name, ps->rc_path);
 	free (ps->prj_name);
 	free (ps->rc_path);
@@ -348,13 +347,13 @@ static RThreadFunctionRet project_load_background(RThread *th) {
 }
 
 R_API RThread *r_core_project_load_bg(RCore *core, const char *prj_name, const char *rc_path) {
-	projectState *ps = R_NEW0 (projectState);
+	ProjectState *ps = R_NEW0 (ProjectState);
 	ps->core = core;
 	ps->prj_name = strdup (prj_name);
 	ps->rc_path = strdup (rc_path);
 	RThread *th = r_th_new (project_load_background, ps, false);
 	if (th) {
-		r_th_start (th, false);
+		r_th_start (th);
 		char thname[32] = {0};
 		size_t thlen = R_MIN (strlen (prj_name), sizeof (thname) - 1);
 		r_str_ncpy (thname, prj_name, thlen);
@@ -372,13 +371,12 @@ R_API bool r_core_project_open(RCore *core, const char *prj_path) {
 		R_LOG_ERROR ("There's a project already opened");
 		ask_for_closing = false;
 		bool ccs = interactive? r_cons_yesno ('y', "Close current session? (Y/n)"): true;
-		if (ccs) {
-			r_core_cmd0 (core, "o--");
-			r_core_cmd0 (core, "P-");
-		} else {
+		if (!ccs) {
 			R_LOG_ERROR ("Project not loaded");
 			return false;
 		}
+		r_core_cmd0 (core, "o--");
+		r_core_cmd0 (core, "P-");
 	}
 	char *prj_name = r_core_project_name (core, prj_path);
 	char *prj_script = get_project_script_path (core, prj_path);
@@ -419,9 +417,14 @@ static char *get_project_name(const char *prj_script) {
 			if (feof (fd)) {
 				break;
 			}
-			if (!strncmp (buf, "\"e prj.name = ", 14)) {
-				buf[strlen (buf) - 2] = 0;
-				file = r_str_new (buf + 14);
+			if (r_str_startswith (buf, "\"\"e prj.name = ")) {
+				file = strdup (buf + strlen ("\"\"e prj.name"));
+				break;
+			}
+			if (r_str_startswith (buf, "\"e prj.name = ")) {
+				// if (!strncmp (buf, "\"e prj.name = ", 14))
+				buf[strlen (buf) - 2] = 0; // remove trailing '"'
+				file = strdup (buf + 14);
 				break;
 			}
 		}
@@ -489,13 +492,6 @@ static bool store_files_and_maps(RCore *core, RIODesc *desc, ut32 id) {
 }
 #endif
 
-static char *r_cons_drain(void) {
-	const char *buf = r_cons_get_buffer ();
-	size_t buf_size = r_cons_get_buffer_len ();
-	char *s = r_str_ndup (buf, buf_size);
-	r_cons_reset ();
-	return s;
-}
 static void flush(RStrBuf *sb) {
 	char * s = r_cons_drain ();
 	if (s) {
@@ -622,6 +618,27 @@ R_API bool r_core_project_save_script(RCore *core, const char *file, int opts) {
 	return true;
 }
 
+static void r_core_project_zip(RCore *core, const char *prj_dir) {
+	char *cwd = r_sys_getdir ();
+	const char *prj_name = r_file_basename (prj_dir);
+	if (r_sys_chdir (prj_dir)) {
+		if (!strchr (prj_name, '\'')) {
+			r_sys_chdir ("..");
+			char *zipfile = r_str_newf ("%s.zip", prj_name);
+			r_file_rm (zipfile);
+			// XXX use the ZIP api instead!
+			r_sys_cmdf ("zip -r %s %s", zipfile, prj_name);
+			free (zipfile);
+		} else {
+			R_LOG_WARN ("Command injection attempt?");
+		}
+	} else {
+		R_LOG_ERROR ("Cannot chdir %s", prj_dir);
+	}
+	r_sys_chdir (cwd);
+	free (cwd);
+}
+
 R_API bool r_core_project_save(RCore *core, const char *prj_name) {
 	bool scr_null = false;
 	bool ret = true;
@@ -698,7 +715,9 @@ R_API bool r_core_project_save(RCore *core, const char *prj_name) {
 	if (core->prj->rvc || r_config_get_b (core->config, "prj.vc")) {
 		// assume that if the repo is not loaded, the repo doesn't exist
 		if (!core->prj->rvc) {
-			if (!(core->prj->rvc = rvc_git_init (core, prj_dir))) {
+			core->prj->rvc = rvc_open (prj_dir, RVC_TYPE_GIT);
+			if (!core->prj->rvc) {
+				R_LOG_WARN ("Cannot initialize git repositorty");
 				free (prj_dir);
 				free (script_path);
 				return false;
@@ -707,13 +726,15 @@ R_API bool r_core_project_save(RCore *core, const char *prj_name) {
 		RList *paths = r_list_new ();
 		if (paths) {
 			if (r_list_append (paths, prj_dir)) {
-				if (!rvc_git_commit (core, core->prj->rvc,
-							NULL, NULL, paths)) {
+				const char *author = r_config_get (core->config, "cfg.user");
+				const char *message = r_config_get (core->config, "prj.vc.message");
+				if (!rvc_commit (core->prj->rvc, message, author, paths)) {
 					r_list_free (paths);
 					free (prj_dir);
 					free (script_path);
 					return false;
 				}
+				rvc_save (core->prj->rvc);
 			} else {
 				r_list_free (paths);
 				free (prj_dir);
@@ -726,25 +747,12 @@ R_API bool r_core_project_save(RCore *core, const char *prj_name) {
 			return false;
 		}
 	}
-	if (r_config_get_i (core->config, "prj.zip")) {
-		char *cwd = r_sys_getdir ();
-		const char *prj_name = r_file_basename (prj_dir);
-		if (r_sys_chdir (prj_dir)) {
-			if (!strchr (prj_name, '\'')) {
-				r_sys_chdir ("..");
-				r_sys_cmdf ("rm -f '%s.zip'; zip -r '%s'.zip '%s'",
-					prj_name, prj_name, prj_name);
-			} else {
-				R_LOG_WARN ("Command injection attempt?");
-			}
-		} else {
-			R_LOG_ERROR ("Cannot chdir %s", prj_dir);
-		}
-		r_sys_chdir (cwd);
-		free (cwd);
+	if (r_config_get_b (core->config, "prj.zip")) {
+		r_core_project_zip (core, prj_dir);
 	}
 	// LEAK : not always in heap free (prj_name);
-	free (prj_dir);
+	free (core->prj->path);
+	core->prj->path = prj_dir;
 	if (scr_null) {
 		r_config_set_b (core->config, "scr.null", true);
 	}
@@ -769,7 +777,9 @@ R_API bool r_core_project_is_dirty(RCore *core) {
 }
 
 R_API void r_core_project_undirty(RCore *core) {
+	R_CRITICAL_ENTER (core);
 	core->config->is_dirty = false;
 	core->anal->is_dirty = false;
 	core->flags->is_dirty = false;
+	R_CRITICAL_LEAVE (core);
 }
