@@ -55,7 +55,7 @@ R_API void r_core_wait(RCore *core) {
 }
 
 static void http_logf(RCore *core, const char *fmt, ...) {
-	bool http_log_enabled = r_config_get_i (core->config, "http.log");
+	bool http_log_enabled = r_config_get_b (core->config, "http.log");
 	va_list ap;
 	va_start (ap, fmt);
 	if (http_log_enabled) {
@@ -96,7 +96,7 @@ static char *rtrcmd(TextLog T, const char *str) {
 static void showcursor(RCore *core, int x) {
 	if (core && core->vmode) {
 		r_cons_show_cursor (x);
-		r_cons_enable_mouse (x? r_config_get_i (core->config, "scr.wheel"): false);
+		r_cons_enable_mouse (x? r_config_get_b (core->config, "scr.wheel"): false);
 	} else {
 		r_cons_enable_mouse (false);
 	}
@@ -112,7 +112,7 @@ static void rtr_textlog_chat(RCore *core, TextLog T) {
 	const char *me = r_config_get (core->config, "cfg.user");
 	char *ret, msg[1024] = {0};
 
-	eprintf ("Type '/help' for commands and ^D to quit:\n");
+	R_LOG_INFO ("Type '/help' for commands and ^D to quit:");
 	char *oldprompt = strdup (r_line_singleton ()->prompt);
 	snprintf (prompt, sizeof (prompt) - 1, "[%s]> ", me);
 	r_line_set_prompt (prompt);
@@ -160,8 +160,8 @@ static void rtr_textlog_chat(RCore *core, TextLog T) {
 			free (rtrcmd (T, "T-"));
 		} else if (!strcmp (buf, "/quit")) {
 			goto beach;
-		} else if (*buf=='/') {
-			eprintf ("Unknown command: %s\n", buf);
+		} else if (*buf == '/') {
+			R_LOG_ERROR ("Unknown command: %s", buf);
 		} else {
 			char *cmd = r_str_newf ("T [%s] %s", me, buf);
 			free (rtrcmd (T, cmd));
@@ -179,7 +179,7 @@ R_API int r_core_rtr_http_stop(RCore *u) {
 	const char *port;
 	RSocket* sock;
 
-#if __WINDOWS__
+#if R2__WINDOWS__
 	r_socket_http_server_set_breaked (&r_cons_context ()->breaked);
 #endif
 	if (((size_t)u) > 0xff) {
@@ -201,7 +201,7 @@ static char *rtr_dir_files(const char *path) {
 	RListIter *iter;
 	// list files
 	RList *files = r_sys_dir (path);
-	eprintf ("Listing directory %s\n", path);
+	R_LOG_INFO ("Listing directory %s", path);
 	r_list_foreach (files, iter, file) {
 		if (file[0] == '.') {
 			continue;
@@ -213,9 +213,9 @@ static char *rtr_dir_files(const char *path) {
 	return r_str_append (ptr, "</body></html>\n");
 }
 
-#if __UNIX__
+#if R2__UNIX__ && !__wasi__
 static void dietime(int sig) {
-	eprintf ("It's Die Time!\n");
+	eprintf ("It's Time To Die!\n");
 	exit (0);
 }
 #endif
@@ -223,7 +223,7 @@ static void dietime(int sig) {
 static void activateDieTime(RCore *core) {
 	int dt = r_config_get_i (core->config, "http.dietime");
 	if (dt > 0) {
-#if __UNIX__ && !__wasi__
+#if R2__UNIX__ && !__wasi__
 		r_sys_signal (SIGALRM, dietime);
 		alarm (dt);
 #else
@@ -232,8 +232,8 @@ static void activateDieTime(RCore *core) {
 	}
 }
 
-#include "rtr_http.c"
-#include "rtr_shell.c"
+#include "rtr_http.inc.c"
+#include "rtr_shell.inc.c"
 
 static int write_reg_val(char *buf, ut64 sz, ut64 reg, int regsize, bool bigendian) {
 	if (!bigendian) {
@@ -248,8 +248,7 @@ static int write_reg_val(char *buf, ut64 sz, ut64 reg, int regsize, bool bigendi
 			reg = r_swap_ut64 (reg);
 			break;
 		default:
-			eprintf ("%s: Unsupported reg size: %d\n",
-				 __func__, regsize);
+			R_LOG_ERROR ("Unsupported reg size: %d", regsize);
 			return -1;
 		}
 	}
@@ -288,8 +287,7 @@ static int write_big_reg(char *buf, ut64 sz, const utX *val, int regsize, bool b
 				 r_swap_ut64 (val->v128.Low),
 				 r_swap_ut64 (val->v128.High));
 	default:
-		eprintf ("%s: big registers (%d byte(s)) not yet supported\n",
-			 __func__, regsize);
+		R_LOG_ERROR ("big registers (%d byte(s)) not yet supported", regsize);
 		return -1;
 	}
 }
@@ -335,8 +333,7 @@ static int swap_big_regs(char *dest, ut64 sz, const char *src, int regsz) {
 		return snprintf (dest, sz, "0x%016"PFMT64x"%016"PFMT64x,
 				 val.v128.High, val.v128.Low);
 	default:
-		eprintf ("%s: big registers (%d byte(s)) not yet supported\n",
-			 __func__, regsz);
+		R_LOG_ERROR ("big registers (%d byte(s)) not yet supported", regsz);
 		return -1;
 	}
 }
@@ -392,26 +389,29 @@ static int r_core_rtr_gdb_cb(libgdbr_t *g, void *core_ptr, const char *cmd,
 			case 't':
 				switch (cmd[3]) {
 				case '\0': // dpt
-					if (!core->dbg->h->threads) {
-						return -1;
-					}
-					if (!(list = core->dbg->h->threads(core->dbg, core->dbg->pid))) {
-						return -1;
-					}
-					memset (out_buf, 0, max_len);
-					out_buf[0] = 'm';
-					ret = 1;
-					r_list_foreach (list, iter, dbgpid) {
-						// Max length of a hex pid = 8?
-						if (ret >= max_len - 9) {
-							break;
+					{
+						RDebugPlugin *plugin = R_UNWRAP3 (core->dbg, current, plugin);
+						if (!plugin || !plugin->threads) {
+							return -1;
+						}
+						if (!(list = plugin->threads (core->dbg, core->dbg->pid))) {
+							return -1;
+						}
+						memset (out_buf, 0, max_len);
+						out_buf[0] = 'm';
+						ret = 1;
+						r_list_foreach (list, iter, dbgpid) {
+							// Max length of a hex pid = 8?
+							if (ret >= max_len - 9) {
+								break;
+							}
+							snprintf (out_buf + ret, max_len - ret - 1, "%x,", dbgpid->pid);
+							ret = strlen (out_buf);
+						}
+						if (ret > 1) {
+							ret--;
+							out_buf[ret] = '\0';
 						}
-						snprintf (out_buf + ret, max_len - ret - 1, "%x,", dbgpid->pid);
-						ret = strlen (out_buf);
-					}
-					if (ret > 1) {
-						ret--;
-						out_buf[ret] = '\0';
 					}
 					return 0;
 				case 'r': // dptr -> return current tid as int
@@ -423,7 +423,7 @@ static int r_core_rtr_gdb_cb(libgdbr_t *g, void *core_ptr, const char *cmd,
 			break;
 		case 'r': // dr
 			r_debug_reg_sync (core->dbg, R_REG_TYPE_ALL, false);
-			be = r_config_get_i (core->config, "cfg.bigendian");
+			be = r_config_get_b (core->config, "cfg.bigendian");
 			if (isspace ((ut8)cmd[2])) { // dr reg
 				const char *name, *val_ptr;
 				char new_cmd[128] = {0};
@@ -547,7 +547,6 @@ static int r_core_rtr_gdb_cb(libgdbr_t *g, void *core_ptr, const char *cmd,
 static int r_core_rtr_gdb_run(RCore *core, int launch, const char *path) {
 	RSocket *sock;
 	int p, ret;
-	bool debug_msg = false;
 	char port[10];
 	char *file = NULL, *args = NULL;
 	libgdbr_t *g;
@@ -555,25 +554,21 @@ static int r_core_rtr_gdb_run(RCore *core, int launch, const char *path) {
 	if (!core || !path) {
 		return -1;
 	}
-	if (*path == '!') {
-		debug_msg = true;
-		path++;
-	}
 	if (!(path = r_str_trim_head_ro (path)) || !*path) {
-		eprintf ("gdbserver: Port not specified\n");
+		R_LOG_ERROR ("gdbserver: Port not specified");
 		return -1;
 	}
 	if (!(p = atoi (path)) || p < 0 || p > 65535) {
-		eprintf ("gdbserver: Invalid port: %d\n", p);
+		R_LOG_ERROR ("gdbserver: Invalid port: %d", p);
 		return -1;
 	}
 	snprintf (port, sizeof (port) - 1, "%d", p);
 	if (!(file = strchr (path, ' '))) {
-		eprintf ("gdbserver: File not specified\n");
+		R_LOG_ERROR ("gdbserver: File not specified");
 		return -1;
 	}
 	if (!(file = (char *)r_str_trim_head_ro (file)) || !*file) {
-		eprintf ("gdbserver: File not specified\n");
+		R_LOG_ERROR ("gdbserver: File not specified");
 		return -1;
 	}
 	args = strchr (file, ' ');
@@ -607,7 +602,6 @@ static int r_core_rtr_gdb_run(RCore *core, int launch, const char *path) {
 		return -1;
 	}
 	gdbr_init (g, true);
-	g->server_debug = debug_msg;
 	int arch = r_sys_arch_id (r_config_get (core->config, "asm.arch"));
 	int bits = r_config_get_i (core->config, "asm.bits");
 	gdbr_set_architecture (g, arch, bits);
@@ -684,7 +678,7 @@ R_API void r_core_rtr_pushout(RCore *core, const char *input) {
 		r_socket_write (rtr_host[rtr_n].fd, str, strlen (str));
 		break;
 	case RTR_PROTOCOL_HTTP:
-		R_LOG_INFO ("TODO");
+		R_LOG_TODO ("RTR_PROTOCOL_HTTP");
 		break;
 	case RTR_PROTOCOL_TCP:
 	case RTR_PROTOCOL_UDP:
@@ -734,12 +728,12 @@ R_API void r_core_rtr_add(RCore *core, const char *_input) {
 			const char *name;
 			int protocol;
 		} uris[7] = {
-			{"tcp", RTR_PROTOCOL_TCP},
-			{"udp", RTR_PROTOCOL_UDP},
-			{"rap", RTR_PROTOCOL_RAP},
-			{"r2p", RTR_PROTOCOL_RAP},
-			{"http", RTR_PROTOCOL_HTTP},
-			{"unix", RTR_PROTOCOL_UNIX},
+			{ "tcp", RTR_PROTOCOL_TCP},
+			{ "udp", RTR_PROTOCOL_UDP},
+			{ "rap", RTR_PROTOCOL_RAP},
+			{ "r2p", RTR_PROTOCOL_RAP},
+			{ "http", RTR_PROTOCOL_HTTP},
+			{ "unix", RTR_PROTOCOL_UNIX},
 			{NULL, 0}
 		};
 		char *s = r_str_ndup (input, pikaboo - input);
@@ -780,7 +774,7 @@ R_API void r_core_rtr_add(RCore *core, const char *_input) {
 	}
 
 	if (r_sandbox_enable (0)) {
-		eprintf ("sandbox: connect disabled\n");
+		R_LOG_ERROR ("sandbox: connect disabled");
 		return;
 	}
 
@@ -812,7 +806,7 @@ R_API void r_core_rtr_add(RCore *core, const char *_input) {
 			return;
 		} else {
 			int n = r_socket_rap_client_open (fd, file, 0);
-			eprintf ("opened as fd = %d\n", n);
+			R_LOG_INFO ("opened as fd = %d", n);
 		}
 		break;
 	case RTR_PROTOCOL_UNIX:
@@ -823,7 +817,7 @@ R_API void r_core_rtr_add(RCore *core, const char *_input) {
 			return;
 		}
 		r_core_return_value (core, R_CMD_RC_SUCCESS);
-		eprintf ("Connected to: 'unix://%s'\n", host);
+		R_LOG_INFO ("Connected to: 'unix://%s'", host);
 		break;
 	case RTR_PROTOCOL_TCP:
 		if (!r_socket_connect_tcp (fd, host, port, timeout)) { //TODO: Use rap.ssl
@@ -833,7 +827,7 @@ R_API void r_core_rtr_add(RCore *core, const char *_input) {
 			return;
 		}
 		r_core_return_value (core, R_CMD_RC_SUCCESS);
-		eprintf ("Connected to: %s at port %s\n", host, port);
+		R_LOG_INFO ("Connected to: %s at port %s", host, port);
 		break;
 	case RTR_PROTOCOL_UDP:
 		if (!r_socket_connect_udp (fd, host, port, timeout)) { //TODO: Use rap.ssl
@@ -843,7 +837,7 @@ R_API void r_core_rtr_add(RCore *core, const char *_input) {
 			return;
 		}
 		r_core_return_value (core, R_CMD_RC_SUCCESS);
-		eprintf ("Connected to: %s at port %s\n", host, port);
+		R_LOG_INFO ("Connected to: %s at port %s", host, port);
 		break;
 	}
 	ret = core->num->value;
@@ -863,7 +857,7 @@ R_API void r_core_rtr_add(RCore *core, const char *_input) {
 		break;
 	}
 	r_core_return_value (core, ret);
-	// double free wtf is freed this here? r_socket_free(fd);
+	// double free wtf is freed this here? r_socket_free (fd);
 	//r_core_rtr_list (core);
 }
 
@@ -907,7 +901,7 @@ R_API void r_core_rtr_event(RCore *core, const char *input) {
 	}
 	if (!strcmp (input, "errmsg")) {
 		// TODO: support udp, tcp, rap, ...
-#if __UNIX__ && !__wasi__
+#if R2__UNIX__ && !__wasi__
 		char *f = r_file_temp ("errmsg");
 		r_cons_printf ("%s\n", f);
 		r_file_rm (f);
@@ -927,7 +921,7 @@ R_API void r_core_rtr_event(RCore *core, const char *input) {
 		// r_core_event (core, );
 		free (s);
 		free (f);
-		// TODO: those files are leaked when closing r_core_free() should be deleted
+		// TODO: those files are leaked when closing r_core_free () should be deleted
 #else
 		R_LOG_ERROR ("Not supported for your platform");
 #endif
@@ -993,8 +987,8 @@ R_API void r_core_rtr_cmd(RCore *core, const char *input) {
 
 	if (*input == '&') { // "=h&" "=&:9090"
 		if (rapthread) {
-			eprintf ("RAP Thread is already running\n");
-			eprintf ("This is experimental and probably buggy. Use at your own risk\n");
+			R_LOG_INFO ("RAP Thread is already running");
+			R_LOG_INFO ("This is experimental and probably buggy. Use at your own risk");
 		} else {
 			// TODO: use tasks
 			RapThread *RT = R_NEW0 (RapThread);
@@ -1003,11 +997,13 @@ R_API void r_core_rtr_cmd(RCore *core, const char *input) {
 				RT->input = strdup (input + 1);
 				//RapThread rt = { core, strdup (input + 1) };
 				rapthread = r_th_new (r_core_rtr_rap_thread, RT, false);
+#if 0
 				int cpuaff = (int)r_config_get_i (core->config, "cfg.cpuaffinity");
 				r_th_setaffinity (rapthread, cpuaff);
+#endif
 				r_th_setname (rapthread, "rapthread");
-				r_th_start (rapthread, false);
-				eprintf ("Background rap server started.\n");
+				r_th_start (rapthread);
+				R_LOG_INFO ("Background rap server started");
 			}
 		}
 		return;
@@ -1151,7 +1147,7 @@ static void rtr_cmds_client_close(uv_tcp_t *client, bool remove) {
 	rtr_cmds_context *context = loop->data;
 	if (remove) {
 		size_t i;
-		for (i = 0; i < r_pvector_len (&context->clients); i++) {
+		for (i = 0; i < r_pvector_length (&context->clients); i++) {
 			if (r_pvector_at (&context->clients, i) == client) {
 				r_pvector_remove_at (&context->clients, i);
 				break;
@@ -1172,11 +1168,9 @@ static void rtr_cmds_alloc_buffer(uv_handle_t *handle, size_t suggested_size, uv
 
 static void rtr_cmds_write(uv_write_t *req, int status) {
 	rtr_cmds_client_context *context = req->data;
-
 	if (status) {
-		eprintf ("Write error: %s\n", uv_strerror (status));
+		R_LOG_ERROR ("Cannot write %s", uv_strerror (status));
 	}
-
 	free (req);
 	rtr_cmds_client_close (context->client, true);
 }
@@ -1211,7 +1205,7 @@ static void rtr_cmds_read(uv_stream_t *client, ssize_t nread, const uv_buf_t *bu
 		client_context->res = strdup ("\n");
 	}
 
-	if (!client_context->res || (!r_config_get_i (client_context->core->config, "scr.prompt") &&
+	if (!client_context->res || (!r_config_get_b (client_context->core->config, "scr.prompt") &&
 				 !strcmp ((char *)buf, "q!")) ||
 				 !strcmp ((char *)buf, ".--")) {
 		rtr_cmds_client_close ((uv_tcp_t *) client, true);
@@ -1229,7 +1223,7 @@ static void rtr_cmds_read(uv_stream_t *client, ssize_t nread, const uv_buf_t *bu
 
 static void rtr_cmds_new_connection(uv_stream_t *server, int status) {
 	if (status < 0) {
-		eprintf ("New connection error: %s\n", uv_strerror (status));
+		R_LOG_ERROR ("New connection failed: %s", uv_strerror (status));
 		return;
 	}
 
@@ -1331,7 +1325,7 @@ beach:
 #else
 
 R_API int r_core_rtr_cmds(RCore *core, const char *port) {
-	unsigned char buf[4097];
+	ut8 buf[4097];
 	RSocket *ch = NULL;
 	int i, ret;
 	char *str;
@@ -1350,7 +1344,7 @@ R_API int r_core_rtr_cmds(RCore *core, const char *port) {
 		return false;
 	}
 
-	eprintf ("Listening for commands on port %s\n", port);
+	R_LOG_INFO ("Listening for commands on port %s", port);
 	listenport = port;
 	r_cons_break_push ((RConsBreak)r_core_rtr_http_stop, core);
 	for (;;) {
@@ -1369,9 +1363,7 @@ R_API int r_core_rtr_cmds(RCore *core, const char *port) {
 					buf[i] = buf[i + 1]? ';': '\0';
 				}
 			}
-			if ((!r_config_get_i (core->config, "scr.prompt") &&
-			     !strcmp ((char *)buf, "q!")) ||
-			    !strcmp ((char *)buf, ".--")) {
+			if ((!r_config_get_b (core->config, "scr.prompt") && !strcmp ((char *)buf, "q!")) || !strcmp ((char *)buf, ".--")) {
 				r_socket_close (ch);
 				break;
 			}
