@@ -1,41 +1,47 @@
-/* radare - LGPL - Copyright 2009-2022 - pancake */
+/* radare - LGPL - Copyright 2009-2023 - pancake */
 
-#include <r_core.h> // just to get the RPrint instance
 #include <r_debug.h>
-#include <r_cons.h>
-#include <r_reg.h>
+#include <r_core.h> // just to get the RPrint instance
 
-R_API bool r_debug_reg_sync(RDebug *dbg, int type, int write) {
-	int i, n, size;
-	if (!dbg || !dbg->reg || !dbg->h) {
-		return false;
+R_API bool r_debug_reg_sync(RDebug *dbg, int type, int must_write) {
+	r_return_val_if_fail (dbg && dbg->reg, false);
+	if (dbg->current == NULL) {
+		return true;
 	}
-	// There's no point in syncing a dead target
+	RDebugPlugin *plugin = R_UNWRAP3 (dbg, current, plugin);
+	if (!plugin) {
+		// if dbg->current is null means that we didnt selected any debug plugin
+		// this function is only needed to sync the local regstate into the target process
+		return true;
+	}
+	int n, size;
 	if (r_debug_is_dead (dbg)) {
 		return false;
 	}
-	// Check if the functions needed are available
-	if (write && !dbg->h->reg_write) {
-		return false;
-	}
-	if (!write && !dbg->h->reg_read) {
-		return false;
+	if (must_write) {
+		if (!plugin->reg_write) {
+			return false;
+		}
+	} else {
+		if (!plugin->reg_read) {
+			return false;
+		}
 	}
 	// Sync all the types sequentially if asked
-	i = (type == R_REG_TYPE_ALL)? R_REG_TYPE_GPR: type;
+	ut32 i = (type == R_REG_TYPE_ALL)? R_REG_TYPE_GPR: type;
 	// Check to get the correct arena when using @ into reg profile (arena!=type)
 	// if request type is positive and the request regset don't have regs
 	if (i >= R_REG_TYPE_GPR && dbg->reg->regset[i].regs && !dbg->reg->regset[i].regs->length) {
 		// seek into the other arena for redirections.
 		for (n = R_REG_TYPE_GPR; n < R_REG_TYPE_LAST; n++) {
 			// get regset mask
-			int mask = dbg->reg->regset[n].maskregstype;
+			const ut32 mask = dbg->reg->regset[n].maskregstype;
 			// convert request arena to mask value
-			int v = ((int)1 << i);
+			const ut32 v = ((ut32)1 << i);
 			// skip checks on same request arena and check if this arena have inside the request arena type
 			if (n != i && (mask & v)) {
-				//eprintf(" req = %i arena = %i mask = %x search = %x \n", i, n, mask, v);
-				//eprintf(" request arena %i found at arena %i\n", i, n );
+				// eprintf(" req = %i arena = %i mask = %x search = %x \n", i, n, mask, v);
+				// eprintf(" request arena %i found at arena %i\n", i, n );
 				// if this arena have the request arena type, force to use this arena.
 				i = n;
 				break;
@@ -43,11 +49,11 @@ R_API bool r_debug_reg_sync(RDebug *dbg, int type, int write) {
 		}
 	}
 	do {
-		if (write) {
+		if (must_write) {
 			ut8 *buf = r_reg_get_bytes (dbg->reg, i, &size);
-			if (!buf || !dbg->h->reg_write (dbg, i, buf, size)) {
+			if (!buf || !plugin->reg_write (dbg, i, buf, size)) {
 				if (i == R_REG_TYPE_GPR) {
-					eprintf ("r_debug_reg: error writing registers %d to %d\n", i, dbg->tid);
+					R_LOG_ERROR ("cannot write registers %d to %d", i, dbg->tid);
 				}
 				if (type != R_REG_TYPE_ALL || i == R_REG_TYPE_GPR) {
 					free (buf);
@@ -56,23 +62,15 @@ R_API bool r_debug_reg_sync(RDebug *dbg, int type, int write) {
 			}
 			free (buf);
 		} else {
-			// int bufsize = R_MAX (1024, dbg->reg->size*2); // i know. its hacky
 			int bufsize = dbg->reg->size;
-			//int bufsize = dbg->reg->regset[i].arena->size;
 			if (bufsize > 0) {
-				ut8 *buf = calloc (1 + 1, bufsize);
-				if (!buf) {
-					return false;
+				ut8 *buf = calloc (2, bufsize);
+				if (buf) {
+					if (plugin->reg_read (dbg, i, buf, bufsize)) {
+						r_reg_set_bytes (dbg->reg, i, buf, bufsize);
+					}
+					free (buf);
 				}
-				//we have already checked dbg->h and dbg->h->reg_read above
-				size = dbg->h->reg_read (dbg, i, buf, bufsize);
-				// we need to check against zero because reg_read can return false
-				if (size > 0) {
-					r_reg_set_bytes (dbg->reg, i, buf, size); //R_MIN (size, bufsize));
-			//		free (buf);
-			//		return true;
-				}
-				free (buf);
 			}
 		}
 		// DO NOT BREAK R_REG_TYPE_ALL PLEASE
@@ -83,6 +81,18 @@ R_API bool r_debug_reg_sync(RDebug *dbg, int type, int write) {
 	return true;
 }
 
+static bool is_mandatory(RRegItem *item, const char *pcname, const char *spname) {
+	r_return_val_if_fail (item, true);
+	// if regname is PC or SP should return false, otherwise return true
+	if (pcname && !strcmp (item->name, pcname)) {
+		return false;
+	}
+	if (spname && !strcmp (item->name, spname)) {
+		return false;
+	}
+	return true;
+}
+
 R_API bool r_debug_reg_list(RDebug *dbg, int type, int size, PJ *pj, int rad, const char *use_color) {
 	r_return_val_if_fail (dbg && dbg->reg, false);
 	int delta, cols, n = 0;
@@ -91,7 +101,6 @@ R_API bool r_debug_reg_list(RDebug *dbg, int type, int size, PJ *pj, int rad, co
 	int colwidth = 20;
 	RListIter *iter;
 	RRegItem *item;
-	RList *head;
 	ut64 diff;
 	char strvalue[256];
 	bool isJson = (rad == 'j' || rad == 'J');
@@ -111,14 +120,12 @@ R_API bool r_debug_reg_list(RDebug *dbg, int type, int size, PJ *pj, int rad, co
 		}
 	}
 	if (dbg->bits & R_SYS_BITS_64) {
-		//fmt = "%s = 0x%08"PFMT64x"%s";
 		fmt = "%s = %s%s";
-		fmt2 = "%s%7s%s %s%s";
+		fmt2 = "%s%6s%s %s%s";
 		kwhites = "         ";
-		colwidth = dbg->regcols? 20: 25;
+		colwidth = dbg->regcols? 30: 25;
 		cols = 3;
 	} else {
-		//fmt = "%s = 0x%08"PFMT64x"%s";
 		fmt = "%s = %s%s";
 		fmt2 = "%s%7s%s %s%s";
 		kwhites = "    ";
@@ -136,21 +143,26 @@ R_API bool r_debug_reg_list(RDebug *dbg, int type, int size, PJ *pj, int rad, co
 
 	int itmidx = -1;
 	dbg->creg = NULL;
-	head = r_reg_get_list (dbg->reg, type);
-	if (!head) {
+	RList *list = r_reg_get_list (dbg->reg, type);
+	if (!list) {
 		return false;
 	}
 	if (rad == 1 || rad == '*') {
 		dbg->cb_printf ("fs+%s\n", R_FLAGS_FS_REGISTERS);
 	}
-	r_list_foreach (head, iter, item) {
+	const char *pcname = r_reg_get_name_by_type (dbg->reg, "PC");
+	const char *spname = r_reg_get_name_by_type (dbg->reg, "SP");
+	bool isfirst = true;
+	r_list_foreach (list, iter, item) {
 		ut64 value;
 		utX valueBig;
-		if (type != -1) {
+		if (type != -1 && is_mandatory (item, pcname, spname)) {
 			if (type != item->type && R_REG_TYPE_FLG != item->type) {
 				continue;
 			}
-			if (size != 0 && size != item->size) {
+			if (size == 8 && item->size == 16) {
+				// avr workaround
+			} else if (size != 0 && size != item->size) {
 				continue;
 			}
 		}
@@ -191,7 +203,7 @@ R_API bool r_debug_reg_list(RDebug *dbg, int type, int size, PJ *pj, int rad, co
 				pj_kn (pj, item->name, value);
 			} else {
 				if (pr && pr->wide_offsets && dbg->bits & R_SYS_BITS_64) {
-					snprintf (strvalue, sizeof (strvalue),"0x%016"PFMT64x, value);
+					snprintf (strvalue, sizeof (strvalue), "0x%016"PFMT64x, value);
 				} else {
 					snprintf (strvalue, sizeof (strvalue),"0x%08"PFMT64x, value);
 				}
@@ -214,6 +226,7 @@ R_API bool r_debug_reg_list(RDebug *dbg, int type, int size, PJ *pj, int rad, co
 				break;
 			default:
 				snprintf (strvalue, sizeof (strvalue), "ERROR");
+				break;
 			}
 			if (isJson) {
 				pj_ks (pj, item->name, strvalue);
@@ -236,12 +249,16 @@ R_API bool r_debug_reg_list(RDebug *dbg, int type, int size, PJ *pj, int rad, co
 		case '*':
 			dbg->cb_printf ("f %s %d %s\n", item->name, item->size / 8, strvalue);
 			break;
+		case 'e':
+			dbg->cb_printf ("%s%s,%s,:=", isfirst?"":",", strvalue, item->name);
+			isfirst = false;
+			break;
 		case '.':
 			dbg->cb_printf ("dr %s=%s\n", item->name, strvalue);
 			break;
 		case '=':
 			{
-				int len, highlight = use_color && pr && pr->cur_enabled && itmidx == pr->cur;
+				bool highlight = (use_color && pr && pr->cur_enabled && itmidx == pr->cur);
 				char whites[32], content[300];
 				const char *a = "", *b = "";
 				if (highlight) {
@@ -255,12 +272,13 @@ R_API bool r_debug_reg_list(RDebug *dbg, int type, int size, PJ *pj, int rad, co
 				}
 				snprintf (content, sizeof (content),
 						fmt2, "", item->name, "", strvalue, "");
-				len = colwidth - strlen (content);
+				int len = colwidth - strlen (content);
 				if (len < 0) {
 					len = 0;
 				}
 				memset (whites, ' ', sizeof (whites));
 				whites[len] = 0;
+
 				dbg->cb_printf (fmt2, a, item->name, b, strvalue,
 						((n+1)%cols)? whites: "\n");
 				if (highlight) {
@@ -291,6 +309,9 @@ R_API bool r_debug_reg_list(RDebug *dbg, int type, int size, PJ *pj, int rad, co
 		}
 		n++;
 	}
+	if (rad == 'e') {
+		dbg->cb_printf ("\n");
+	}
 	if (rad == 1 || rad == '*') {
 		dbg->cb_printf ("fs-\n");
 	}
@@ -305,28 +326,22 @@ beach:
 
 R_API bool r_debug_reg_set(RDebug *dbg, const char *name, ut64 num) {
 	r_return_val_if_fail (dbg && name, false);
-	RRegItem *ri;
 	int role = r_reg_get_name_idx (name);
-	if (!dbg || !dbg->reg) {
+	if (!dbg->reg) {
 		return false;
 	}
 	if (role != -1) {
 		name = r_reg_get_name (dbg->reg, role);
 	}
-	ri = r_reg_get (dbg->reg, name, R_REG_TYPE_ALL);
+	RRegItem *ri = r_reg_get (dbg->reg, name, R_REG_TYPE_ALL);
 	if (ri) {
 		r_reg_set_value (dbg->reg, ri, num);
 		r_debug_reg_sync (dbg, R_REG_TYPE_ALL, true);
+		r_unref (ri);
 	}
 	return (ri);
 }
 
-// XXX deprecate
-R_API ut64 r_debug_reg_get(RDebug *dbg, const char *name) {
-	// ignores errors
-	return r_debug_reg_get_err (dbg, name, NULL, NULL);
-}
-
 R_API ut64 r_debug_reg_get_err(RDebug *dbg, const char *name, int *err, utX *value) {
 	RRegItem *ri = NULL;
 	ut64 ret = 0LL;
@@ -360,6 +375,7 @@ R_API ut64 r_debug_reg_get_err(RDebug *dbg, const char *name, int *err, utX *val
 		} else {
 			ret = r_reg_get_value (dbg->reg, ri);
 		}
+		r_unref (ri);
 	} else {
 		if (err) {
 			*err = 1;
@@ -368,9 +384,7 @@ R_API ut64 r_debug_reg_get_err(RDebug *dbg, const char *name, int *err, utX *val
 	return ret;
 }
 
-// XXX: dup for get_Err!
-R_API ut64 r_debug_num_callback(RNum *userptr, const char *str, int *ok) {
-	RDebug *dbg = (RDebug *)userptr;
-	// resolve using regnu
-	return r_debug_reg_get_err (dbg, str, ok, NULL);
+R_API ut64 r_debug_reg_get(RDebug *dbg, const char *name) {
+	return r_debug_reg_get_err (dbg, name, NULL, NULL);
 }
+
