@@ -1,4 +1,6 @@
-/* radare2 - LGPL - Copyright 2015-2021 - pancake, alvaro_fe */
+/* radare2 - LGPL - Copyright 2015-2023 - pancake, alvaro_fe */
+
+#define R_LOG_ORIGIN "debug.xnu"
 
 #include <r_userconf.h>
 #if DEBUGGER
@@ -8,7 +10,6 @@
 #else
 #define XNU_USE_EXCTHR 1
 #endif
-// ------------------------------------
 
 #include <r_debug.h>
 #include <r_asm.h>
@@ -18,10 +19,29 @@
 #include <string.h>
 #include <mach/mach_host.h>
 #include <mach/host_priv.h>
-#include <mach/mach_vm.h>
 #include <mach/thread_status.h>
 #include <mach/vm_statistics.h>
 
+#if APPLE_SDK_IPHONEOS
+kern_return_t mach_vm_protect
+(
+ vm_map_t target_task,
+ mach_vm_address_t address,
+ mach_vm_size_t size,
+ boolean_t set_maximum,
+ vm_prot_t new_protection
+ );
+kern_return_t mach_vm_read
+(
+ vm_map_t target_task,
+ mach_vm_address_t address,
+ mach_vm_size_t size,
+ vm_offset_t *data,
+ mach_msg_type_number_t *dataCnt
+ );
+#else
+#include <mach/mach_vm.h>
+#endif
 static task_t task_dbg = 0;
 #include "xnu_debug.h"
 #include "xnu_threads.c"
@@ -29,6 +49,76 @@ static task_t task_dbg = 0;
 #include "xnu_excthreads.c"
 #endif
 
+#if defined (__ppc__)
+
+static coredump_thread_state_flavor_t
+thread_flavor_array[] = {
+	{ PPC_THREAD_STATE,	PPC_THREAD_STATE_COUNT },
+	{ PPC_FLOAT_STATE, PPC_FLOAT_STATE_COUNT },
+	{ PPC_EXCEPTION_STATE, PPC_EXCEPTION_STATE_COUNT },
+	{ PPC_VECTOR_STATE,	PPC_VECTOR_STATE_COUNT },
+};
+
+static int coredump_nflavors = 4;
+
+#elif defined (__ppc64__)
+
+coredump_thread_state_flavor_t
+thread_flavor_array[] = {
+	{ PPC_THREAD_STATE64, PPC_THREAD_STATE64_COUNT },
+	{ PPC_FLOAT_STATE, PPC_FLOAT_STATE_COUNT },
+	{ PPC_EXCEPTION_STATE64, PPC_EXCEPTION_STATE64_COUNT },
+	{ PPC_VECTOR_STATE,	PPC_VECTOR_STATE_COUNT },
+};
+
+static int coredump_nflavors = 4;
+
+#elif defined (__i386__)
+
+static coredump_thread_state_flavor_t
+thread_flavor_array[] = {
+	{ x86_THREAD_STATE32, x86_THREAD_STATE32_COUNT },
+	{ x86_FLOAT_STATE32, x86_FLOAT_STATE32_COUNT },
+	{ x86_EXCEPTION_STATE32, x86_EXCEPTION_STATE32_COUNT },
+};
+
+static int coredump_nflavors = 3;
+
+#elif defined (__x86_64__)
+
+static coredump_thread_state_flavor_t
+thread_flavor_array[] = {
+	{ x86_THREAD_STATE64, x86_THREAD_STATE64_COUNT },
+	{ x86_FLOAT_STATE64, x86_FLOAT_STATE64_COUNT },
+	{ x86_EXCEPTION_STATE64, x86_EXCEPTION_STATE64_COUNT },
+};
+
+static int coredump_nflavors = 3;
+
+#elif defined (__aarch64__) || defined (__arm64__) || defined (__arm64e__)
+
+static coredump_thread_state_flavor_t
+thread_flavor_array[] = {
+	{ ARM_UNIFIED_THREAD_STATE, ARM_UNIFIED_THREAD_STATE_COUNT}
+};
+
+static int coredump_nflavors = 1;
+
+#elif defined (__arm__)
+
+static coredump_thread_state_flavor_t
+thread_flavor_array[] = {
+	{ ARM_THREAD_STATE64, ARM_THREAD_STATE64_COUNT }
+};
+
+static int coredump_nflavors = 1;
+
+#else
+// XXX: Add __arm__ for iOS devices?
+#warning Unsupported architecture
+
+#endif
+
 extern int proc_regionfilename(int pid, uint64_t address, void * buffer, uint32_t buffersize);
 
 #define MAX_MACH_HEADER_SIZE (64 * 1024)
@@ -124,7 +214,7 @@ static task_t task_for_pid_workaround(int Pid) {
 	}
 	kr = host_processor_set_priv (myhost, psDefault, &psDefault_control);
 	if (kr != KERN_SUCCESS) {
-		eprintf ("host_processor_set_priv failed for pid %d with error 0x%x\n", Pid, kr);
+		R_LOG_ERROR ("host_processor_set_priv failed for pid %d with error 0x%x", Pid, kr);
 		// mach_error ("host_processor_set_priv",kr);
 		return -1;
 	}
@@ -132,7 +222,7 @@ static task_t task_for_pid_workaround(int Pid) {
 	numTasks = 0;
 	kr = processor_set_tasks (psDefault_control, &tasks, &numTasks);
 	if (kr != KERN_SUCCESS) {
-		eprintf ("processor_set_tasks failed with error %x\n", kr);
+		R_LOG_ERROR ("processor_set_tasks failed with error %x", kr);
 		return -1;
 	}
 	/* kernel task */
@@ -172,14 +262,13 @@ bool xnu_step(RDebug *dbg) {
 	int ret = r_debug_ptrace (dbg, PT_STEP, dbg->pid, (caddr_t)1, 0) == 0; //SIGINT
 	if (!ret) {
 		r_sys_perror ("ptrace-step");
-		eprintf ("mach-error: %d, %s\n", ret, MACH_ERROR_STRING (ret));
+		R_LOG_ERROR ("mach-error: %d, %s", ret, MACH_ERROR_STRING (ret));
 	}
 	return ret;
 #else
-	//we must find a way to get the current thread not just the first one
 	task_t task = pid_to_task (dbg->pid);
 	if (!task) {
-		eprintf ("step failed on task %d for pid %d\n", task, dbg->tid);
+		R_LOG_ERROR ("step failed on task %d for pid %d", task, dbg->tid);
 		return false;
 	}
 	xnu_thread_t *th = get_xnu_thread (dbg, getcurthread (dbg));
@@ -187,7 +276,7 @@ bool xnu_step(RDebug *dbg) {
 		return false;
 	}
 	if (!set_trace_bit (dbg, th)) {
-		eprintf ("xnu_step modificy_trace_bit error\n");
+		R_LOG_ERROR ("xnu_step modificy_trace_bit error");
 		return false;
 	}
 	th->stepping = true;
@@ -224,12 +313,17 @@ bool xnu_detach(RDebug *dbg, int pid) {
 #if XNU_USE_PTRACE
 	return r_debug_ptrace (dbg, PT_DETACH, pid, NULL, 0);
 #else
+	// Not calling PT_DETACH results in a zombie
+	int r = r_debug_ptrace (dbg, PT_DETACH, pid, NULL, 0);
+	if (r < 0) {
+		r_sys_perror ("ptrace(PT_DETACH)");
+	}
 	//do the cleanup necessary
 	//XXX check for errors and ref counts
 	(void)xnu_restore_exception_ports (pid);
 	kern_return_t kr = mach_port_deallocate (mach_task_self (), task_dbg);
 	if (kr != KERN_SUCCESS) {
-		eprintf ("xnu_detach: failed to deallocate port\n");
+		R_LOG_ERROR ("xnu_detach: failed to deallocate port");
 		return false;
 	}
 	//we mark the task as not longer available since we deallocated the ref
@@ -246,7 +340,7 @@ static int task_suspend_count(task_t task) {
 	mach_msg_type_number_t count = TASK_BASIC_INFO_COUNT;
 	kr = task_info (task, TASK_BASIC_INFO, (task_info_t) &info, &count);
 	if (kr != KERN_SUCCESS) {
-		eprintf ("failed to get task info\n");
+		R_LOG_ERROR ("failed to get task info");
 		return -1;
 	}
 	return info.suspend_count;
@@ -254,7 +348,7 @@ static int task_suspend_count(task_t task) {
 
 bool xnu_stop(RDebug *dbg, int pid) {
 #if XNU_USE_PTRACE
-	eprintf ("xnu_stop: not implemented\n");
+	R_LOG_ERROR ("xnu_stop: not implemented for ptrace");
 	return false;
 #else
 	task_t task = pid_to_task (pid);
@@ -277,7 +371,7 @@ bool xnu_stop(RDebug *dbg, int pid) {
 
 	kern_return_t kr = task_suspend (task);
 	if (kr != KERN_SUCCESS) {
-		eprintf ("failed to suspend task\n");
+		R_LOG_ERROR ("failed to suspend task");
 		return false;
 	}
 
@@ -329,12 +423,12 @@ char *xnu_reg_profile(RDebug *dbg) {
 	} else if (dbg->bits == R_SYS_BITS_64) {
 #		include "reg/darwin-x64.h"
 	} else {
-		eprintf ("invalid bit size\n");
+		R_LOG_ERROR ("invalid bit size");
 		return NULL;
 	}
 #elif __POWERPC__
 #	include "reg/darwin-ppc.h"
-#elif __APPLE__ && (__aarch64__ || __arm64__ || __arm__)
+#elif __APPLE__ && (__aarch64__ || __arm64__ || __arm__ || __arm64e__)
 	if (dbg->bits == R_SYS_BITS_64) {
 #		include "reg/darwin-arm64.h"
 	} else {
@@ -348,11 +442,11 @@ char *xnu_reg_profile(RDebug *dbg) {
 //using getcurthread has some drawbacks. You lose the ability to select
 //the thread you want to write or read from. but how that feature
 //is not implemented yet i don't care so much
-int xnu_reg_write(RDebug *dbg, int type, const ut8 *buf, int size) {
+bool xnu_reg_write(RDebug *dbg, int type, const ut8 *buf, int size) {
 	bool ret;
 	xnu_thread_t *th = get_xnu_thread (dbg, getcurthread (dbg));
 	if (!th) {
-		return 0;
+		return false;
 	}
 	switch (type) {
 	case R_REG_TYPE_DRX:
@@ -360,7 +454,7 @@ int xnu_reg_write(RDebug *dbg, int type, const ut8 *buf, int size) {
 		memcpy (&th->drx.uds.ds32, buf, R_MIN (size, sizeof (th->drx)));
 #elif __i386__
 		memcpy (&th->drx.uds.ds64, buf, R_MIN (size, sizeof (th->drx)));
-#elif __arm64 || __aarch64
+#elif __arm64 || __aarch64 || __arm64e
 		if (dbg->bits == R_SYS_BITS_64) {
 			memcpy (&th->debug.drx64, buf, R_MIN (size, sizeof (th->debug.drx64)));
 		} else {
@@ -376,43 +470,53 @@ int xnu_reg_write(RDebug *dbg, int type, const ut8 *buf, int size) {
 #if __POWERPC__
 #warning TODO powerpc support here
 #else
-		memcpy (&th->gpr.uts, buf, R_MIN (size, sizeof (th->gpr.uts)));
+		// memcpy (&th->gpr.uts, buf, R_MIN (size, sizeof (th->gpr.uts)));
+		{
+#if __x86_64__ || __i386__
+		size_t buf_size = R_MIN (size, sizeof (th->gpr));
+		memcpy (&th->gpr.uts, buf, buf_size);
+#else
+		size_t buf_size = R_MIN (size, th->state_size);
+		memcpy (th->state, buf, buf_size);
 #endif
+#endif
+		}
 		ret = xnu_thread_set_gpr (dbg, th);
 		break;
 	}
 	return ret;
 }
 
-int xnu_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
+bool xnu_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 	xnu_thread_t *th = get_xnu_thread (dbg, getcurthread (dbg));
 	if (!th) {
-		return 0;
+		return false;
 	}
 	switch (type) {
 	case R_REG_TYPE_SEG:
 	case R_REG_TYPE_FLG:
 	case R_REG_TYPE_GPR:
 		if (!xnu_thread_get_gpr (dbg, th)) {
-			return 0;
+			return false;
 		}
 		break;
 	case R_REG_TYPE_DRX:
 		if (!xnu_thread_get_drx (dbg, th)) {
-			return 0;
+			return false;
 		}
 		break;
 	default:
-		return 0;
+		return false;
 	}
 	if (th->state) {
 		int rsz = R_MIN (th->state_size, size);
 		if (rsz > 0) {
 			memcpy (buf, th->state, rsz);
-			return rsz;
+			// return rsz;
+			return true;
 		}
 	}
-	return 0;
+	return false;
 }
 
 RDebugMap *xnu_map_alloc(RDebug *dbg, ut64 addr, int size) {
@@ -430,7 +534,7 @@ RDebugMap *xnu_map_alloc(RDebug *dbg, ut64 addr, int size) {
 			  (vm_size_t)size,
 			  anywhere | VM_MAKE_TAG(DEBUG_MAP_TAG_ID));
 	if (ret != KERN_SUCCESS) {
-		eprintf ("vm_allocate failed\n");
+		R_LOG_ERROR ("vm_allocate failed");
 		return NULL;
 	}
 	r_debug_map_sync (dbg); // update process memory maps
@@ -520,7 +624,12 @@ static void xnu_free_threads_ports(RDebugPid *p) {
 */
 
 RList *xnu_thread_list(RDebug *dbg, int pid, RList *list) {
-#if __arm__ || __arm64__ || __aarch_64__
+#if __arm64__ || __aarch_64__ || __arm64e__
+	//#define CPU_PC (dbg->bits == R_SYS_BITS_64) ? \
+	//	state.arm64.__pc : state.arm32.__pc
+	#define CPU_PC (dbg->bits == R_SYS_BITS_64) ? \
+		__darwin_arm_thread_state64_get_pc (state.ts_64) : state.ts_32.__pc
+#elif __arm__ || __arm
 	#define CPU_PC (dbg->bits == R_SYS_BITS_64) ? \
 		state.ts_64.__pc : state.ts_32.__pc
 #elif __POWERPC__
@@ -582,7 +691,7 @@ task_t pid_to_task(int pid) {
 		//since we are going to get a new task
 		kr = mach_port_deallocate (mach_task_self (), task_dbg);
 		if (kr != KERN_SUCCESS) {
-			eprintf ("pid_to_task: fail to deallocate port\n");
+			R_LOG_ERROR ("pid_to_task: fail to deallocate port");
 			/* ignore on purpose to not break process reload: ood */
 			//return 0;
 		}
@@ -668,7 +777,7 @@ static void get_mach_header_sizes(size_t *mach_header_sz, size_t *segment_comman
 // XXX: What about arm?
 }
 
-#if __ppc64__ || __x86_64__|| __i386__ || __ppc__ || __POWERPC__
+#if __ppc64__ || __x86_64__|| __i386__ || __ppc__ || __POWERPC__ || __arm64__ || __arm64e__
 // XXX: This function could use less function calls, but works.
 static cpu_type_t xnu_get_cpu_type(pid_t pid) {
 	int mib[CTL_MAXNAME];
@@ -719,6 +828,15 @@ static void xnu_build_corefile_header(vm_offset_t header, int segment_count, int
 	mh->filetype = MH_CORE;
 	mh->ncmds = segment_count + thread_count;
 	mh->sizeofcmds = command_size;
+#elif __arm64__ || __arm64e__
+	struct mach_header_64 *mh64 = (struct mach_header_64 *)header;
+	mh64->magic = MH_MAGIC_64;
+	mh64->cputype = xnu_get_cpu_type (pid);
+	mh64->cpusubtype = xnu_get_cpu_subtype ();
+	mh64->filetype = MH_CORE;
+	mh64->ncmds = segment_count + thread_count;
+	mh64->sizeofcmds = command_size;
+	mh64->reserved = 0; // 8-byte alignment
 #else
 #warning xnu_build_corefile_header not implemented for this platform
 #endif
@@ -765,7 +883,7 @@ static int xnu_write_mem_maps_to_buffer(RBuffer *buffer, RList *mem_maps, int st
 	struct segment_command *sc;
 #endif
 	r_list_foreach_safe (mem_maps, iter, iter2, curr_map) {
-		eprintf ("Writing section from 0x%"PFMT64x" to 0x%"PFMT64x" (%"PFMT64d")\n",
+		R_LOG_INFO ("Writing section from 0x%"PFMT64x" to 0x%"PFMT64x" (%"PFMT64d")",
 			curr_map->addr, curr_map->addr_end, curr_map->size);
 
 		vm_map_offset_t vmoffset = curr_map->addr;
@@ -824,7 +942,7 @@ static int xnu_write_mem_maps_to_buffer(RBuffer *buffer, RList *mem_maps, int st
 					if (kr > 1) error = -1; // XXX: INVALID_ADDRESS is not a bug right know
 					goto cleanup;
 				}
-#if __ppc64__ || __x86_64__ || __aarch64__ || __arm64__
+#if __ppc64__ || __x86_64__ || __aarch64__ || __arm64__ || __arm64e__
 				rc = r_buf_append_bytes (buffer, (const ut8*)local_address, xfer_size);
 // #elif __i386__ || __ppc__ || __arm__
 #else
@@ -845,6 +963,9 @@ static int xnu_write_mem_maps_to_buffer(RBuffer *buffer, RList *mem_maps, int st
 		foffset += curr_map->size;
 		vmoffset += curr_map->size;
 	}
+	if (foffset > r_buf_size (buffer)) {
+		R_LOG_DEBUG ("Out out band parsing");
+	}
 
 cleanup:
 	*hoffset_out = hoffset;
@@ -867,7 +988,7 @@ static void xnu_collect_thread_state(thread_t port, void *tirp) {
 	header = tir->header;
 	hoffset = tir->hoffset;
 	flavors = tir->flavors;
-	eprintf ("[DEBUG] tc location: 0x%" PFMT64x "\n", hoffset);
+	R_LOG_DEBUG  ("tc location: 0x%" PFMT64x, hoffset);
 
 	tc = (struct thread_command *)(header + hoffset);
 	tc->cmd = LC_THREAD;
@@ -875,7 +996,7 @@ static void xnu_collect_thread_state(thread_t port, void *tirp) {
 	hoffset += sizeof (struct thread_command);
 
 	for (i = 0; i < coredump_nflavors; i++) {
-		eprintf ("[DEBUG] %d/%d\n", i+1, coredump_nflavors);
+		R_LOG_DEBUG ("threadstate %d/%d", i + 1, coredump_nflavors);
 		*(coredump_thread_state_flavor_t *)(header + hoffset) = flavors[i];
 		hoffset += sizeof (coredump_thread_state_flavor_t);
 		xnu_get_thread_status (port, flavors[i].flavor,
@@ -958,7 +1079,7 @@ bool xnu_generate_corefile (RDebug *dbg, RBuffer *dest) {
 
 	for (i = 0; i < coredump_nflavors; i++) {
 		tstate_size += sizeof (coredump_thread_state_flavor_t) +
-			(flavors[i].count * sizeof(int));
+			(flavors[i].count * sizeof (int));
 	}
 
 	command_size = COMMAND_SIZE (segment_count,segment_command_sz,
@@ -973,7 +1094,7 @@ bool xnu_generate_corefile (RDebug *dbg, RBuffer *dest) {
 	}
 	if (xnu_write_mem_maps_to_buffer (mem_maps_buffer, dbg->maps, round_page (header_size),
 		header, mach_header_sz, segment_command_sz, &hoffset) < 0) {
-		eprintf ("There was an error while writing the memory maps");
+		R_LOG_ERROR ("There was an error while writing the memory maps");
 		error = false;
 		goto cleanup;
 	}
@@ -1013,9 +1134,9 @@ RDebugPid *xnu_get_pid (int pid) {
 	/* Get the maximum process arguments size. */
 	mib[0] = CTL_KERN;
 	mib[1] = KERN_ARGMAX;
-	size = sizeof(argmax);
+	size = sizeof (argmax);
 	if (sysctl (mib, 2, &argmax, &size, NULL, 0) == -1) {
-		eprintf ("sysctl() error on getting argmax\n");
+		R_LOG_ERROR ("sysctl() on getting argmax");
 		return NULL;
 	}
 #endif
@@ -1024,7 +1145,7 @@ RDebugPid *xnu_get_pid (int pid) {
 	/* Allocate space for the arguments. */
 	procargs = (char *)malloc (argmax);
 	if (!procargs) {
-		eprintf ("getcmdargs(): insufficient memory for procargs %d\n",
+		R_LOG_ERROR ("getcmdargs(): insufficient memory for procargs %d",
 			(int)(size_t)argmax);
 		return NULL;
 	}
@@ -1040,11 +1161,11 @@ RDebugPid *xnu_get_pid (int pid) {
 	procargs[0] = 0;
 	if (sysctl (mib, 3, procargs, &size, NULL, 0) == -1) {
 		if (EINVAL == errno) { // invalid == access denied for some reason
-			//eprintf("EINVAL returned fetching argument space\n");
+			R_LOG_DEBUG ("EINVAL returned fetching argument space");
 			free (procargs);
 			return NULL;
 		}
-		eprintf ("sysctl(): unspecified sysctl error - %i\n", errno);
+		R_LOG_ERROR ("sysctl(): unspecified sysctl error - %i", errno);
 		free (procargs);
 		return NULL;
 	}
@@ -1054,7 +1175,7 @@ RDebugPid *xnu_get_pid (int pid) {
 	iter_args = procargs + sizeof (nargs);
 	end_args = &procargs[size - 30]; // end of the argument space
 	if (iter_args >= end_args) {
-		eprintf ("getcmdargs(): argument length mismatch");
+		R_LOG_ERROR ("getcmdargs(): argument length mismatch");
 		free (procargs);
 		return NULL;
 	}
@@ -1100,7 +1221,7 @@ RDebugPid *xnu_get_pid (int pid) {
 	 */
 	if (curr_arg == start_args || nargs > 0) {
 		psname[0] = 0;
-//		eprintf ("getcmdargs(): argument parsing failed");
+		R_LOG_DEBUG ("getcmdargs(): argument parsing failed");
 		free (procargs);
 		return NULL;
 	}
@@ -1150,7 +1271,6 @@ vm_address_t get_kernel_base(task_t ___task) {
 	if (ret != KERN_SUCCESS) {
 		return 0;
 	}
-	// eprintf ("%d vs %d\n", task, ___task);
 	for (count = 128; count; count--) {
 		// get next memory region
 		naddr = addr;
@@ -1167,7 +1287,7 @@ vm_address_t get_kernel_base(task_t ___task) {
 			addr += size;
 			continue;
 		}
-		eprintf ("0x%08"PFMT64x" size 0x%08"PFMT64x" perm 0x%x\n",
+		R_LOG_INFO ("0x%08"PFMT64x" size 0x%08"PFMT64x" perm 0x%x",
 			(ut64)addr, (ut64)size, info.max_protection);
 		// the kernel maps over a GB of RAM at the address where it maps
 		// itself so we use that fact to detect it's position
@@ -1178,7 +1298,7 @@ vm_address_t get_kernel_base(task_t ___task) {
 	}
 	ret = mach_port_deallocate (mach_task_self (), 0);
 	if (ret != KERN_SUCCESS) {
-		eprintf ("get_kernel_base: leaking kernel port\n");
+		R_LOG_ERROR ("get_kernel_base: leaking kernel port");
 	}
 	return (vm_address_t)0;
 }
@@ -1246,8 +1366,7 @@ static RList *xnu_dbg_modules(RDebug *dbg) {
 	info_array_size = R_ABS (info_array_size);
 	info_array = calloc (1, info_array_size);
 	if (!info_array) {
-		eprintf ("Cannot allocate info_array_size %d\n",
-			info_array_size);
+		R_LOG_ERROR ("Cannot allocate info_array_size %d", info_array_size);
 		return NULL;
 	}
 
@@ -1273,7 +1392,6 @@ static RList *xnu_dbg_modules(RDebug *dbg) {
 		memset (file_path, 0, MAXPATHLEN);
 		dbg->iob.read_at (dbg->iob.io, file_path_address,
 				(ut8*)file_path, MAXPATHLEN - 1);
-		//eprintf ("--> %d 0x%08"PFMT64x" %s\n", i, addr, file_path);
 		size = mach0_size (dbg, addr);
 		mr = r_debug_map_new (file_path, addr, addr + size, 7, 7);
 		if (!mr) {
@@ -1328,7 +1446,6 @@ RList *xnu_dbg_maps(RDebug *dbg, int only_modules) {
 	//bool contiguous = false;
 	//ut32 oldprot = UT32_MAX;
 	//ut32 oldmaxprot = UT32_MAX;
-	char buf[1024];
 	char module_name[MAXPATHLEN];
 	mach_vm_address_t address = MACH_VM_MIN_ADDRESS;
 	mach_vm_size_t size = (mach_vm_size_t) 0;
@@ -1346,7 +1463,7 @@ RList *xnu_dbg_maps(RDebug *dbg, int only_modules) {
 	if (only_modules) {
 		return modules;
 	}
-#if __arm64__ || __aarch64__
+#if __arm64__ || __aarch64__ || __arm64e__
 	size = osize = 16384;
 #else
 	size = osize = 4096;
@@ -1354,7 +1471,7 @@ RList *xnu_dbg_maps(RDebug *dbg, int only_modules) {
 #if 0
 	if (dbg->pid == 0) {
 		vm_address_t base = get_kernel_base (task);
-		eprintf ("Kernel Base Address: 0x%"PFMT64x"\n", (ut64)base);
+		R_LOG_INFO ("Kernel Base Address: 0x%"PFMT64x, (ut64)base);
 		return NULL;
 	}
 #endif
@@ -1399,16 +1516,18 @@ RList *xnu_dbg_maps(RDebug *dbg, int only_modules) {
 				maxperm[0] = 0;
 			}
 			// XXX: if its shared, it cannot be read?
-			snprintf (buf, sizeof (buf), "%02x_%s%s%s%s%s%s%s%s",
+			char *buf = r_str_newf ("%02x_%s%s%s%s%s%s%s%s",
 				i, unparse_inheritance (info.inheritance),
 				info.user_tag? "_user": "",
 				info.is_submap? "_sub": "",
 				"", info.is_submap ? "_submap": "",
 				module_name, maxperm, depthstr);
 			if (!(mr = r_debug_map_new (buf, address, address + size, xwr2rwx (info.protection), xwr2rwx (info.max_protection)))) {
+				free (buf);
 				R_LOG_ERROR ("Cannot create r_debug_map_new");
 				break;
 			}
+			free (buf);
 			RDebugMap *rdm = moduleAt (modules, address);
 			if (rdm) {
 				mr->file = strdup (rdm->name);
@@ -1431,7 +1550,7 @@ RList *xnu_dbg_maps(RDebug *dbg, int only_modules) {
 			r_list_append (list, mr);
 		}
 		if (size < 1) {
-			eprintf ("size error\n");
+			R_LOG_ERROR ("size error");
 			size = osize;
 		}
 		address += size;
@@ -1450,7 +1569,7 @@ RList *xnu_dbg_maps(RDebug *dbg, int only_modules) {
 				}
 			}
 		}
-		r_list_append (list, m2);	
+		r_list_append (list, m2);
 	}
 	r_list_sort (list, cmp);
  	r_list_free (modules);
