@@ -4,21 +4,14 @@
 
 #define CTX egg->context
 
-static inline int is_var(char *x) {
+static inline bool is_var(char *x) {
 	return x[0] == '.' || ((x[0] == '*' || x[0] == '&') && x[1] == '.');
 }
 
-static inline int is_space(char c) {
+static inline bool is_space(char c) {
 	return c == ' ' || c == '\t' || c == '\n' || c == '\r';
 }
 
-static const char *skipspaces(const char *s) {
-	while (is_space (*s)) {
-		s++;
-	}
-	return s;
-}
-
 static inline int is_op(char x) {
 	switch (x) {
 	case '-':
@@ -36,25 +29,13 @@ static inline int is_op(char x) {
 	}
 }
 
-static inline int get_op(char **pos){
+static inline int get_op(char **pos) {
 	while (**pos && !(is_op (**pos) && !is_var (*pos))) {
 		(*pos)++;
 	}
 	return (**pos)? ((is_op (**pos)) + 1): 0;
 }
 
-/* chop word by space/tab/.. */
-/* NOTE: ensure string does not starts with spaces */
-static char *trim(char *s) {
-	char *o;
-	for (o = s; *o; o++) {
-		if (is_space (*o)) {
-			*o = 0;
-		}
-	}
-	return s;
-}
-
 static void rcc_pushstr(REgg *egg, char *str, int filter);
 static void rcc_context(REgg *egg, int delta);
 
@@ -169,7 +150,7 @@ R_API void r_egg_lang_include_init(REgg *egg) {
 static void rcc_set_callname(REgg *egg, const char *s) {
 	R_FREE (egg->lang.callname);
 	egg->lang.nargs = 0;
-	egg->lang.callname = trim (strdup (skipspaces (s)));
+	egg->lang.callname = r_str_trim_dup (s);
 	egg->lang.pushargs = !((!strcmp (s, "goto")) || (!strcmp (s, "break")));
 }
 
@@ -217,9 +198,7 @@ static void rcc_internal_mathop(REgg *egg, const char *ptr, char *ep, char op) {
 		*q = '\x00';
 	}
 	REggEmit *e = egg->remit;
-	while (*p && is_space (*p)) {
-		p++;
-	}
+	p = (char *)r_str_trim_head_ro (p);
 	if (is_var (p)) {
 		p = r_egg_mkvar (egg, buf, p, 0);
 		if (egg->lang.varxs == '*') {
@@ -257,9 +236,7 @@ static void rcc_mathop(REgg *egg, char **pos, int level) {
 	int op_ret = level;
 	char op, *next_pos;
 
-	while (**pos && is_space (**pos)) {
-		(*pos)++;
-	}
+	*pos = (char *)r_str_trim_head_ro (*pos);
 	next_pos = *pos + 1;
 
 	do {
@@ -289,8 +266,7 @@ static void rcc_mathop(REgg *egg, char **pos, int level) {
 		rcc_internal_mathop (egg, p, strdup (e->regs (egg, op_ret-1)), op);
 		rcc_internal_mathop (egg, (char *)e->regs (egg, op_ret-1),
 				strdup (e->regs (egg, level-1)), '=');
-	}
-	else rcc_internal_mathop(egg, p, strdup (e->regs (egg, level-1)), op);
+	} else { rcc_internal_mathop(egg, p, strdup (e->regs (egg, level-1)), op); }
 */
 }
 
@@ -343,7 +319,7 @@ static void rcc_element(REgg *egg, char *str) {
 					inside ^= 1;
 				} else if (*p == ',' && !inside) {
 					*p = '\0';
-					p = (char *) skipspaces (p + 1);
+					p = (char *) r_str_trim_head_ro (p + 1);
 					rcc_pusharg (egg, p);
 				}
 			}
@@ -353,12 +329,12 @@ static void rcc_element(REgg *egg, char *str) {
 		switch (egg->lang.mode) {
 		case LANG_MODE_ALIAS:
 			if (!egg->lang.dstvar) {
-				eprintf ("does not set name or content for alias\n");
+				R_LOG_ERROR ("does not set name or content for alias");
 				break;
 			}
 			e->equ (egg, egg->lang.dstvar, str);
 			if (egg->lang.nalias > 255) {
-				eprintf ("global-buffer-overflow in aliases\n");
+				R_LOG_ERROR ("global-buffer-overflow in aliases");
 				break;
 			}
 			for (i = 0; i < egg->lang.nalias; i++) {
@@ -377,11 +353,11 @@ static void rcc_element(REgg *egg, char *str) {
 			break;
 		case LANG_MODE_SYSCALL:
 			if (!egg->lang.dstvar) {
-				eprintf ("does not set name or arg for syscall\n");
+				R_LOG_ERROR ("does not set name or arg for syscall");
 				break;
 			}
 			if (egg->lang.nsyscalls > 255) {
-				eprintf ("global-buffer-overflow in syscalls\n");
+				R_LOG_ERROR ("global-buffer-overflow in syscalls");
 				break;
 			}
 			{
@@ -410,18 +386,18 @@ static void rcc_element(REgg *egg, char *str) {
 			e->jmp (egg, egg->lang.elem, 0);
 			break;
 		case LANG_MODE_INCLUDE:
-			str = ptr = (char *) find_alias (egg, skipspaces (str));
+			str = ptr = (char *) find_alias (egg, r_str_trim_head_ro (str));
 			if (ptr) {
 				if (strchr (ptr, '"')) {
 					ptr = strchr (ptr, '"') + 1;
 					if ((p = strchr (ptr, '"'))) {
 						*p = '\x00';
 					} else {
-						eprintf ("loss back quote in include directory\n");
+						R_LOG_ERROR ("loss back quote in include directory");
 					}
 					egg->lang.includedir = strdup (ptr);
 				} else {
-					eprintf ("wrong include syntax\n");
+					R_LOG_ERROR ("wrong include syntax");
 					// for must use string to symbolize directory
 					egg->lang.includedir = NULL;
 				}
@@ -479,7 +455,7 @@ static void rcc_pushstr(REgg *egg, char *str, int filter) {
 				case 'x':
 					ch = r_hex_pair2bin (str + i + 2);
 					if (ch == -1) {
-						eprintf ("%s:%d Error string format\n",
+						R_LOG_ERROR ("%s:%d Error string format",
 							egg->lang.file, egg->lang.line);
 					}
 					str[i] = (char) ch;
@@ -512,8 +488,8 @@ R_API char *r_egg_mkvar(REgg *egg, char *out, const char *_str, int delta) {
 		return NULL;	/* fix segfault, but not badparsing */
 	}
 	/* XXX memory leak */
-	ret = str = oldstr = strdup (skipspaces (_str));
-	// if (num || str[0]=='0') { sprintf(out, "$%d", num); ret = out; }
+	ret = str = oldstr = r_str_trim_dup (_str);
+	// if (num || str[0] == '0') { sprintf(out, "$%d", num); ret = out; }
 	if ((q = strchr (str, ':'))) {
 		*q = '\0';
 		qi = atoi (q + 1);
@@ -561,9 +537,9 @@ R_API char *r_egg_mkvar(REgg *egg, char *out, const char *_str, int delta) {
 							return strdup (r_str_get (egg->lang.syscalls[i].arg));
 						}
 					}
-					eprintf ("Unknown arg for syscall '%s'\n", r_str_get (egg->lang.callname));
+					R_LOG_ERROR ("Unknown arg for syscall '%s'", r_str_get (egg->lang.callname));
 				} else {
-					eprintf ("NO CALLNAME '%s'\n", r_str_get (egg->lang.callname));
+					R_LOG_WARN ("No CallName '%s'", r_str_get (egg->lang.callname));
 				}
 			}
 		} else if (!strncmp (str + 1, "reg", 3)) {
@@ -575,7 +551,7 @@ R_API char *r_egg_mkvar(REgg *egg, char *out, const char *_str, int delta) {
 			}
 		} else {
 			out = str;	/* TODO: show error, invalid var name? */
-			eprintf ("Something is really wrong\n");
+			R_LOG_ERROR ("Something is really wrong in here");
 		}
 		ret = strdup (out);
 		free (oldstr);
@@ -591,7 +567,7 @@ R_API char *r_egg_mkvar(REgg *egg, char *out, const char *_str, int delta) {
 		str[len] = '\0';
 		snprintf (foo, sizeof (foo) - 1, ".fix%d", egg->lang.nargs * 16);	/* XXX FIX DELTA !!!1 */
 		free (egg->lang.dstvar);
-		egg->lang.dstvar = strdup (skipspaces (foo));
+		egg->lang.dstvar = r_str_trim_dup (foo);
 		rcc_pushstr (egg, str, mustfilter);
 		ret = r_egg_mkvar (egg, out, foo, 0);
 		free (oldstr);
@@ -602,22 +578,21 @@ R_API char *r_egg_mkvar(REgg *egg, char *out, const char *_str, int delta) {
 static void rcc_fun(REgg *egg, const char *str) {
 	char *ptr, *ptr2;
 	REggEmit *e = egg->remit;
-	str = skipspaces (str);
+	str = r_str_trim_head_ro (str);
 	if (CTX) {
 		ptr = strchr (str, '=');
 		if (ptr) {
 			*ptr++ = '\0';
 			free (egg->lang.dstvar);
-			egg->lang.dstvar = strdup (skipspaces (str));
-			ptr2 = (char *) skipspaces (ptr);
+			egg->lang.dstvar = r_str_trim_dup (str);
+			ptr2 = (char *) r_str_trim_head_ro (ptr);
 			if (*ptr2) {
-				rcc_set_callname (egg, skipspaces (ptr));
+				rcc_set_callname (egg, ptr2);
 			}
 		} else {
-			str = skipspaces (str);
-			rcc_set_callname (egg, skipspaces (str));
-			egg->remit->comment (egg, "rcc_fun %d (%s)",
-				CTX, egg->lang.callname);
+			str = r_str_trim_head_ro (str);
+			rcc_set_callname (egg, str);
+			egg->remit->comment (egg, "rcc_fun %d (%s)", CTX, egg->lang.callname);
 		}
 	} else {
 		ptr = strchr (str, '@');
@@ -627,14 +602,14 @@ static void rcc_fun(REgg *egg, const char *str) {
 			if (strstr (ptr, "env")) {
 				// eprintf ("SETENV (%s)\n", str);
 				free (egg->lang.setenviron);
-				egg->lang.setenviron = strdup (skipspaces (str));
+				egg->lang.setenviron = r_str_trim_dup (str);
 				egg->lang.slurp = 0;
 			} else if (strstr (ptr, "fastcall")) {
 				/* TODO : not yet implemented */
 			} else if (strstr (ptr, "syscall")) {
 				if (*str) {
 					egg->lang.mode = LANG_MODE_SYSCALL;
-					egg->lang.dstvar = strdup (skipspaces (str));
+					egg->lang.dstvar = r_str_trim_dup (str);
 				} else {
 					egg->lang.mode = LANG_MODE_INLINE;
 					free (egg->lang.syscallbody);
@@ -647,12 +622,12 @@ static void rcc_fun(REgg *egg, const char *str) {
 			} else if (strstr (ptr, "include")) {
 				egg->lang.mode = LANG_MODE_INCLUDE;
 				free (egg->lang.includefile);
-				egg->lang.includefile = strdup (skipspaces (str));
+				egg->lang.includefile = r_str_trim_dup (str);
 				// egg->lang.slurp = 0;
 				// try to deal with alias
 			} else if (strstr (ptr, "alias")) {
 				egg->lang.mode = LANG_MODE_ALIAS;
-				ptr2 = egg->lang.dstvar = strdup (skipspaces (str));
+				ptr2 = egg->lang.dstvar = r_str_trim_dup (str);
 				while (*ptr2 && !is_space (*ptr2)) {
 					ptr2++;
 				}
@@ -661,13 +636,13 @@ static void rcc_fun(REgg *egg, const char *str) {
 			} else if (strstr (ptr, "data")) {
 				egg->lang.mode = LANG_MODE_DATA;
 				egg->lang.ndstval = 0;
-				egg->lang.dstvar = strdup (skipspaces (str));
+				egg->lang.dstvar = r_str_trim_dup (str);
 				egg->lang.dstval = malloc (4096);
 			} else if (strstr (ptr, "naked")) {
 				egg->lang.mode = LANG_MODE_NAKED;
 				/*
 				free (egg->lang.dstvar);
-				egg->lang.dstvar = strdup (skipspaces (str));
+				egg->lang.dstvar = r_str_trim_dup (str);
 				egg->lang.dstval = malloc (4096);
 				egg->lang.ndstval = 0;
 				*/
@@ -675,7 +650,7 @@ static void rcc_fun(REgg *egg, const char *str) {
 			} else if (strstr (ptr, "inline")) {
 				egg->lang.mode = LANG_MODE_INLINE;
 				free (egg->lang.dstvar);
-				egg->lang.dstvar = strdup (skipspaces (str));
+				egg->lang.dstvar = r_str_trim_dup (str);
 				egg->lang.dstval = malloc (4096);
 				egg->lang.ndstval = 0;
 			} else {
@@ -764,7 +739,7 @@ static void rcc_context(REgg *egg, int delta) {
 	} else {
 		/* conditional block */
 // eprintf ("Callname is (%s)\n", callname);
-		const char *elm = skipspaces (egg->lang.elem);
+		const char *elm = r_str_trim_head_ro (egg->lang.elem);
 		// const char *cn = callname;
 		// seems cn is useless in nowadays content
 // if (egg->lang.nested[context-1])
@@ -803,7 +778,7 @@ static void rcc_context(REgg *egg, int delta) {
 			}
 			if (!strcmp (egg->lang.callname, "while")) {
 				char lab[128];
-				sprintf (lab, "__begin_%d_%d_%d", egg->lang.nfunctions,
+				snprintf (lab, sizeof (lab), "__begin_%d_%d_%d", egg->lang.nfunctions,
 					CTX - 1, egg->lang.nestedi[CTX - 1] - 1);
 				// the egg->lang.nestedi[CTX-1] has increased
 				// so we should decrease it in label
@@ -824,7 +799,7 @@ static void rcc_context(REgg *egg, int delta) {
 				// CTX-1, egg->lang.nestedi[CTX-1]);
 				// nestede[CTX-1] = strdup (str);
 				// where give nestede value
-				sprintf (str, "__end_%d_%d_%d", egg->lang.nfunctions, CTX - 1, egg->lang.nestedi[CTX - 1] - 1);
+				snprintf (str, sizeof (str), "__end_%d_%d_%d", egg->lang.nfunctions, CTX - 1, egg->lang.nestedi[CTX - 1] - 1);
 				emit->branch (egg, b, g, e, n, egg->lang.varsize, str);
 				if (CTX > 0) {
 					/* XXX .. */
@@ -926,14 +901,14 @@ static int parseinlinechar(REgg *egg, char c) {
 			if (egg->lang.dstval && egg->lang.dstvar) {
 				egg->lang.dstval[egg->lang.ndstval] = '\0';
 				// printf(" /* END OF LANG_MODE_INLINE (%s)(%s) */\n", egg->lang.dstvar, egg->lang.dstval);
-				egg->lang.inlines[egg->lang.ninlines].name = strdup (skipspaces (egg->lang.dstvar));
-				egg->lang.inlines[egg->lang.ninlines].body = strdup (skipspaces (egg->lang.dstval));
+				egg->lang.inlines[egg->lang.ninlines].name = r_str_trim_dup (egg->lang.dstvar);
+				egg->lang.inlines[egg->lang.ninlines].body = r_str_trim_dup (egg->lang.dstval);
 				egg->lang.ninlines++;
 				R_FREE (egg->lang.dstvar);
 				R_FREE (egg->lang.dstval);
 				return 1;
 			}
-			eprintf ("Parse error\n");
+			R_LOG_ERROR ("Cannot parse expression");
 		}
 	}
 	if (egg->lang.dstval) {
@@ -1005,7 +980,7 @@ static void rcc_next(REgg *egg) {
 			*ptr = '\0';
 			// ocn = ptr+1; // what is the point of this?
 		}
-		ocn = skipspaces (egg->lang.callname);
+		ocn = r_str_trim_head_ro (egg->lang.callname);
 		if (!ocn) {
 			return;
 		}
@@ -1023,11 +998,11 @@ static void rcc_next(REgg *egg) {
 				R_LOG_ERROR ("Unsupported while syntax");
 				return;
 			}
-			sprintf (var, "__begin_%d_%d_%d\n", egg->lang.nfunctions, CTX, egg->lang.nestedi[CTX - 1]);
+			snprintf (var, sizeof (var), "__begin_%d_%d_%d\n", egg->lang.nfunctions, CTX, egg->lang.nestedi[CTX - 1]);
 			e->while_end (egg, var);// get_frame_label (1));
 #if 0
 			eprintf ("------------------------------------------ lastctx: %d\n", egg->lang.lastctxdelta);
-			// TODO: the pushvar is required for the if(){}while(); constructions
+			// TODO: the pushvar is required for the if () {} while (); constructions
 			// char *pushvar = egg->lang.ctxpush[context+egg->lang.nbrackets-1];
 			/* TODO: support to compare more than one expression (LOGICAL OR) */
 			rcc_printf ("  pop %%eax\n");
@@ -1131,7 +1106,7 @@ static void rcc_next(REgg *egg) {
 		int vs = 'l';
 		char type, *eq, *ptr = egg->lang.elem, *tmp;
 		egg->lang.elem[egg->lang.elem_n] = '\0';
-		ptr = (char *) skipspaces (ptr);
+		ptr = (char *) r_str_trim_head_ro (ptr);
 		if (*ptr) {
 			eq = strchr (ptr, '=');
 			if (eq) {
@@ -1139,7 +1114,7 @@ static void rcc_next(REgg *egg) {
 				*buf = *eq = '\x00';
 				e->mathop (egg, '=', vs, '$', "0", e->regs (egg, 1));
 				// avoid situation that egg->lang.mathline starts with a single '-'
-				egg->lang.mathline = strdup ((char *) skipspaces (eq + 1));
+				egg->lang.mathline = r_str_trim_dup (eq + 1);
 				tmp = egg->lang.mathline;
 				rcc_mathop (egg, &tmp, 2);
 				R_FREE (egg->lang.mathline);
@@ -1153,7 +1128,7 @@ static void rcc_next(REgg *egg) {
 						p = q;
 					}
 					if (egg->lang.varxs == '*' || egg->lang.varxs == '&') {
-						eprintf ("not support for *ptr in egg->lang.dstvar\n");
+						R_LOG_ERROR ("not support for *ptr in egg->lang.dstvar");
 					}
 					// XXX: Not support for pointer
 					type = ' ';
@@ -1170,9 +1145,9 @@ static void rcc_next(REgg *egg) {
 				vs = egg->lang.varsize;
 				if (is_var (eq)) {
 					eq = r_egg_mkvar (egg, buf, eq, 0);
-					if (egg->lang.varxs=='*') {
+					if (egg->lang.varxs == '*') {
 						e->load (egg, eq, egg->lang.varsize);
-					} else if (egg->lang.varxs=='&') {
+					} else if (egg->lang.varxs == '&') {
 						// XXX this is a hack .. must be integrated with pusharg
 						e->load_ptr (egg, eq);
 					}
@@ -1188,7 +1163,7 @@ static void rcc_next(REgg *egg) {
 				eprintf("Getting into e->mathop with eq: %s\n", eq);
 				eprintf("Getting into e->mathop with p: %s\n", p);
 				e->mathop (egg, ch, vs, type, eq, p);
-				free(p);
+				free (p);
 #endif
 			} else {
 				if (!strcmp (ptr, "break")) {	// handle 'break;'
@@ -1205,7 +1180,7 @@ static void rcc_next(REgg *egg) {
 
 R_API int r_egg_lang_parsechar(REgg *egg, char c) {
 	REggEmit *e = egg->remit;
-	char *ptr, str[64], *tmp_ptr = NULL;
+	char str[64], *tmp_ptr = NULL;
 	int i, j;
 	if (c == '\n') {
 		egg->lang.line++;
@@ -1266,7 +1241,7 @@ R_API int r_egg_lang_parsechar(REgg *egg, char c) {
 	}
 	if (egg->lang.slurp) {
 		if (egg->lang.slurp != '"' && c == egg->lang.slurpin) {	// only happend when (...(...)...)
-			eprintf ("%s:%d Nesting of expressions not yet supported\n",
+			R_LOG_ERROR ("%s:%d Nesting of expressions not yet supported",
 					egg->lang.file, egg->lang.line);
 			return -1;
 		}
@@ -1298,7 +1273,7 @@ R_API int r_egg_lang_parsechar(REgg *egg, char c) {
 		case '{':
 			if (CTX > 0) {
 				if (CTX > 31 || CTX < 0) {
-					eprintf ("Sinking before overflow\n");
+					R_LOG_ERROR ("Sinking before overflow");
 					CTX = 0;
 					break;
 				}
@@ -1329,19 +1304,13 @@ R_API int r_egg_lang_parsechar(REgg *egg, char c) {
 				if (egg->lang.nested_callname[CTX - 1] && strstr (egg->lang.nested_callname[CTX - 1], "if")) {
 					tmp_ptr = r_str_newf ("__ifelse_%d_%d", CTX - 1, egg->lang.nestedi[CTX - 1] - 1);
 					e->jmp (egg, tmp_ptr, 0);
-					R_FREE (tmp_ptr);	// mem leak
+					R_FREE (tmp_ptr);
 					egg->lang.ifelse_table[CTX - 1][egg->lang.nestedi[CTX - 1] - 1] =
 						r_str_newf ("__end_%d_%d_%d",
 							egg->lang.nfunctions, CTX - 1, egg->lang.nestedi[CTX - 1] - 1);
 				}
-				// if (nestede[CTX]) {
-				// r_egg_printf (egg, "%s:\n", nestede[CTX]);
-				////nestede[CTX] = NULL;
-				// } else {
 				r_egg_printf (egg, "  __end_%d_%d_%d:\n",
 					egg->lang.nfunctions, CTX - 1, egg->lang.nestedi[CTX - 1] - 1);
-				// get_end_frame_label (egg));
-				// }
 			}
 			if (CTX > 0) {
 				egg->lang.nbrackets++;
@@ -1391,7 +1360,7 @@ R_API int r_egg_lang_parsechar(REgg *egg, char c) {
 		}
 		if (egg->lang.slurp) {
 			if (egg->lang.elem_n) {
-				ptr = egg->lang.elem;
+				char *ptr = egg->lang.elem;
 				egg->lang.elem[egg->lang.elem_n] = '\0';
 				while (is_space (*ptr)) {
 					ptr++;
