@@ -1,130 +1,46 @@
-/* radare2 - LGPL - Copyright 2008-2021 - condret, pancake, alvaro_fe */
+/* radare2 - LGPL - Copyright 2008-2023 - condret, pancake, alvaro_fe */
 
 #include <r_io.h>
-#include <sdb.h>
+#include <sdb/sdb.h>
 #include <config.h>
-#include "io_private.h"
 
 R_LIB_VERSION (r_io);
 
-static int fd_read_at_wrap (RIO *io, int fd, ut64 addr, ut8 *buf, int len, RIOMap *map, void *user) {
-	return r_io_fd_read_at (io, fd, addr, buf, len);
-}
-
-static int fd_write_at_wrap (RIO *io, int fd, ut64 addr, ut8 *buf, int len, RIOMap *map, void *user) {
-	return r_io_fd_write_at (io, fd, addr, buf, len);
-}
-
-typedef int (*cbOnIterMap)(RIO *io, int fd, ut64 addr, ut8 *buf, int len, RIOMap *map, void *user);
-
-// If prefix_mode is true, returns the number of bytes of operated prefix; returns < 0 on error.
-// If prefix_mode is false, operates in non-stop mode and returns true iff all IO operations on overlapped maps are complete.
-static st64 on_map_skyline(RIO *io, ut64 vaddr, ut8 *buf, int len, int match_flg, cbOnIterMap op, bool prefix_mode) {
-	RVector *skyline = &io->map_skyline.v;
-	ut64 addr = vaddr;
-	size_t i;
-	bool ret = true, wrap = !prefix_mode && vaddr + len < vaddr;
-#define CMP(addr, part) ((addr) < r_itv_end (((RSkylineItem *)(part))->itv) - 1 ? -1 : \
-			(addr) > r_itv_end (((RSkylineItem *)(part))->itv) - 1 ? 1 : 0)
-	// Let i be the first skyline part whose right endpoint > addr
-	if (!len) {
-		i = r_vector_len (skyline);
-	} else {
-		r_vector_lower_bound (skyline, addr, i, CMP);
-		if (i == r_vector_len (skyline) && wrap) {
-			wrap = false;
-			i = 0;
-			addr = 0;
-		}
-	}
-#undef CMP
-	while (i < r_vector_len (skyline)) {
-		const RSkylineItem *part = r_vector_index_ptr (skyline, i);
-		// Right endpoint <= addr
-		if (r_itv_end (part->itv) - 1 < addr) {
-			i++;
-			if (wrap && i == r_vector_len (skyline)) {
-				wrap = false;
-				i = 0;
-				addr = 0;
-			}
-			continue;
-		}
-		if (addr < part->itv.addr) {
-			// [addr, part->itv.addr) is a gap
-			if (prefix_mode || len <= part->itv.addr - vaddr) {
-				break;
-			}
-			addr = part->itv.addr;
-		}
-		// Now left endpoint <= addr < right endpoint
-		ut64 len1 = R_MIN (vaddr + len - addr, r_itv_end (part->itv) - addr);
-		if (len1 < 1) {
-			break;
-		}
-		RIOMap *map = part->user;
-		// The map satisfies the permission requirement or p_cache is enabled
-		if (((map->perm & match_flg) == match_flg || io->p_cache)) {
-			st64 result = op (io, map->fd, map->delta + addr - r_io_map_begin(map),
-					buf + (addr - vaddr), len1, map, NULL);
-			if (prefix_mode) {
-				if (result < 0) {
-					return result;
-				}
-				addr += result;
-				if (result != len1) {
-					break;
-				}
-			} else {
-				if (result != len1) {
-					ret = false;
-				}
-				addr += len1;
-			}
-		} else if (prefix_mode) {
-			break;
-		} else {
-			addr += len1;
-			ret = false;
-		}
-		// Reaches the end
-		if (addr == vaddr + len) {
-			break;
-		}
-		// Wrap to the beginning of skyline if address wraps
-		if (!addr) {
-			i = 0;
-		}
-	}
-	return prefix_mode ? addr - vaddr : ret;
-}
-
 R_API RIO* r_io_new(void) {
-	return r_io_init (R_NEW0 (RIO));
+	RIO *io = R_NEW0 (RIO);
+	r_io_init (io);
+	return io;
 }
 
-R_API RIO* r_io_init(RIO* io) {
-	r_return_val_if_fail (io, NULL);
+R_API void r_io_init(RIO* io) {
+	r_return_if_fail (io);
 	io->addrbytes = 1;
+	io->overlay = true;
+	io->cb_printf = printf;
 	r_io_desc_init (io);
-	r_skyline_init (&io->map_skyline);
+	r_io_bank_init (io);
 	r_io_map_init (io);
 	r_io_cache_init (io);
 	r_io_plugin_init (io);
 	r_io_undo_init (io);
 	io->event = r_event_new (io);
-	return io;
+	RIOBank *bank = r_io_bank_new ("default");
+	if (bank) {
+		io->bank = bank->id;
+		r_io_bank_add (io, bank);
+	}
 }
 
 R_API void r_io_free(RIO *io) {
 	if (io) {
 		r_io_fini (io);
-		r_cache_free (io->buffer);
 		free (io);
 	}
 }
 
 R_API RIODesc *r_io_open_buffer(RIO *io, RBuffer *b, int perm, int mode) {
+	r_return_val_if_fail (io && b, NULL);
+#if 0
 	ut64 bufSize = r_buf_size (b);
 	char *uri = r_str_newf ("malloc://%" PFMT64d, bufSize);
 	RIODesc *desc = r_io_open_nomap (io, uri, perm, mode);
@@ -134,6 +50,12 @@ R_API RIODesc *r_io_open_buffer(RIO *io, RBuffer *b, int perm, int mode) {
 	}
 	free (uri);
 	return desc;
+#else
+	char *uri = r_str_newf ("rbuf://%p", b);
+	RIODesc *desc = r_io_open_nomap (io, uri, perm, mode);
+	free (uri);
+	return desc;
+#endif
 }
 
 R_API RIODesc *r_io_open_nomap(RIO *io, const char *uri, int perm, int mode) {
@@ -151,7 +73,7 @@ R_API RIODesc* r_io_open(RIO* io, const char* uri, int perm, int mode) {
 	r_return_val_if_fail (io, NULL);
 	RIODesc* desc = r_io_open_nomap (io, uri, perm, mode);
 	if (desc) {
-		r_io_map_new (io, desc->fd, desc->perm, 0LL, 0LL, r_io_desc_size (desc));
+		r_io_map_add (io, desc->fd, desc->perm, 0LL, 0LL, r_io_desc_size (desc));
 	}
 	return desc;
 }
@@ -165,15 +87,16 @@ R_API RIODesc* r_io_open_at(RIO* io, const char* uri, int perm, int mode, ut64 a
 		return NULL;
 	}
 	ut64 size = r_io_desc_size (desc);
+#if 0
 	// second map
 	if (size && ((UT64_MAX - size + 1) < at)) {
 		// split map into 2 maps if only 1 big map results into interger overflow
-		io_map_new (io, desc->fd, desc->perm, UT64_MAX - at + 1, 0LL, size - (UT64_MAX - at) - 1);
+		r_io_map_add (io, desc->fd, desc->perm, UT64_MAX - at + 1, 0LL, size - (UT64_MAX - at) - 1);
 		// someone pls take a look at this confusing stuff
 		size = UT64_MAX - at + 1;
 	}
-	// skyline not updated
-	r_io_map_new (io, desc->fd, desc->perm, 0LL, at, size);
+#endif
+	r_io_map_add (io, desc->fd, desc->perm, 0LL, at, size);
 	return desc;
 }
 
@@ -210,6 +133,7 @@ R_API RList* r_io_open_many(RIO* io, const char* uri, int perm, int mode) {
 	return desc_list;
 }
 
+#if R2__WINDOWS__
 R_API bool r_io_reopen(RIO* io, int fd, int perm, int mode) {
 	RIODesc	*old, *new;
 	char *uri;
@@ -218,8 +142,7 @@ R_API bool r_io_reopen(RIO* io, int fd, int perm, int mode) {
 	}
 	//does this really work, or do we have to handler debuggers ugly
 	uri = old->referer? old->referer: old->uri;
-#if __WINDOWS__ //TODO: workaround, see https://github.com/radareorg/radare2/issues/8840
-	if (old->plugin->close && old->plugin->close (old)) {
+	if (old->plugin->close && !old->plugin->close (old)) {
 		return false; // TODO: this is an unrecoverable scenario
 	}
 	if (!(new = r_io_open_nomap (io, uri, perm, mode))) {
@@ -228,27 +151,37 @@ R_API bool r_io_reopen(RIO* io, int fd, int perm, int mode) {
 	r_io_desc_exchange (io, old->fd, new->fd);
 	r_io_desc_del (io, old->fd);
 	return true;
-#else
-	if (!(new = r_io_open_nomap (io, uri, perm, mode))) {
-		return false;
-	}
-	r_io_desc_exchange (io, old->fd, new->fd);
-	return r_io_desc_close (old); // magic
-#endif // __WINDOWS__
 }
-
-R_API int r_io_close_all(RIO* io) { // what about undo?
-	if (!io) {
+#else
+R_API bool r_io_reopen(RIO* io, int fd, int perm, int mode) {
+	RIODesc *od = r_io_desc_get (io, fd);
+	if (!od) {
 		return false;
 	}
+	const char *uri = od->referer? od->referer: od->uri;
+	RIODesc *nd = r_io_open_nomap (io, uri, perm, mode);
+	if (nd) {
+		r_io_desc_exchange (io, od->fd, nd->fd);
+		r_io_desc_close (od);
+		if (nd->perm & R_PERM_W) {
+			io->coreb.cmdf (io->coreb.core, "omfg");
+		}
+		return true;
+	}
+	R_LOG_ERROR ("Cannot reopen");
+	return false;
+}
+#endif
+
+R_API void r_io_close_all(RIO* io) {
+	r_return_if_fail (io);
 	r_io_desc_fini (io);
 	r_io_map_fini (io);
 	ls_free (io->plugins);
 	r_io_desc_init (io);
 	r_io_map_init (io);
-	r_io_cache_fini (io);
+	r_io_cache_reset (io);
 	r_io_plugin_init (io);
-	return true;
 }
 
 R_API int r_io_pread_at(RIO* io, ut64 paddr, ut8* buf, int len) {
@@ -265,16 +198,46 @@ R_API int r_io_pwrite_at(RIO* io, ut64 paddr, const ut8* buf, int len) {
 }
 
 // Returns true iff all reads on mapped regions are successful and complete.
-R_API bool r_io_vread_at_mapped(RIO* io, ut64 vaddr, ut8* buf, int len) {
+R_API bool r_io_vread_at(RIO *io, ut64 vaddr, ut8* buf, int len) {
 	r_return_val_if_fail (io && buf && len > 0, false);
+	if ((UT64_MAX - (len - 1)) < vaddr) {
+		int _len = UT64_MAX - vaddr + 1;
+		len -= _len;
+		if (!r_io_vread_at (io, 0ULL, &buf[_len], len)) {
+			return false;
+		}
+		len = _len;
+	}
 	if (io->ff) {
 		memset (buf, io->Oxff, len);
 	}
-	return on_map_skyline (io, vaddr, buf, len, R_PERM_R, fd_read_at_wrap, false);
+	return r_io_bank_read_at (io, io->bank, vaddr, buf, len);
 }
 
-static bool r_io_vwrite_at(RIO* io, ut64 vaddr, const ut8* buf, int len) {
-	return on_map_skyline (io, vaddr, (ut8*)buf, len, R_PERM_W, fd_write_at_wrap, false);
+R_API bool r_io_vwrite_at(RIO *io, ut64 vaddr, const ut8 *buf, int len) {
+	r_return_val_if_fail (io && buf && len > 0, false);
+	if ((UT64_MAX - (len - 1)) < vaddr) {
+		int _len = UT64_MAX - vaddr + 1;
+		len -= _len;
+		if (!r_io_vwrite_at (io, 0ULL, &buf[_len], len)) {
+			return false;
+		}
+		len = _len;
+	}
+	return r_io_bank_write_at (io, io->bank, vaddr, buf, len);
+}
+
+R_API bool r_io_vwrite_to_overlay_at(RIO *io, ut64 vaddr, const ut8 *buf, int len) {
+	r_return_val_if_fail (io && buf && len > 0, false);
+	if ((UT64_MAX - (len - 1)) < vaddr) {
+		int _len = UT64_MAX - vaddr + 1;
+		len -= _len;
+		if (!r_io_vwrite_to_overlay_at (io, 0ULL, &buf[_len], len)) {
+			return false;
+		}
+		len = _len;
+	}
+	return r_io_bank_write_to_overlay_at (io, io->bank, vaddr, buf, len);
 }
 
 static bool internal_r_io_read_at(RIO *io, ut64 addr, ut8 *buf, int len) {
@@ -282,15 +245,16 @@ static bool internal_r_io_read_at(RIO *io, ut64 addr, ut8 *buf, int len) {
 		return false;
 	}
 	bool ret = (io->va)
-		? r_io_vread_at_mapped (io, addr, buf, len)
+		? r_io_vread_at (io, addr, buf, len)
 		: r_io_pread_at (io, addr, buf, len) > 0;
-	if (io->cached & R_PERM_R) {
-		(void)r_io_cache_read (io, addr, buf, len);
+	// if (io->cache.mode & R_PERM_X && io->cache.mode & R_PERM_R)
+	// read works even when io.cache=false, but io.cache.read=true
+	if (io->cache.mode & R_PERM_R) {
+		(void)r_io_cache_read_at (io, addr, buf, len);
 	}
 	return ret;
 }
 
-// Deprecated, use either r_io_read_at_mapped or r_io_nread_at instead.
 // For virtual mode, returns true if all reads on mapped regions are successful
 // and complete.
 // For physical mode, the interface is broken because the actual read bytes are
@@ -302,7 +266,6 @@ R_API bool r_io_read_at(RIO *io, ut64 addr, ut8 *buf, int len) {
 	}
 	if (io->mask) {
 		ut64 p = addr;
-		ut8 *b = buf;
 		size_t q = 0;
 		while (q < len) {
 			p &= io->mask;
@@ -315,7 +278,6 @@ R_API bool r_io_read_at(RIO *io, ut64 addr, ut8 *buf, int len) {
 				return false;
 			}
 			q += sz;
-			b += sz;
 			p = 0;
 		}
 		return true;
@@ -323,27 +285,6 @@ R_API bool r_io_read_at(RIO *io, ut64 addr, ut8 *buf, int len) {
 	return internal_r_io_read_at (io, addr, buf, len);
 }
 
-// Returns true iff all reads on mapped regions are successful and complete.
-// Unmapped regions are filled with io->Oxff in both physical and virtual modes.
-// Use this function if you want to ignore gaps or do not care about the number
-// of read bytes.
-R_API bool r_io_read_at_mapped(RIO *io, ut64 addr, ut8 *buf, int len) {
-	bool ret;
-	r_return_val_if_fail (io && buf, false);
-	if (io->ff) {
-		memset (buf, io->Oxff, len);
-	}
-	if (io->va) {
-		ret = on_map_skyline (io, addr, buf, len, R_PERM_R, fd_read_at_wrap, false);
-	} else {
-		ret = r_io_pread_at (io, addr, buf, len) > 0;
-	}
-	if (io->cached & R_PERM_R) {
-		(void)r_io_cache_read(io, addr, buf, len);
-	}
-	return ret;
-}
-
 // For both virtual and physical mode, returns the number of bytes of read
 // prefix.
 // Returns -1 on error.
@@ -357,34 +298,41 @@ R_API int r_io_nread_at(RIO *io, ut64 addr, ut8 *buf, int len) {
 		if (io->ff) {
 			memset (buf, io->Oxff, len);
 		}
-		ret = on_map_skyline (io, addr, buf, len, R_PERM_R, fd_read_at_wrap, true);
+		r_io_bank_drain (io, io->bank);
+		ret = r_io_bank_read_from_submap_at (io, io->bank, addr, buf, len);
 	} else {
 		ret = r_io_pread_at (io, addr, buf, len);
 	}
-	if (ret > 0 && io->cached & R_PERM_R) {
-		(void)r_io_cache_read (io, addr, buf, len);
+	if (ret > 0 && io->cache.mode & R_PERM_R) {
+		(void)r_io_cache_read_at (io, addr, buf, len);
 	}
 	return ret;
 }
 
 R_API bool r_io_write_at(RIO* io, ut64 addr, const ut8* buf, int len) {
-	int i;
+	r_return_val_if_fail (io && buf && len > 0, false);
 	bool ret = false;
 	ut8 *mybuf = (ut8*)buf;
-	r_return_val_if_fail (io && buf && len > 0, false);
 	if (io->write_mask) {
 		mybuf = r_mem_dup ((void*)buf, len);
+		int i;
 		for (i = 0; i < len; i++) {
-			//this sucks
+			// this sucks
 			mybuf[i] &= io->write_mask[i % io->write_mask_len];
 		}
 	}
-	if (io->cached & R_PERM_W) {
-		ret = r_io_cache_write (io, addr, mybuf, len);
-	} else if (io->va) {
-		ret = r_io_vwrite_at (io, addr, mybuf, len);
+	if ((io->cache.mode & R_PERM_X) == R_PERM_X) {
+		if (io->cache.mode & R_PERM_W) {
+			ret = r_io_cache_write_at (io, addr, mybuf, len);
+		} else {
+			R_LOG_ERROR ("enable io.cache.write");
+		}
 	} else {
-		ret = r_io_pwrite_at (io, addr, mybuf, len) > 0;
+		if (io->va) {
+			ret = r_io_vwrite_at (io, addr, mybuf, len);
+		} else {
+			ret = r_io_pwrite_at (io, addr, mybuf, len) > 0; // == len;
+		}
 	}
 	if (buf != mybuf) {
 		free (mybuf);
@@ -393,7 +341,8 @@ R_API bool r_io_write_at(RIO* io, ut64 addr, const ut8* buf, int len) {
 }
 
 R_API bool r_io_read(RIO* io, ut8* buf, int len) {
-	if (io && r_io_read_at (io, io->off, buf, len)) {
+	r_return_val_if_fail (io, false);
+	if (r_io_read_at (io, io->off, buf, len)) {
 		io->off += len;
 		return true;
 	}
@@ -401,7 +350,8 @@ R_API bool r_io_read(RIO* io, ut8* buf, int len) {
 }
 
 R_API bool r_io_write(RIO* io, ut8* buf, int len) {
-	if (io && buf && len > 0 && r_io_write_at (io, io->off, buf, len)) {
+	r_return_val_if_fail (io, false);
+	if (buf && len > 0 && r_io_write_at (io, io->off, buf, len)) {
 		io->off += len;
 		return true;
 	}
@@ -414,52 +364,37 @@ R_API ut64 r_io_size(RIO* io) {
 }
 
 R_API bool r_io_is_listener(RIO* io) {
-	if (io && io->desc && io->desc->plugin && io->desc->plugin->listener) {
+	r_return_val_if_fail (io, false);
+	if (io->desc && io->desc->plugin && io->desc->plugin->listener) {
 		return io->desc->plugin->listener (io->desc);
 	}
 	return false;
 }
 
 R_API char *r_io_system(RIO* io, const char* cmd) {
-	if (io && io->desc && io->desc->plugin && io->desc->plugin->system) {
-		return io->desc->plugin->system (io, io->desc, cmd);
-	}
-	return NULL;
+	r_return_val_if_fail (io && cmd, NULL);
+	return io->desc? r_io_desc_system (io->desc, cmd): NULL;
 }
 
 R_API bool r_io_resize(RIO* io, ut64 newsize) {
-	if (io) {
-		RList *maps = r_io_map_get_by_fd (io, io->desc->fd);
-		RIOMap *current_map;
-		RListIter *iter;
-		ut64 fd_size = r_io_fd_size (io, io->desc->fd);
-		r_list_foreach (maps, iter, current_map) {
-			// we just resize map of the same size of its fd
-			if (r_io_map_size (current_map) == fd_size) {
-				r_io_map_resize (io, current_map->id, newsize);
-			}
-		}
-		r_list_free (maps);
-		return r_io_desc_resize (io->desc, newsize);
-	}
-	return false;
+	r_return_val_if_fail (io, false);
+	return r_io_desc_resize (io->desc, newsize);
 }
 
 R_API bool r_io_close(RIO *io) {
-	return io ? r_io_desc_close (io->desc) : false;
+	r_return_val_if_fail (io, false);
+	return r_io_desc_close (io->desc);
 }
 
-R_API int r_io_extend_at(RIO* io, ut64 addr, ut64 size) {
-	ut64 cur_size, tmp_size;
-	ut8* buffer;
-	if (!io || !io->desc || !io->desc->plugin || !size) {
+R_API bool r_io_extend_at(RIO* io, ut64 addr, ut64 size) {
+	r_return_val_if_fail (io, false);
+	if (!io->desc || !io->desc->plugin || !size) {
 		return false;
 	}
 	if (io->desc->plugin->extend) {
-		int ret;
 		ut64 cur_off = io->off;
 		r_io_seek (io, addr, R_IO_SEEK_SET);
-		ret = r_io_desc_extend (io->desc, size);
+		int ret = r_io_desc_extend (io->desc, size);
 		//no need to seek here
 		io->off = cur_off;
 		return ret;
@@ -467,7 +402,7 @@ R_API int r_io_extend_at(RIO* io, ut64 addr, ut64 size) {
 	if ((io->desc->perm & R_PERM_RW) != R_PERM_RW) {
 		return false;
 	}
-	cur_size = r_io_desc_size (io->desc);
+	ut64 cur_size = r_io_desc_size (io->desc);
 	if (addr > cur_size) {
 		return false;
 	}
@@ -477,10 +412,12 @@ R_API int r_io_extend_at(RIO* io, ut64 addr, ut64 size) {
 	if (!r_io_resize (io, cur_size + size)) {
 		return false;
 	}
-	if ((tmp_size = cur_size - addr) == 0LL) {
+	ut64 tmp_size = cur_size - addr;
+	if (tmp_size == 0LL) {
 		return true;
 	}
-	if (!(buffer = calloc (1, (size_t) tmp_size + 1))) {
+	ut8 *buffer = calloc (1, (size_t) tmp_size + 1);
+	if (!buffer) {
 		return false;
 	}
 	r_io_pread_at (io, addr, buffer, (int) tmp_size);
@@ -496,7 +433,8 @@ R_API int r_io_extend_at(RIO* io, ut64 addr, ut64 size) {
 }
 
 R_API bool r_io_set_write_mask(RIO* io, const ut8* mask, int len) {
-	if (!io || len < 1) {
+	r_return_val_if_fail (io, false);
+	if (len < 1) {
 		return false;
 	}
 	free (io->write_mask);
@@ -512,6 +450,7 @@ R_API bool r_io_set_write_mask(RIO* io, const ut8* mask, int len) {
 }
 
 R_API ut64 r_io_p2v(RIO *io, ut64 pa) {
+	r_return_val_if_fail (io, 0);
 	RIOMap *map = r_io_map_get_paddr (io, pa);
 	if (map) {
 		return pa - map->delta + r_io_map_begin (map);
@@ -520,10 +459,10 @@ R_API ut64 r_io_p2v(RIO *io, ut64 pa) {
 }
 
 R_API ut64 r_io_v2p(RIO *io, ut64 va) {
+	r_return_val_if_fail (io, 0);
 	RIOMap *map = r_io_map_get_at (io, va);
 	if (map) {
-		st64 delta = va - r_io_map_begin (map);
-		return r_io_map_begin (map) + map->delta + delta;
+		return va - r_io_map_begin (map) + map->delta;
 	}
 	return UT64_MAX;
 }
@@ -543,6 +482,7 @@ R_API void r_io_bind(RIO *io, RIOBind *bnd) {
 	bnd->close = r_io_fd_close;
 	bnd->read_at = r_io_read_at;
 	bnd->write_at = r_io_write_at;
+	bnd->overlay_write_at = r_io_vwrite_to_overlay_at;
 	bnd->system = r_io_system;
 	bnd->fd_open = r_io_fd_open;
 	bnd->fd_close = r_io_fd_close;
@@ -558,6 +498,8 @@ R_API void r_io_bind(RIO *io, RIOBind *bnd) {
 	bnd->fd_get_map = r_io_map_get_by_fd;
 	bnd->fd_remap = r_io_map_remap_fd;
 	bnd->is_valid_offset = r_io_is_valid_offset;
+	bnd->bank_get = r_io_bank_get;
+	bnd->map_get = r_io_map_get;
 	bnd->map_get_at = r_io_map_get_at;
 	bnd->map_get_paddr = r_io_map_get_paddr;
 	bnd->addr_is_mapped = r_io_addr_is_mapped;
@@ -570,7 +512,7 @@ R_API void r_io_bind(RIO *io, RIOBind *bnd) {
 
 /* moves bytes up (+) or down (-) within the specified range */
 R_API bool r_io_shift(RIO* io, ut64 start, ut64 end, st64 move) {
-	ut8* buf;
+	r_return_val_if_fail (io && start < end, false);
 	ut64 chunksize = 0x10000;
 	ut64 saved_off = io->off;
 	ut64 src, shiftsize = r_num_abs (move);
@@ -578,7 +520,8 @@ R_API bool r_io_shift(RIO* io, ut64 start, ut64 end, st64 move) {
 		return false;
 	}
 	ut64 rest = (end - start) - shiftsize;
-	if (!(buf = calloc (1, chunksize + 1))) {
+	ut8 *buf = calloc (1, chunksize + 1);
+	if (!buf) {
 		return false;
 	}
 	if (move > 0) {
@@ -605,10 +548,8 @@ R_API bool r_io_shift(RIO* io, ut64 start, ut64 end, st64 move) {
 	return true;
 }
 
-R_API ut64 r_io_seek(RIO* io, ut64 offset, int whence) {
-	if (!io) {
-		return 0LL;
-	}
+R_API ut64 r_io_seek(RIO *io, ut64 offset, int whence) {
+	r_return_val_if_fail (io, 0);
 	switch (whence) {
 	case R_IO_SEEK_SET:
 		io->off = offset;
@@ -624,6 +565,30 @@ R_API ut64 r_io_seek(RIO* io, ut64 offset, int whence) {
 	return io->off;
 }
 
+static bool drain_cb(void *user, void *data, ut32 id) {
+	r_io_map_drain_overlay ((RIOMap *)data);
+	return true;
+}
+
+R_API void r_io_drain_overlay(RIO *io) {
+	r_return_if_fail (io);
+	r_id_storage_foreach (io->maps, drain_cb, NULL);
+}
+
+R_API bool r_io_get_region_at(RIO *io, RIORegion *region, ut64 addr) {
+	r_return_val_if_fail (io && region, false);
+	if (!io->va) {
+		if (io->desc) {
+			region->perm = io->desc->perm;
+			region->itv.addr = 0ULL;
+			region->itv.size = r_io_desc_size (io->desc);
+			return addr < region->itv.size;
+		}
+		return false;
+	}
+	return r_io_bank_get_region_at (io, io->bank, region, addr);
+}
+
 #if HAVE_PTRACE
 
 #if USE_PTRACE_WRAP
@@ -653,13 +618,13 @@ R_API long r_io_ptrace(RIO *io, r_ptrace_request_t request, pid_t pid, void *add
 			errno = 0;
 			return -1;
 		}
-		return ptrace_wrap (wrap, request, pid, addr, data);
+		return ptrace_wrap (wrap, request, pid, addr, (void*)(size_t)data);
 	}
 #endif
-	return ptrace (request, pid, addr, data);
+	return ptrace (request, pid, addr, (size_t)data);
 }
 
-R_API pid_t r_io_ptrace_fork(RIO *io, void (*child_callback)(void *), void *child_callback_user) {
+R_API pid_t r_io_ptrace_fork(RIO *io, void(*child_callback)(void *), void *child_callback_user) {
 #if USE_PTRACE_WRAP
 	if (io->want_ptrace_wrap) {
 		ptrace_wrap_instance *wrap = io_ptrace_wrap_instance (io);
@@ -688,21 +653,16 @@ R_API void *r_io_ptrace_func(RIO *io, void *(*func)(void *), void *user) {
 }
 #endif
 
-
-//remove all descs and maps
-R_API int r_io_fini(RIO* io) {
-	if (!io) {
-		return false;
-	}
+R_API void r_io_fini(RIO* io) {
+	r_return_if_fail (io);
+	r_io_bank_fini (io);
+	r_io_map_fini (io);
 	r_io_desc_cache_fini_all (io);
 	r_io_desc_fini (io);
-	r_io_map_fini (io);
 	ls_free (io->plugins);
 	r_io_cache_fini (io);
 	r_list_free (io->undo.w_list);
-	if (io->runprofile) {
-		R_FREE (io->runprofile);
-	}
+	R_FREE (io->runprofile);
 	r_event_free (io->event);
 #if R_IO_USE_PTRACE_WRAP
 	if (io->ptrace_wrap) {
@@ -710,5 +670,4 @@ R_API int r_io_fini(RIO* io) {
 		free (io->ptrace_wrap);
 	}
 #endif
-	return true;
 }
