@@ -1,4 +1,6 @@
-/* radare - MIT - Copyright 2011-2020 - pancake */
+/* radare - MIT - Copyright 2011-2023 - pancake */
+
+#define R_LOG_ORIGIN "io.rap"
 
 #include <r_io.h>
 #include <r_lib.h>
@@ -29,23 +31,23 @@ static int __rap_read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 	return r_socket_rap_client_read (s, buf, count);
 }
 
-static bool __rap_close(RIODesc *fd) {
-	int ret = false;
-	if (RIORAP_IS_VALID (fd)) {
-		if (RIORAP_FD (fd)) {
-			RIORap *r = fd->data;
-			if (r && fd->fd != -1) {
-				if (r->fd) {
-					(void)r_socket_close (r->fd);
+static bool __rap_close(RIODesc *desc) {
+	bool ret = false;
+	if (RIORAP_IS_VALID (desc)) {
+		if (RIORAP_FD (desc)) {
+			RIORap *rap = desc->data;
+			if (rap && desc->fd != -1) {
+				if (rap->fd) {
+					r_socket_close (rap->fd);
 				}
-				if (r->client) {
-					ret = r_socket_close (r->client) != -1;
+				if (rap->client) {
+					r_socket_close (rap->client);
 				}
-				R_FREE (r);
+				free (rap);
 			}
 		}
 	} else {
-		eprintf ("__rap_close: fdesc is not a r_io_rap plugin\n");
+		R_LOG_ERROR ("fdesc is not a r_io_rap plugin");
 	}
 	return ret;
 }
@@ -60,8 +62,8 @@ static bool __rap_plugin_open(RIO *io, const char *pathname, bool many) {
 }
 
 static RIODesc *__rap_open(RIO *io, const char *pathname, int rw, int mode) {
-	int i, p, listenmode;
-	char *file, *port;
+	int i, listenmode;
+	char *port;
 
 	if (!__rap_plugin_open (io, pathname, 0)) {
 		return NULL;
@@ -77,22 +79,19 @@ static RIODesc *__rap_open(RIO *io, const char *pathname, int rw, int mode) {
 	if (!*port) {
 		return NULL;
 	}
-	p = atoi (port);
-	if ((file = strchr (port + 1, '/'))) {
-		*file = 0;
-		file++;
-	}
+	int p = atoi (port);
+	char *file = r_str_after (port + 1, '/');
 	if (r_sandbox_enable (0)) {
 		R_LOG_ERROR ("sandbox: Cannot use network");
 		return NULL;
 	}
 	if (listenmode) {
 		if (p <= 0) {
-			eprintf ("rap: cannot listen here. Try rap://:9999\n");
+			R_LOG_ERROR ("cannot listen. Try rap://:9999");
 			return NULL;
 		}
-		//TODO: Handle ^C signal (SIGINT, exit); // ???
-		eprintf ("rap: listening at port %s ssl %s\n", port, (is_ssl)?"on":"off");
+		// TODO: Handle ^C signal (SIGINT, exit); // ???
+		R_LOG_INFO ("listening at port %s ssl %s", port, is_ssl? "on": "off");
 		RIORap *rior = R_NEW0 (RIORap);
 		rior->listener = true;
 		rior->client = rior->fd = r_socket_new (is_ssl);
@@ -101,7 +100,7 @@ static RIODesc *__rap_open(RIO *io, const char *pathname, int rw, int mode) {
 			return NULL;
 		}
 		if (is_ssl) {
-			if (file && *file) {
+			if (R_STR_ISNOTEMPTY (file)) {
 				if (!r_socket_listen (rior->fd, port, file)) {
 					r_socket_free (rior->fd);
 					free (rior);
@@ -140,7 +139,7 @@ static RIODesc *__rap_open(RIO *io, const char *pathname, int rw, int mode) {
 	}
 	rior->listener = false;
 	rior->client = rior->fd = s;
-	if (file && *file) {
+	if (R_STR_ISNOTEMPTY (file)) {
 		i = r_socket_rap_client_open (s, file, rw);
 		if (i == -1) {
 			free (rior);
@@ -148,11 +147,11 @@ static RIODesc *__rap_open(RIO *io, const char *pathname, int rw, int mode) {
 			return NULL;
 		}
 		if (i > 0) {
-			eprintf ("rap connection was successful. open %d\n", i);
+			R_LOG_INFO ("rap connection was successful. open %d", i);
 			// io->coreb.cmd (io->coreb.core, "e io.va=0");
-			io->coreb.cmd (io->coreb.core, ".=!i*");
-			io->coreb.cmd (io->coreb.core, ".=!f*");
-			io->coreb.cmd (io->coreb.core, ".=!om*");
+			io->coreb.cmd (io->coreb.core, ".:i*");
+			io->coreb.cmd (io->coreb.core, ".:f*");
+			io->coreb.cmd (io->coreb.core, ".:om*");
 		}
 	}
 	return r_io_desc_new (io, &r_io_plugin_rap,
@@ -176,7 +175,7 @@ static char *__rap_system(RIO *io, RIODesc *fd, const char *command) {
 	buf[0] = RMT_CMD;
 	i = strlen (command) + 1;
 	if (i > RMT_MAX - 5) {
-		eprintf ("Command too long\n");
+		R_LOG_ERROR ("Command too long");
 		return NULL;
 	}
 	r_write_be32 (buf + 1, i);
@@ -209,13 +208,13 @@ static char *__rap_system(RIO *io, RIODesc *fd, const char *command) {
 		}
 		str = calloc (1, cmdlen + 1);
 		ret = r_socket_read_block (s, (ut8*)str, cmdlen);
-		eprintf ("RUN %d CMD(%s)\n", ret, str);
+		R_LOG_INFO ("RUN %d CMD(%s)", ret, str);
 		if (str && *str) {
 			res = io->cb_core_cmdstr (io->user, str);
 		} else {
 			res = strdup ("");
 		}
-		eprintf ("[%s]=>(%s)\n", str, res);
+		R_LOG_INFO ("[%s]=>(%s)", str, res);
 		reslen = strlen (res);
 		free (str);
 		r_write_be32 (buf + 1, reslen);
@@ -231,14 +230,14 @@ static char *__rap_system(RIO *io, RIODesc *fd, const char *command) {
 		return NULL;
 	}
 	if (buf[0] != (RMT_CMD | RMT_REPLY)) {
-		eprintf ("Unexpected rap cmd reply\n");
+		R_LOG_ERROR ("Unexpected rap cmd reply");
 		return NULL;
 	}
 
 	i = r_read_at_be32 (buf, 1);
 	ret = 0;
 	if (i > ST32_MAX) {
-		eprintf ("Invalid length\n");
+		R_LOG_ERROR ("Invalid length");
 		return NULL;
 	}
 	ptr = (char *)calloc (1, i + 1);
@@ -269,15 +268,16 @@ static char *__rap_system(RIO *io, RIODesc *fd, const char *command) {
 	}
 #endif
 #endif
-	
 	return NULL;
 }
 
 RIOPlugin r_io_plugin_rap = {
-	.name = "rap",
-	.desc = "Remote binary protocol plugin",
+	.meta = {
+		.name = "rap",
+		.desc = "Remote binary protocol plugin",
+		.license = "MIT",
+	},
 	.uris = "rap://,raps://",
-	.license = "MIT",
 	.listener = __rap_listener,
 	.open = __rap_open,
 	.close = __rap_close,
