@@ -1,14 +1,8 @@
-/* radare - LPGL - Copyright 2017-2020 condret */
+/* radare - LPGL - Copyright 2017-2022 condret */
 
-#include <r_lib.h>
-#include <r_core.h>
 #include <r_lang.h>
 
-static bool lang_lib_init(RLang *user) {
-	return true;
-}
-
-static bool lang_lib_file_run(RLang *user, const char *file) {
+static bool lang_lib_file_run(RLangSession *user, const char *file) {
 	char *libpath;
 	void *lib;
 	if (!(libpath = r_str_new (file))) {
@@ -25,14 +19,14 @@ static bool lang_lib_file_run(RLang *user, const char *file) {
 	if (!r_file_exists (libpath)) {
 		free (libpath);
 		return false;
-	}	
-	
+	}
+
 	lib = r_lib_dl_open (libpath);
 	if (lib) {
 		void (*fcn)(RCore *);
 		fcn = r_lib_dl_sym (lib, "entry");
 		if (fcn) {
-			fcn (user->user);
+			fcn (user->user_data);
 		} else {
 			R_LOG_ERROR ("Cannot find 'entry' symbol in library");
 		}
@@ -43,10 +37,12 @@ static bool lang_lib_file_run(RLang *user, const char *file) {
 }
 
 static RLangPlugin r_lang_plugin_lib = {
-	.name = "lib",
+	.meta = {
+		.name = "lib",
+		.author = "pancake",
+		.desc = "Load libs directly into r2",
+		.license = "LGPL",
+	},
 	.ext = R_LIB_EXT,
-	.desc = "Load libs directly into r2",
-	.license = "LGPL",
-	.init = lang_lib_init,
 	.run_file = lang_lib_file_run,
 };
