@@ -15,11 +15,10 @@
 
 R_API RRegItem *r_reg_cond_get(RReg *reg, const char *name) {
 	r_return_val_if_fail (reg && name, NULL);
-	int i = R_REG_TYPE_GPR;
 	RListIter *iter;
 	RRegItem *r;
 
-	r_list_foreach (reg->regset[i].regs, iter, r) {
+	r_list_foreach (reg->regset[R_REG_TYPE_GPR].regs, iter, r) {
 		if (r->flags && !strcmp (name, r->flags)) {
 			return r;
 		}
@@ -41,7 +40,7 @@ R_API bool r_reg_cond_set(RReg *r, const char *name, bool val) {
 	return false;
 }
 
-R_API const char *r_reg_cond_to_string(int n) {
+R_API const char *r_reg_cond_tostring(int n) {
 	const char *cs[] = {
 		"eq", "ne", "cf", "neg", "of", "hi", "he",
 		"lo", "loe", "ge", "gt", "lt", "le"
@@ -53,47 +52,22 @@ R_API const char *r_reg_cond_to_string(int n) {
 }
 
 R_API int r_reg_cond_from_string(const char *str) {
-	if (!strcmp (str, "eq")) {
-		return R_REG_COND_EQ;
+#define CK(x,y) ((x) | ((y)<<8))
+	switch (str[0] | (str[1] << 8)) {
+	case CK('e','q'): return R_REG_COND_EQ;
+	case CK('n','e'): return strcmp (str, "neg")? R_REG_COND_NE: R_REG_COND_NEG;
+	case CK('c','f'): return R_REG_COND_CF;
+	case CK('o','f'): return R_REG_COND_OF;
+	case CK('h','i'): return R_REG_COND_HI;
+	case CK('h','e'): return R_REG_COND_HE;
+	case CK('l','o'): return strcmp (str, "loe")? R_REG_COND_LO: R_REG_COND_LOE;
+	case CK('g','e'): return R_REG_COND_GE;
+	case CK('g','t'): return R_REG_COND_GT;
+	case CK('l','t'): return R_REG_COND_LT;
+	case CK('l','e'): return R_REG_COND_LE;
 	}
-	if (!strcmp (str, "ne")) {
-		return R_REG_COND_NE;
-	}
-	if (!strcmp (str, "cf")) {
-		return R_REG_COND_CF;
-	}
-	if (!strcmp (str, "neg")) {
-		return R_REG_COND_NEG;
-	}
-	if (!strcmp (str, "of")) {
-		return R_REG_COND_OF;
-	}
-	if (!strcmp (str, "hi")) {
-		return R_REG_COND_HI;
-	}
-	if (!strcmp (str, "he")) {
-		return R_REG_COND_HE;
-	}
-	if (!strcmp (str, "lo")) {
-		return R_REG_COND_LO;
-	}
-	if (!strcmp (str, "loe")) {
-		return R_REG_COND_LOE;
-	}
-	if (!strcmp (str, "ge")) {
-		return R_REG_COND_GE;
-	}
-	if (!strcmp (str, "gt")) {
-		return R_REG_COND_GT;
-	}
-	if (!strcmp (str, "lt")) {
-		return R_REG_COND_LT;
-	}
-	if (!strcmp (str, "le")) {
-		return R_REG_COND_LE;
-	}
-	// TODO: move this into core
-	eprintf ("| Usage: drc[=] [condition](=1,0)\n"
+	// TODO: move this help message into the core
+	eprintf ("Usage: drc[=] [condition](=1,0)\n"
 		 "| eq    equal\n"
 		 "| ne    not equal\n"
 		 "| cf    carry flag set\n"
@@ -112,8 +86,7 @@ R_API int r_reg_cond_from_string(const char *str) {
 	return -1;
 }
 
-// R2_580: R_API bool r_reg_cond_bits(RReg *r, int type, RRegFlags *f) {
-R_API int r_reg_cond_bits(RReg *r, int type, RRegFlags *f) {
+R_API bool r_reg_cond_bits(RReg *r, int type, RRegFlags *f) {
 	switch (type) {
 	case R_REG_COND_EQ: return Z;
 	case R_REG_COND_NE: return !Z;
@@ -224,8 +197,7 @@ R_API bool r_reg_cond_bits_set(RReg *r, int type, RRegFlags *f, bool v) {
 	return true;
 }
 
-// R2_580: R_API bool r_reg_cond(RReg *r, int type) {
-R_API int r_reg_cond(RReg *r, int type) {
+R_API bool r_reg_cond(RReg *r, int type) {
 	r_return_val_if_fail (r, false);
 	RRegFlags f = {0};
 	r_reg_cond_retrieve (r, &f);
