@@ -1,6 +1,7 @@
-/* radare - LGPL - Copyright 2007-2022 - pancake */
+/* radare - LGPL - Copyright 2007-2024 - pancake */
 
 #include <r_bin.h>
+#include <r_userconf.h>
 
 /* stable code */
 static const char * const rwxstr[] = {
@@ -23,7 +24,23 @@ static const char * const rwxstr[] = {
 	[15] = "rwx",
 };
 
+// equal string, same case
+R_API bool r_str_eq(const char *s1, const char *s2) {
+	return s1 && s2 && !strcmp (s1, s2);
+}
+
+// equal string, ignoring case
+R_API bool r_str_eqi(const char *s1, const char *s2) {
+	return s1 && s2 && !r_str_casecmp (s1, s2);
+}
+
 R_API int r_str_casecmp(const char *s1, const char *s2) {
+	if (!s1) {
+		return -1;
+	}
+	if (R_UNLIKELY (!s2)) {
+		return 1;
+	}
 #ifdef _MSC_VER
 	return stricmp (s1, s2);
 #else
@@ -100,7 +117,7 @@ R_API void r_str_reverse(char *str) {
 R_API int r_str_bits(char *strout, const ut8 *buf, int len, const char *bitz) {
 	int i, j, idx;
 	if (bitz) {
-		for (i = j = 0; i<len && (!bitz||bitz[i]); i++) {
+		for (i = j = 0; i < len && (!bitz||bitz[i]); i++) {
 			if (i > 0 && (i % 8) == 0) {
 				buf++;
 			}
@@ -121,8 +138,11 @@ R_API int r_str_bits(char *strout, const ut8 *buf, int len, const char *bitz) {
 
 R_API const char *r_str_sysbits(const int v) {
 	switch (v) {
+	case R_SYS_BITS_4: return "4";
 	case R_SYS_BITS_8: return "8";
+	case R_SYS_BITS_4 | R_SYS_BITS_8: return "4,8";
 	case R_SYS_BITS_16: return "16";
+	case R_SYS_BITS_27: return "27";
 	case R_SYS_BITS_32: return "32";
 	case R_SYS_BITS_64: return "64";
 	case R_SYS_BITS_16 | R_SYS_BITS_32: return "16,32";
@@ -148,14 +168,9 @@ static void trimbits(char *b) {
 // The string is then trimmed using the "trimbits" function above.
 R_API int r_str_bits64(char* strout, ut64 in) {
 	int i, bit, count = 0;
-	count = 0;
 	for (i = (sizeof (in) * 8) - 1; i >= 0; i--) {
 		bit = in >> i;
-		if (bit & 1) {
-			strout[count] = '1';
-		} else {
-			strout[count] = '0';
-		}
+		strout[count] = (bit & 1)? '1': '0';
 		count++;
 	}
 	strout[count] = '\0';
@@ -169,12 +184,13 @@ R_API int r_str_bits64(char* strout, ut64 in) {
  *
  */
 R_API ut64 r_str_bits_from_string(const char *buf, const char *bitz) {
+	R_RETURN_VAL_IF_FAIL (buf && bitz, UT64_MAX);
 	ut64 out = 0LL;
 	/* return the numeric value associated to a string (rflags) */
 	for (; *buf; buf++) {
-		char *ch = strchr (bitz, toupper ((const unsigned char)*buf));
+		char *ch = strchr (bitz, toupper ((int)(ut8)*buf));
 		if (!ch) {
-			ch = strchr (bitz, tolower ((const unsigned char)*buf));
+			ch = strchr (bitz, tolower ((int)(ut8)*buf));
 		}
 		if (ch) {
 			int bit = (int)(size_t)(ch - bitz);
@@ -186,47 +202,19 @@ R_API ut64 r_str_bits_from_string(const char *buf, const char *bitz) {
 	return out;
 }
 
-R_API int r_str_binstr2bin(const char *str, ut8 *out, int outlen) {
-	int n, i, j, k, ret, len;
-	len = strlen (str);
-	for (n = i = 0; i < len; i += 8) {
-		ret = 0;
-		while (str[i]==' ') {
-			str++;
-		}
-		if (i + 7 < len) {
-			for (k = 0, j = i + 7; j >= i; j--, k++) {
-				// INVERSE for (k=0,j=i; j<i+8; j++,k++) {
-				if (str[j] == ' ') {
-					//k--;
-					continue;
-				}
-				//		printf ("---> j=%d (%c) (%02x)\n", j, str[j], str[j]);
-				if (str[j] == '1') {
-					ret|=1 << k;
-				} else if (str[j] != '0') {
-					return n;
-				}
-			}
-		}
-	//	printf ("-======> %02x\n", ret);
-		out[n++] = ret;
-		if (n == outlen) {
-			return n;
-		}
-	}
-	return n;
-}
-
 // Returns the permissions as in integer given an input in the form of rwx, rx,
 // etc.
 R_API int r_str_rwx(const char *str) {
+	R_RETURN_VAL_IF_FAIL (str, -1);
 	int ret = atoi (str);
 	if (!ret) {
-		ret |= strchr (str, 'm') ? 16 : 0;
-		ret |= strchr (str, 'r') ? 4 : 0;
-		ret |= strchr (str, 'w') ? 2 : 0;
+		ret |= strchr (str, 'm')? 16: 0;
+		ret |= strchr (str, 'r')? 4: 0;
+		ret |= strchr (str, 'w')? 2: 0;
 		ret |= strchr (str, 'x') ? 1 : 0;
+		if (*str && ret == 0 && strchr (str, '-')) {
+			ret = -1;
+		}
 	} else if (ret < 0 || ret >= R_ARRAY_SIZE (rwxstr)) {
 		ret = 0;
 	}
@@ -247,7 +235,7 @@ R_API void r_str_case(char *str, bool up) {
 	if (up) {
 		char oc = 0;
 		for (; *str; oc = *str++) {
-			*str = (*str=='x' && oc=='0') ? 'x': toupper ((int)(ut8)*str);
+			*str = (*str == 'x' && oc == '0') ? 'x': toupper ((int)(ut8)*str);
 		}
 	} else {
 		for (; *str; str++) {
@@ -256,34 +244,6 @@ R_API void r_str_case(char *str, bool up) {
 	}
 }
 
-R_API R_MUSTUSE char *r_str_home(const char *str) {
-	char *dst, *home = r_sys_getenv (R_SYS_HOME);
-	size_t length;
-	if (!home) {
-		home = r_file_tmpdir ();
-		if (!home) {
-			return NULL;
-		}
-	}
-	length = strlen (home) + 1;
-	if (R_STR_ISNOTEMPTY (str)) {
-		length += strlen (R_SYS_DIR) + strlen (str);
-	}
-	dst = (char *)calloc (1, length);
-	if (!dst) {
-		goto fail;
-	}
-	int home_len = strlen (home);
-	memcpy (dst, home, home_len + 1);
-	if (R_STR_ISNOTEMPTY (str)) {
-		dst[home_len] = R_SYS_DIR[0];
-		strcpy (dst + home_len + 1, str);
-	}
-fail:
-	free (home);
-	return dst;
-}
-
 R_API R_MUSTUSE char *r_str_r2_prefix(const char *str) {
 	return r_str_newf ("%s%s%s", r_sys_prefix (NULL), R_SYS_DIR, str);
 }
@@ -318,7 +278,7 @@ R_API int r_str_delta(char *p, char a, char b) {
 R_API int r_str_split(char *str, char ch) {
 	int i;
 	char *p;
-	if (!str || !*str) {
+	if (R_STR_ISEMPTY (str)) {
 		return 0;
 	}
 	/* TODO: sync with r1 code */
@@ -342,7 +302,7 @@ R_API int r_str_word_set0(char *str) {
 		return 0;
 	}
 	for (i = 0; str[i] && str[i + 1]; i++) {
-		if (i > 0 && str[i-1] == ' ' && str[i] == ' ') {
+		if (i > 0 && str[i - 1] == ' ' && str[i] == ' ') {
 			int len = strlen (str + i);
 			memmove (str + i, str + i + 1, len);
 			i--;
@@ -523,14 +483,15 @@ R_API const char *r_str_word_get0(const char *str, int idx) {
 }
 
 // Return the number of times that the character ch appears in the string.
-R_API int r_str_char_count(const char *string, char ch) {
-	int i, count = 0;
+R_API size_t r_str_char_count(const char *string, char ch) {
+	R_RETURN_VAL_IF_FAIL (string, 0);
+	size_t i, count = 0;
 	for (i = 0; string[i]; i++) {
 		if (string[i] == ch) {
 			count++;
 		}
 	}
-	return R_MAX (0, count);
+	return count;
 }
 
 // Counts the number of words (separated by separator characters: newlines, tabs,
@@ -595,6 +556,20 @@ R_API const char *r_sub_str_rchr(const char *str, int start, int end, char chr)
 	return str[start] == chr ? str + start : NULL;
 }
 
+R_API const char *r_str_nchr(const char *str, char chr, int maxlen) {
+	int count = 0;
+	while (*str) {
+		if (maxlen > 0 && count++ >= maxlen) {
+			return NULL;
+		}
+		if (*str == chr) {
+			return str;
+		}
+		str++;
+	}
+	return *str? str: NULL;
+}
+
 R_API const char *r_str_sep(const char *base, const char *sep) {
 	int i;
 	while (*base) {
@@ -671,7 +646,7 @@ R_API const char *r_str_nstr(const char *s, const char *find, int slen) {
 }
 
 // Returns a new heap-allocated copy of str.
-// XXX what's the diff with r_str_dup ?
+// XXX what's the diff with strdup ?
 R_API char *r_str_new(const char *str) {
 	return str? strdup (str): NULL;
 }
@@ -722,8 +697,9 @@ R_API char *r_str_newf(const char *fmt, ...) {
 
 	va_start (ap, fmt);
 	if (!strchr (fmt, '%')) {
+		char *p = strdup (fmt);
 		va_end (ap);
-		return strdup (fmt);
+		return p;
 	}
 	va_copy (ap2, ap);
 	int ret = vsnprintf (NULL, 0, fmt, ap2);
@@ -737,6 +713,7 @@ R_API char *r_str_newf(const char *fmt, ...) {
 	return p;
 }
 
+#if 0
 // Secure string copy with null terminator (like strlcpy or strscpy but ours
 R_API size_t r_str_ncpy(char *dst, const char *src, size_t n) {
 	size_t i;
@@ -753,8 +730,9 @@ R_API size_t r_str_ncpy(char *dst, const char *src, size_t n) {
 	dst[i] = 0;
 	return i;
 }
+#endif
 
-/* memccmp("foo.bar", "foo.cow, '.') == 0 */
+/* memccmp ("foo.bar", "foo.cow, '.') == 0 */
 // Returns 1 if src and dst are equal up until the first instance of ch in src.
 R_API bool r_str_ccmp(const char *dst, const char *src, int ch) {
 	int i;
@@ -787,33 +765,6 @@ R_API bool r_str_cmp_list(const char *list, const char *item, char sep) {
 	return true;
 }
 
-// like strncmp, but checking for null pointers
-R_API int r_str_cmp(const char *a, const char *b, int len) {
-	if ((a == b) || (!a && !b)) {
-		return 0;
-	}
-	if (!a && b) {
-		return -1;
-	}
-	if (a && !b) {
-		return 1;
-	}
-	if (len < 0) {
-		return strcmp (a, b);
-	}
-	return strncmp (a, b, len);
-}
-
-// Copies all characters from src to dst up until the character 'ch'.
-R_API int r_str_ccpy(char *dst, char *src, int ch) {
-	int i;
-	for (i = 0; src[i] && src[i] != ch; i++) {
-		dst[i] = src[i];
-	}
-	dst[i] = '\0';
-	return i;
-}
-
 R_API char *r_str_word_get_first(const char *text) {
 	for (; *text && IS_SEPARATOR (*text); text++) {
 		;
@@ -848,13 +799,6 @@ R_API char *r_str_ndup(const char *ptr, int len) {
 	return out;
 }
 
-// TODO: deprecate?
-R_API char *r_str_dup(char *ptr, const char *string) {
-	char *str = r_str_new (string);
-	free (ptr); // in case ptr == string
-	return str;
-}
-
 R_API char *r_str_prepend(char *ptr, const char *string) {
 	int slen, plen;
 	if (!ptr) {
@@ -872,6 +816,7 @@ R_API char *r_str_prepend(char *ptr, const char *string) {
 }
 
 R_API char *r_str_appendlen(char *ptr, const char *string, int slen) {
+	r_return_val_if_fail (string, NULL);
 	char *msg = r_str_newlen (string, slen);
 	char *ret = r_str_append (ptr, msg);
 	free (msg);
@@ -936,6 +881,25 @@ R_API char *r_str_appendch(char *x, char y) {
 	return r_str_append (x, b);
 }
 
+R_API R_MUSTUSE char* r_str_replace_all(char *str, const char *key, const char *val) {
+	r_return_val_if_fail (str && key, NULL);
+	if (strstr (val, key)) {
+		// XXX value cant contain the key otherwise we go into infinite loop
+		R_LOG_ERROR ("RStr.replaceAll() value can't contain key");
+		free (str);
+		return NULL;
+	}
+	char *res = str;
+	while (strstr (str, key)) {
+		res = r_str_replace (str, key, val, true);
+		if (!res) {
+			return NULL;
+		}
+		str = res;
+	}
+	return res;
+}
+
 R_API R_MUSTUSE char* r_str_replace(char *str, const char *key, const char *val, int g) {
 	if (g == 'i') {
 		return r_str_replace_icase (str, key, val, g, true);
@@ -990,26 +954,24 @@ R_API R_MUSTUSE char* r_str_replace(char *str, const char *key, const char *val,
 
 R_API R_MUSTUSE char *r_str_replace_icase(char *str, const char *key, const char *val, int g, int keep_case) {
 	r_return_val_if_fail (str && key && val, NULL);
-
-	int off, i, klen, vlen, slen;
 	char *newstr, *p = str;
-	klen = strlen (key);
-	vlen = strlen (val);
-
-	slen = strlen (str);
+	size_t off, i;
+	size_t klen = strlen (key);
+	size_t vlen = strlen (val);
+	size_t slen = strlen (str);
 	for (i = 0; i < slen;) {
 		p = (char *)r_str_casestr (str + i, key);
 		if (!p) {
 			break;
 		}
-		off = (int)(size_t) (p - str);
+		off = (size_t) (p - str);
 		if (vlen != klen) {
 			int tlen = slen - (off + klen);
 			slen += vlen - klen;
 			if (vlen > klen) {
 				newstr = realloc (str, slen + 1);
 				if (!newstr) {
-					goto alloc_fail;
+					return NULL;
 				}
 				str = newstr;
 			}
@@ -1023,12 +985,12 @@ R_API R_MUSTUSE char *r_str_replace_icase(char *str, const char *key, const char
 			if (!tmp_val || !str_case) {
 				free (tmp_val);
 				free (str_case);
-				goto alloc_fail;
+				return NULL;
 			}
 			tmp_val = r_str_replace_icase (tmp_val, key, str_case, 0, 0);
 			free (str_case);
 			if (!tmp_val) {
-				goto alloc_fail;
+				return NULL;
 			}
 			memcpy (p, tmp_val, vlen);
 			free (tmp_val);
@@ -1042,9 +1004,6 @@ R_API R_MUSTUSE char *r_str_replace_icase(char *str, const char *key, const char
 		}
 	}
 	return str;
-alloc_fail:
-	free (str);
-	return NULL;
 }
 
 /* replace the key in str with val.
@@ -1112,15 +1071,16 @@ R_API R_MUSTUSE char* r_str_replace_thunked(char *str, char *clean, int *thunk,
 	return str;
 }
 
-// R580 - return void to avoid confusion imho
-R_API char *r_str_replace_in(char *str, ut32 sz, const char *key, const char *val, int g) {
-	r_return_val_if_fail (str && key && val, NULL);
+R_API void r_str_replace_in(char *str, ut32 sz, const char *key, const char *val, int g) {
+	r_return_if_fail (str && key && val);
 	char *heaped = r_str_replace (strdup (str), key, val, g);
 	if (heaped) {
-		strncpy (str, heaped, sz);
+		if (sz < 0) {
+			sz = strlen (key);
+		}
+		r_str_ncpy (str, heaped, sz);
 		free (heaped);
 	}
-	return str;
 }
 
 R_API int r_str_unescape(char *buf) {
@@ -1250,8 +1210,7 @@ R_API char *r_str_sanitize_sdb_key(const char *s) {
 	char *cur = ret;
 	while (len > 0) {
 		char c = *s;
-		if (!(c >= 'a' && c <= 'z') && !(c >= 'A' && c <= 'Z') && !(c >= '0' && c <= '9')
-			&& c != '_' && c != ':') {
+		if (!(c >= 'a' && c <= 'z') && !(c >= 'A' && c <= 'Z') && !(c >= '0' && c <= '9') && c != '_' && c != ':') {
 			c = '_';
 		}
 		*cur = c;
@@ -1264,6 +1223,7 @@ R_API char *r_str_sanitize_sdb_key(const char *s) {
 }
 
 R_API void r_str_byte_escape(const char *p, char **dst, int dot_nl, bool default_dot, bool esc_bslash) {
+	r_return_if_fail (p && dst);
 	char *q = *dst;
 	switch (*p) {
 	case '\n':
@@ -1436,7 +1396,7 @@ R_API char *r_str_escape_sh(const char *buf) {
 	char *q = new_buf;
 	while (*p) {
 		switch (*p) {
-#if __UNIX__
+#if R2__UNIX__
 		case '$':
 		case '`':
 #endif
@@ -1827,10 +1787,19 @@ R_API char *r_str_escape_utf8_for_json(const char *buf, int buf_size) {
 		} else { // ch_bytes == 0
 			// Outside JSON spec, but apparently no better
 			// alternative if need to reconstruct the original string
+#if 1
+			*q++ = '\\';
+			*q++ = 'u';
+			*q++ = '0';
+			*q++ = '0';
+			*q++ = "0123456789abcdef"[*p >> 4 & 0xf];
+			*q++ = "0123456789abcdef"[*p & 0xf];
+#else
 			*q++ = '\\';
 			*q++ = 'x';
 			*q++ = "0123456789abcdef"[*p >> 4 & 0xf];
 			*q++ = "0123456789abcdef"[*p & 0xf];
+#endif
 			ch_bytes = 1;
 		}
 		p += ch_bytes;
@@ -1839,7 +1808,7 @@ R_API char *r_str_escape_utf8_for_json(const char *buf, int buf_size) {
 	return new_buf;
 }
 
-// http://daviddeley.com/autohotkey/parameters/parameters.htm#WINCRULES
+// https://daviddeley.com/autohotkey/parameters/parameters.htm#WINCRULES
 // https://docs.microsoft.com/en-us/cpp/cpp/main-function-command-line-args?redirectedfrom=MSDN&view=vs-2019#parsing-c-command-line-arguments
 R_API char *r_str_format_msvc_argv(size_t argc, const char **argv) {
 	RStrBuf sb;
@@ -1942,18 +1911,61 @@ R_API size_t r_str_ansi_nlen(const char *str, size_t slen) {
 	return len; // len > 0 ? len: 1;
 }
 
+static size_t __str_ansi_sanitize_length(char const *str) {
+	const signed char str0 = (const signed char)str[0];
+	const signed char str1 = (const signed char)str[1];
+	size_t i = 0;
+	if (str0 == 0x1b || str0 == 0x07 || str0 == 0x05 || str0 == 0x7f) { // ESC, BEL, ENQ, DEL
+		i++;
+	} else if (str0 == -0x3e && str1 >= -0x80 && str1 <= -0x61) { // C1 control codes U+0080 - U+009F
+		i += 2;
+	}
+	return i;
+}
+
+// remove ansi escape codes from string, decolorizing it
+// TODO : optimize by just using two counter variables instead of strcpy()
+R_API size_t r_str_ansi_strip(char *str) {
+	size_t i = 0;
+	while (str[i]) {
+		size_t chlen = __str_ansi_length (str + i);
+		size_t sanitize_len = __str_ansi_sanitize_length (str + i);
+		if (chlen > 1) {
+			r_str_cpy (str + i, str + i + chlen);
+		} else if (sanitize_len > 0) {
+			r_str_cpy (str + i, str + i + sanitize_len);
+		} else {
+			i++;
+		}
+	}
+	return i;
+}
+
+#if 0
+// unused
+// insert a string into another string, supports ansi control chars
+R_API char *r_str_insert(R_OWN char *src, int pos, const char *str) {
+	r_str_var (a, pos, src);
+	char *b = strdup (src + pos + r_str_ansi_len (str));
+	char *r = r_str_newf ("%s%s%s", a, str, b);
+	free (b);
+	free (src);
+	return r;
+}
+#endif
+
 R_API size_t r_str_ansi_len(const char *str) {
+	r_return_val_if_fail (str, 0);
 	return r_str_ansi_nlen (str, 0);
 }
 
 R_API size_t r_str_nlen(const char *str, int n) {
+	r_return_val_if_fail (str && n >= 0, 0);
 	size_t len = 0;
-	if (str) {
-		while (n > 0 && *str) {
-			len++;
-			str++;
-			n--;
-		}
+	while (n > 0 && *str) {
+		len++;
+		str++;
+		n--;
 	}
 	return len;
 }
@@ -1967,8 +1979,8 @@ R_API size_t r_str_nlen_w(const char *str, int n) {
 			len++;
 			str++;
 			if (!*str) {
-				//handle wide strings
-			 	//xx00yy00bb00
+				// handle wide strings
+				// xx00yy00bb00
 				if (n - 2 > 0) {
 					if (str[2]) {
 						break;
@@ -2008,19 +2020,23 @@ R_API bool r_str_is_printable(const char *str) {
 }
 
 R_API bool r_str_is_printable_limited(const char *str, int size) {
+	int left = size;
+
 	while (size > 0 && *str) {
-		int ulen = r_utf8_decode ((const ut8*)str, strlen (str), NULL);
+		int ulen = r_utf8_decode ((const ut8*)str, left, NULL);
 		if (ulen > 1) {
 			str += ulen;
 			continue;
 		}
 		if (!IS_PRINTABLE (*str)) {
-			return false;
+			break;
 		}
 		str++;
 		size--;
+		left -= ulen;
 	}
-	return true;
+
+	return size == 0;
 }
 
 R_API bool r_str_is_printable_incl_newlines(const char *str) {
@@ -2040,7 +2056,7 @@ R_API bool r_str_is_printable_incl_newlines(const char *str) {
 	return true;
 }
 
-// Length in chars of a wide string (find better name?)
+// Length in chars of a wide string and find a better name
 R_API size_t r_wstr_clen(const char *s) {
 	size_t len = 0;
 	if (!*s++) {
@@ -2073,6 +2089,38 @@ R_API const char *r_str_ansi_chrn(const char *str, size_t n) {
 	return str + li;
 }
 
+#if 0
+// modify string in place removing escape chars that modify the background color
+R_API void r_str_ansi_filterbg(char *str) {
+	char *d = str;
+	while (*str) {
+		if (*str == 0x1b && str[1] == '[') {
+			if (str[2] == '4') { // \x1b[4#m
+				str += 4;
+			} else if (str[2] == '1' && str[3] == '0') { // \x1b[104#m
+				str += 5;
+			} else if (str[2] == '4' && str[3] == '8' && str[4] == ';') { // 256 bgcolor
+				char *end = strchr (str + 4, 'm');
+				if (end) {
+					str = end;
+				}
+			}
+		}
+		str++;
+	}
+}
+#endif
+
+// replace \x1b[0m with \x[0m + bgcolor
+R_API char *r_str_ansi_resetbg(const char *str, const char *bgcolor) {
+	char *res = strdup (str);
+	char * resetbg = r_str_newf ("\x1b[0m%s", bgcolor);
+	res = r_str_replace_all (res, "\x1b[0m", "(ºvº)");
+	res = r_str_replace_all (res, "(ºvº)", resetbg);
+	free (resetbg);
+	return res;
+}
+
 /*
  * filter out ansi CSI shit in-place!.
  * str - input string,
@@ -2224,7 +2272,7 @@ R_API char *r_str_ansi_crop(const char *str, ut32 x, ut32 y, ut32 x2, ut32 y2) {
 }
 
 R_API size_t r_str_utf8_codepoint(const char* s, size_t left) {
-	if (!s || left <= 0) {
+	if (!s || !*s || left <= 0) {
 		return 0;
 	}
 	if ((*s & 0x80) != 0x80) {
@@ -2240,15 +2288,13 @@ R_API size_t r_str_utf8_codepoint(const char* s, size_t left) {
 }
 
 R_API bool r_str_char_fullwidth(const char* s, size_t left) {
-	if (!s || left <= 0) {
+	if (!s || !*s || left <= 0) {
 		return false;
 	}
 	size_t codepoint = r_str_utf8_codepoint (s, left);
-	return (codepoint >= 0x1100 &&
-		 (codepoint <= 0x115f ||                  /* Hangul Jamo init. consonants */
-			  codepoint == 0x2329 || codepoint == 0x232a ||
-		 (R_BETWEEN (0x2e80, codepoint, 0xa4cf)
-			&& codepoint != 0x303f) ||        /* CJK ... Yi */
+	if (codepoint >= 0x1100) {
+		 return codepoint <= 0x115f || codepoint == 0x2329 || codepoint == 0x232a || /* Hangul Jamo init. consonants */
+		 (R_BETWEEN (0x2e80, codepoint, 0xa4cf) && codepoint != 0x303f) || /* CJK ... Yi */
 		 R_BETWEEN (0xac00, codepoint, 0xd7a3) || /* Hangul Syllables */
 		 R_BETWEEN (0xf900, codepoint, 0xfaff) || /* CJK Compatibility Ideographs */
 		 R_BETWEEN (0xfe10, codepoint, 0xfe19) || /* Vertical forms */
@@ -2256,7 +2302,9 @@ R_API bool r_str_char_fullwidth(const char* s, size_t left) {
 		 R_BETWEEN (0xff00, codepoint, 0xff60) || /* Fullwidth Forms */
 		 R_BETWEEN (0xffe0, codepoint, 0xffe6) ||
 		 R_BETWEEN (0x20000, codepoint, 0x2fffd) ||
-		 R_BETWEEN (0x30000, codepoint, 0x3fffd)));
+		 R_BETWEEN (0x30000, codepoint, 0x3fffd);
+	}
+	return false;
 }
 
 /**
@@ -2338,7 +2386,8 @@ R_API bool r_str_glob(const char* str, const char *glob) {
 	}
 	char* begin = strchr (glob, '^');
 	if (begin) {
-		glob = ++begin;
+		begin++;
+		glob = begin;
 	}
 	while (*str) {
 		if (!*glob) {
@@ -2346,12 +2395,14 @@ R_API bool r_str_glob(const char* str, const char *glob) {
 		}
 		switch (*glob) {
 		case '*':
-			if (!*++glob) {
+			glob++;
+			if (!*glob) {
 				return true;
 			}
 			// Advance glob an additional time if it is a '**'
 			if (*glob == '*') {
-				if (!*++glob) {
+				glob++;
+				if (!*glob) {
 					return true;
 				}
 			}
@@ -2392,7 +2443,9 @@ R_API bool r_str_glob(const char* str, const char *glob) {
 			glob++;
 		}
 	}
-	while (*glob == '*') { ++glob; }
+	while (*glob == '*') {
+		glob++;
+	}
 	return ((*glob == '$' && !*glob++)  || !*glob);
 }
 
@@ -2723,7 +2776,7 @@ R_API const char *r_str_lastbut(const char *s, char ch, const char *but) {
 // Must be merged inside strlen
 R_API size_t r_str_len_utf8char(const char *s, int left) {
 	size_t i = 1;
-	while (s[i] && (!left || i<left)) {
+	while (s[i] && (!left || i < left)) {
 		if ((s[i] & 0xc0) != 0x80) {
 			i++;
 		} else {
@@ -2770,6 +2823,7 @@ R_API size_t r_str_len_utf8_ansi(const char *str) {
 
 // XXX must find across the ansi tags, as well as support utf8
 R_API const char *r_strstr_ansi(const char *a, const char *b) {
+	// XXX not working well if "\x1b[32mF\x1b[32mOO" -> FOO not found
 	const char *ch, *p = a;
 	do {
 		ch = strchr (p, '\x1b');
@@ -2905,9 +2959,7 @@ R_API char *r_str_uri_encode(const char *s) {
 		return NULL;
 	}
 	for (; *s; s++) {
-		if((*s>='0' && *s<='9')
-		|| (*s>='a' && *s<='z')
-		|| (*s>='A' && *s<='Z')) {
+		if ((*s >= '0' && *s <= '9') || (*s >= 'a' && *s <= 'z') || (*s >= 'A' && *s <= 'Z')) {
 			*d++ = *s;
 		} else {
 			*d++ = '%';
@@ -3239,15 +3291,25 @@ R_API char *r_str_crop(const char *str, unsigned int x, unsigned int y,
 
 // TODO: improve loop to wrap by words
 R_API char *r_str_wrap(const char *str, int w) {
-	char *r, *ret;
 	if (w < 1 || !str) {
 		return strdup ("");
 	}
 	size_t r_size = 8 * strlen (str);
-	r = ret = malloc (r_size);
+	char *r = malloc (r_size);
+	if (!r) {
+		return NULL;
+	}
+	char *ret = r;
 	char *end = r + r_size;
 	int cw = 0;
 	while (*str && r + 1 < end) {
+		size_t ansilen = __str_ansi_length (str);
+		if (ansilen > 1) {
+			memcpy (r, str, ansilen);
+			str += ansilen;
+			r += ansilen;
+			continue;
+		}
 		if (*str == '\t') {
 			// skip
 		} else if (*str == '\r') {
@@ -3270,26 +3332,6 @@ R_API char *r_str_wrap(const char *str, int w) {
 	return ret;
 }
 
-R_API const char *r_str_tok(const char *str1, const char b, size_t len) {
-	const char *p = str1;
-	size_t i = 0;
-	if (!p || !*p) {
-		return p;
-	}
-	if (len == -1) {
-		len = strlen (str1);
-	}
-	for ( ; i < len; i++,p++) {
-		if (*p == b) {
-			break;
-		}
-	}
-	if (i == len) {
-		p = NULL;
-	}
-	return p;
-}
-
 R_API int r_str_do_until_token(str_operation op, char *str, const char tok) {
 	int ret;
 	if (!str) {
@@ -3363,13 +3405,46 @@ R_API bool r_str_endswith(const char *str, const char *needle) {
 	return !strcmp (str + (slen - nlen), needle);
 }
 
+R_API char *r_str_slice(const char *str, RStringSlice s) {
+	size_t len = s.to - s.from;
+	return r_str_ndup (str + s.from, len);
+}
+
+R_API RVecStringSlice *r_str_split_vec(const char *str, const char *c, int n) {
+	r_return_val_if_fail (str && c, NULL);
+	size_t b = 0;
+	// TODO honor 'c' and 'n'. not just split by ' '
+	RVecStringSlice *vs = RVecStringSlice_new ();
+	while (str[b]) {
+		const char ch = str[b];
+		if (isspace (ch)) {
+			b++;
+			continue;
+		}
+		RStringSlice slice = { b, 0 };
+		b++;
+		while (str[b]) {
+			const char ch = str[b];
+			if (isspace (ch)) {
+				break;
+			}
+			b++;
+		}
+		slice.to = b;
+		RVecStringSlice_push_back (vs, &slice);
+	}
+	return vs;
+}
+
 // Splits the string <str> by string <c> and returns the result in a list.
+// XXX should take const char * as argument!!
 R_API RList *r_str_split_list(char *str, const char *c, int n)  {
 	r_return_val_if_fail (str && c, NULL);
 	RList *lst = r_list_newf (NULL);
-	char *aux = str;
+	char *aux = str; // XXX should be an strdup
 	int i = 0;
-	char  *e = aux;
+	char *e = aux;
+	const size_t clen = strlen (c);
 	for (;e;) {
 		e = strstr (aux, c);
 		if (n > 0) {
@@ -3379,7 +3454,8 @@ R_API RList *r_str_split_list(char *str, const char *c, int n)  {
 			}
 		}
 		if (e) {
-			*e++ =  0;
+			*e = 0;
+			e += clen;
 		}
 		r_str_trim (aux);
 		r_list_append (lst, aux);
@@ -3446,8 +3522,8 @@ R_API bool r_str_isnumber(const char *str) {
 	if (!str || (!IS_DIGIT (*str) && *str != '-')) {
 		return false;
 	}
-
-	while (*++str) {
+	for (str++; *str; str++) {
+		// consider '.' part of the number?
 		if (!IS_DIGIT (*str)) {
 			return false;
 		}
@@ -3696,7 +3772,17 @@ R_API char *r_str_from_ut64(ut64 val) {
 		i++;
 	}
 	while (i < 8 && *v) {
+#if 1
 		str[i++] = *v++;
+#else
+		// WIP: experimental behaviour
+		if (IS_PRINTABLE (*v)) {
+			str[i++] = *v;
+		} else {
+//			break;
+		}
+		v++;
+#endif
 	}
 	return str;
 }
@@ -3717,17 +3803,15 @@ R_API int r_snprintf(char *string, int len, const char *fmt, ...) {
 
 // Strips all the lines in str that contain key
 R_API void r_str_stripLine(char *str, const char *key) {
-	size_t i, j, klen, slen, off;
-	const char *ptr;
-
 	if (!str || !key) {
 		return;
 	}
-	klen = strlen (key);
-	slen = strlen (str);
+	size_t i, j;
+	size_t klen = strlen (key);
+	size_t slen = strlen (str);
 
 	for (i = 0; i < slen; ) {
-		ptr = (char*) r_mem_mem ((ut8*) str + i, slen - i, (ut8*) "\n", 1);
+		const char *ptr = (char*) r_mem_mem ((ut8*) str + i, slen - i, (ut8*) "\n", 1);
 		if (!ptr) {
 			ptr = (char*) r_mem_mem ((ut8*) str + i, slen - i, (ut8*) key, klen);
 			if (ptr) {
@@ -3737,7 +3821,7 @@ R_API void r_str_stripLine(char *str, const char *key) {
 			break;
 		}
 
-		off = (size_t) (ptr - (str + i)) + 1;
+		size_t off = (size_t) (ptr - (str + i)) + 1;
 
 		ptr = (char*) r_mem_mem ((ut8*) str + i, off, (ut8*) key, klen);
 		if (ptr) {
@@ -3753,13 +3837,13 @@ R_API void r_str_stripLine(char *str, const char *key) {
 
 R_API char *r_str_list_join(RList *str, const char *sep) {
 	r_return_val_if_fail (str && sep, NULL);
+	RListIter *iter;
 	RStrBuf *sb = r_strbuf_new ("");
-	const char *p;
-	while ((p = r_list_pop_head (str))) {
+	r_list_foreach_iter (str, iter) {
 		if (r_strbuf_length (sb) != 0) {
 			r_strbuf_append (sb, sep);
 		}
-		r_strbuf_append (sb, p);
+		r_strbuf_append (sb, iter->data);
 	}
 	return r_strbuf_drain (sb);
 }
@@ -3767,11 +3851,9 @@ R_API char *r_str_list_join(RList *str, const char *sep) {
 R_API char *r_str_array_join(const char **a, size_t n, const char *sep) {
 	RStrBuf *sb = r_strbuf_new ("");
 	size_t i;
-
 	if (n > 0) {
 		r_strbuf_append (sb, a[0]);
 	}
-
 	for (i = 1; i < n; i++) {
 		r_strbuf_append (sb, sep);
 		r_strbuf_append (sb, a[i]);
@@ -3799,7 +3881,7 @@ R_API int r_str_fmtargs(const char *fmt) {
 // Returns "true" or "false" as a string given an input integer. The returned
 // value is consistent with C's definition of 0 is false, and all other values
 // are true.
-R_API const char *r_str_bool(int b) {
+R_API const char *r_str_bool(bool b) {
 	return b? "true": "false";
 }
 
@@ -3882,7 +3964,7 @@ R_API int r_str_distance(const char *a, const char *b) {
 	ut32 distance = 0;
 	double similarity = 0;
 	r_diff_buffers_distance_levenshtein (NULL, (const ut8*)a, strlen (a), (const ut8*)b, strlen (b), &distance, &similarity);
-	return (int)(similarity * 100);;
+	return (int)(similarity * 100);
 }
 
 R_API const char *r_str_str_xy(const char *s, const char *word, const char *prev, int *x, int *y) {
@@ -3924,15 +4006,42 @@ R_API const char *r_str_str_xy(const char *s, const char *word, const char *prev
 #endif
 
 R_API char *r_str_version(const char *program) {
+	const char *release = "";
+	const char *asanstr = "";
+	const char *gplstr = "lgpl";
+	char optistr[5] = " -O?";
+#if defined(__has_feature)
+#if __has_feature(address_sanitizer)
+	asanstr = " asan";
+#endif
+#endif
+#if R2_VERSION_COMMIT == 0
+	release = " release";
+#endif
+#ifdef _FORTIFY_SOURCE
+	// clang
+	optistr[3] = '0' + _FORTIFY_SOURCE;
+#endif
+#ifdef __OPTIMIZE__
+	// gcc
+	optistr[3] = '0' + __OPTIMIZE__;
+#endif
+#if WITH_GPL
+	gplstr = " gpl";
+#endif
 	char *s = r_str_newf ("%s "R2_VERSION" %d @ "
 			R_SYS_OS"-"
-			R_SYS_ARCH"-%d git.%s\n",
+			R_SYS_ARCH"-%d\n",
 			program, R2_VERSION_COMMIT,
-			(R_SYS_BITS & 8)? 64: 32,
+			(R_SYS_BITS & 8)? 64: 32);
+	int csv = R2_CSVERSION;
+	s = r_str_appendf (s, "birth: git.%s "R2_BIRTH"\n",
 			*R2_GITTAP ? R2_GITTAP: "");
 	if (*R2_GITTIP) {
-		s = r_str_appendf (s, "commit: "R2_GITTIP" build: "R2_BIRTH);
+		s = r_str_append (s, "commit: "R2_GITTIP"\n");
 	}
+	s = r_str_appendf (s, "options:%s%s%s%s cs:%d cl:%d %s",
+			gplstr, asanstr, release, optistr, csv, R_CHECKS_LEVEL, R_BUILDSYSTEM);
 	return s;
 }
 
@@ -3974,3 +4083,58 @@ R_API bool r_str_startswith(const char *str, const char *needle) {
 	return !strncmp (str, needle, strlen (needle));
 }
 
+R_API void r_str_fixspaces(char *str) {
+	// add space after commas
+	char *os = strdup (str);
+	int i, j;
+	for (i = j = 0; os[i]; i++,j++) {
+		char ch = os[i];
+		str[j] = ch;
+		if (ch == ',') {
+			j++;
+			str[j] = ' ';
+			while (os[i + 1] == ' ') {
+				i++;
+			}
+		}
+	}
+	str[j] = 0;
+	free (os);
+	r_str_trim_tail (str);
+}
+
+R_API char *r_str_tok_r(char *str, const char *delim, char **save_ptr) {
+	char *ret = NULL;
+	if (str == NULL) {
+		str = *save_ptr;
+	}
+	str += strspn (str, delim);
+	if (*str == '\0') {
+		return NULL;
+	}
+	ret = str;
+	str += strcspn (str, delim);
+	if (*str) {
+		*str++ = '\0';
+	}
+	*save_ptr = str;
+	return ret;
+}
+
+R_API char *r_str_tok_next(char *s) {
+	if (s) {
+		return s + strlen (s) + 1;
+	}
+	return NULL;
+}
+
+R_API R_MUSTUSE char *r_str_after(char *s, char c) {
+	if (s) {
+		char *p = strchr (s, c);
+		if (p) {
+			*p++ = 0;
+			return p;
+		}
+	}
+	return NULL;
+}
