@@ -1,14 +1,48 @@
 #!/bin/sh
 
-if [ "$(id -u)" = 0 ]; then
-	echo "[WW] Do not run this script as root!"
-	if [ -n "${SUDO_USER}" ]; then
-		echo "[--] Downgrading credentials to ${SUDO_USER}"
-		exec sudo -u "${SUDO_USER}" sys/install.sh $*
+# find
+cd "$(dirname $0)"/..
+pwd
+export PAGER=cat
+unset LINK
+
+if [ "${SHELL}" = "/data/data/com.termux/files/usr/bin/bash" ]; then
+    echo "Termux environment detected. Installing necessary packages"  
+    pkg update -y && pkg install git build-essential binutils pkg-config -y
+    ${PWD}/sys/termux.sh
+    exit $?
+fi
+
+if [ "$(uname)" = "Haiku" ]; then
+	gcc-x86 --version > /dev/null 2>&1
+	if [ $? = 0 ]; then
+		export CC=gcc-x86
+		export HOST_CC=gcc-x86
+		export USERCC=gcc-x86
+	else
+		echo "If compilation fails, install gcc-x86 from depot"
+	fi
+	export PREFIX="${PWD}/prefix"
+
+else
+	if [ "$(id -u)" = 0 ]; then
+		echo "[WW] Do not run this script as root!"
+		if [ -n "${SUDO_USER}" ]; then
+			echo "[--] Downgrading credentials to ${SUDO_USER}"
+			exec sudo -u "${SUDO_USER}" sys/install.sh $*
+		fi
 	fi
 fi
 
+echo "$PWD" | grep -q " "
+if [ $? = 0 ]; then
+	echo "You can't build radare from a directory with spaces with make" >&2
+	echo "To solve this you must 'meson' instead" >&2
+	exit 1
+fi
+
 export USE_CS4=0
+export USE_CSNEXT=0
 # if owner of sys/install.sh != uid && uid == 0 { exec sudo -u id -A $SUDO_UID sys/install.sh $* }
 ARGS=""
 while : ; do
@@ -31,6 +65,9 @@ while : ; do
 	"--without-pull")
 		export WITHOUT_PULL=1
 		;;
+	'--prefix='*)
+		PREFIX=`echo "$1" | cut -d = -f 2`
+		;;
 	-*)
 		# just for the penguin face case
 		ARGS="${ARGS} $1"
@@ -43,26 +80,24 @@ while : ; do
 	shift
 done
 
-MAKE=make
-gmake --help >/dev/null 2>&1
-[ $? = 0 ] && MAKE=gmake
-${MAKE} --help 2>&1 | grep -q gnu
-if [ $? != 0 ]; then
-	echo "You need GNU Make to build me"
-	exit 1
+if [ -z "${MAKE}" ]; then
+	MAKE=make
+	gmake --help >/dev/null 2>&1
+	[ $? = 0 ] && MAKE=gmake
+	${MAKE} --help 2>&1 | grep -q gnu
+	if [ $? != 0 ]; then
+		echo "You need GNU Make to build me"
+		exit 1
+	fi
+	export MAKE="$MAKE"
 fi
 
-export MAKE="$MAKE"
-
 [ -z "${INSTALL_TARGET}" ] && INSTALL_TARGET=symstall
 
-# find
-cd "$(dirname $0)"/..
-pwd
-
 # update
 if [ -z "$WITHOUT_PULL" ]; then
-	if [ -d .git ]; then
+	# if .git is a directory, that's a clone, if it's a file it's a submodule
+	if [ -e .git ]; then
 		git branch | grep "^\* master" > /dev/null
 		if [ $? = 0 ]; then
 			echo "WARNING: Updating from remote repository"
@@ -95,7 +130,12 @@ fi
 if [ "$(id -u)" = 0 ]; then
 	SUDO=""
 else
-	[ -n "${NOSUDO}" ] && SUDO="echo NOTE: sudo not found. Please run as root: "
+	if [ -d /system/bin ]; then
+		# This is an android
+		SUDO=""
+	else
+		[ -n "${NOSUDO}" ] && SUDO="echo NOTE: sudo not found. Please run as root: "
+	fi
 fi
 
 if [ "${USE_SU}" = 1 ]; then
@@ -114,6 +154,16 @@ else
 	fi
 fi
 
+# handle home installations of capstone via r2pm
+export PKG_CONFIG_PATH="${HOME}/.local/share/radare2/prefix/lib/pkgconfig/"
+if [ -d "$HOME/.local/share/radare2/prefix/include/capstone" ]; then
+	# capstone's pkg-config is wrong :_____
+	# export CFLAGS="$(pkg-config --cflags capstone)"
+	# export LDFLAGS="$(pkg-config --libs capstone)"
+	export CFLAGS="-I$HOME/.local/share/radare2/prefix/include"
+	export LDFLAGS="-L$HOME/.local/share/radare2/prefix/lib"
+fi
+
 NEED_CAPSTONE=1
 pkg-config --cflags capstone > /dev/null 2>&1
 if [ $? = 0 ]; then
@@ -135,10 +185,20 @@ else
 fi
 
 if [ "$NEED_CAPSTONE" = 1 ]; then
+	if [ ! -d shlr/capstone/.git -a ! -d .git ]; then
+		NEED_CAPSTONE=0
+	fi
+fi
+
+if [ "$NEED_CAPSTONE" = 1 ]; then
+	if [ -d shlr/capstone ]; then
+		${MAKE} -C shlr headsup 2> /dev/null || rm -rf shlr/capstone
+	fi
 	if [ ! -d shlr/capstone ]; then
 		./preconfigure
 	fi
 fi
+echo "ARGS=$ARGS"
 
 if [ "${M32}" = 1 ]; then
 	${SHELL} ./sys/build-m32.sh ${ARGS} || exit 1
